/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

/*
            var request = new XMLHttpRequest();            
            var url = URL;
            request.open('get', url, true);
            request.onreadystatechange = function() {
                if (request.readyState == 4) {
                    if (request.status == 200) {
                        var xml = request.responseText;
                        var feed = new FTFeed(url);
                        feed.parse(xml, function(bSuccess) { 
                            bSuccess ? dump(feed) : dump('error');
                        });
                    }
                }
           }
           request.send(null);
*/

function FTFeedItem(title, location, date, text, enclosure) {
    title = title.replace(/[\n\r]/g, '');
    this.title = title;
    this.location = foxytunesSanitizeURL(location);
    this.date = date;
    this.text = text;
    this.enclosure = foxytunesSanitizeURL(enclosure);
}

FTFeedItem.prototype.toString = function() {
    var str = '';
    str += 'title: ' + this.title + '\n';
    str += 'location: ' + this.location + '\n';
    str += 'date: ' + this.date + '\n';
    str += 'text: ' + this.text + '\n';
    str += 'enclosure: ' + this.enclosure + '\n';
    return str;
}

function FTFeed(feedURL) {
    this.feedURL = feedURL;
    this.title = null;
    this.location = null;
    this.description = null;
    this.items = [];
    this.isPodcast = false;
}

FTFeed.prototype.toString = function() {
    var str = '';
    str += 'title: ' + this.title + '\n';
    str += 'feedURL: ' + this.feedURL + '\n';
    str += 'location: ' + this.location + '\n';
    str += 'description: ' + this.description + '\n';
    for (var i=0; i < this.items.length; i++) {
        str += '------------\n';
        str += this.items[i].toString();
    }
    return str;
}

FTFeed.prototype.fixup = function() {
    this.location = foxytunesSanitizeURL(this.location);
},

FTFeed.prototype.removeHTML = function(str) {
    str = str.replace(/<[^>]+>/g, ' ');
    str = str.replace(/ +/g, ' ');
    str = str.replace(/^ */, '');
    str = str.replace(/ *^/, '');
    return str;
}

FTFeed.prototype.handleResult = function(result) {
    try {
      var feed = result.doc;
      feed.QueryInterface(Components.interfaces.nsIFeed);
      this.title = feed.title.text;
      this.location = feed.link.resolve("");
      this.description = feed.fields.get("description");
      var itemArray = feed.items;
      var numItems = itemArray.length;

      for (var i=0; i<numItems; i++) {
        var item = itemArray.queryElementAt(i, Components.interfaces.nsIFeedEntry);
        var title = item.title.text;
        var link = item.link.resolve("");
        var description = item.fields.get("description");
        var pubDate = item.updated;
        var enclosureURL="";
        var enclosureArray = item.enclosures;
        // Only taking the first enclosure (so if the entry have 4 mp3's we'll onyl show/play the first one)
        if (enclosureArray && enclosureArray.length > 0) {
            var firstEnclosure = enclosureArray.queryElementAt(0, Components.interfaces.nsIWritablePropertyBag2);
            enclosureURL = firstEnclosure.get("url");
            // Assuming that if an entry has enclosureURL than it's a podcast
        if (enclosureURL) {
            this.isPodcast = true;
        }
        }
        var feedItem = new FTFeedItem(title, link, pubDate, description, enclosureURL);
        this.items.push(feedItem);
    }

      this.fixup();
      this.callback(true);         
    } catch(e) {
      this.callback(false);
    }
}

FTFeed.prototype.parse = function(xml, callback) {
    this.callback = callback;
    var ioService = Components.classes['@mozilla.org/network/io-service;1']
            .getService(Components.interfaces.nsIIOService);
    var uri = ioService.newURI(this.feedURL, null, null);
    var parser = Components.classes["@mozilla.org/feed-processor;1"]
            .createInstance(Components.interfaces.nsIFeedProcessor);
    try {
        parser.listener = this;
        parser.parseFromString(xml, uri);
    } catch(e) {
        callback(false);
    };
}
