/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FoxyTunes Mozilla Extension and Engine.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function FoxyTunesKeyUINames() {
}

FoxyTunesKeyUINames.prototype = {
	GetString : function (bundle, key, defaultValue) {
		var retValue = defaultValue;
		try {
			retValue = bundle.getString(key);
		} catch(e) { }
		return retValue;
	},

	InitCodeToVKStrings : function () {
		this.keyCodeToVKString = new Object();
		for (code in KeyEvent) {
			var keyCode = KeyEvent[code];
			var VKName = code.substring(4);
			this.keyCodeToVKString[keyCode] = VKName;	
		}

		this.keyCodeToVKString[KeyEvent.DOM_VK_BACK_SPACE] = "VK_BACK";	
	},

	Init : function () {
		this.InitCodeToVKStrings();

		this.keyNamesStrings = document.getElementById('foxytunes-keynames-strings');
		this.modifierNamesStrings = document.getElementById('foxytunes-modifiernames-strings');

		this.shiftName 		= this.GetString(this.modifierNamesStrings, 'VK_SHIFT', 'shift');
		this.metaName  		= this.GetString(this.modifierNamesStrings, 'VK_META', 'meta');
		this.altName   		= this.GetString(this.modifierNamesStrings, 'VK_ALT', 'alt');
		this.controlName 	= this.GetString(this.modifierNamesStrings, 'VK_CONTROL', 'Ctrl');
		this.separatorName 	= this.GetString(this.modifierNamesStrings, 'MODIFIER_SEPARATOR', '+');

		this.modifierNames = new Object();
		this.modifierNames['shift'] = this.shiftName;
		this.modifierNames['meta'] = this.metaName;
		this.modifierNames['alt'] = this.altName;
		this.modifierNames['control'] = this.controlName;
		
		var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                		            .getService(Components.interfaces.nsIPrefService).getBranch(null);
	  	switch (prefService.getIntPref("ui.key.accelKey")) {
		    case 17:  this.modifierNames['accel'] = this.controlName; 	break;
		    case 18:  this.modifierNames['accel'] = this.altName; 	break;
		    case 224: this.modifierNames['accel'] = this.metaName; 	break;
		    default:  this.modifierNames['accel'] = (navigator.platform.toLowerCase().indexOf('mac') != -1) ? 										this.metaName : this.controlName;
  		}
	},

	GetKeyName : function (keycode) {	
		return this.GetString(this.keyNamesStrings, keycode, keycode);
	},

	GetModifiersName : function (modifiers) {
		if (!modifiers) {
			 return '';
		}

		var modifiersUIName = new Array();
		var modArray = modifiers.split(",");
		for (var i = 0; i < modArray.length; i++) {
			var modifier = foxytinesTrimString(modArray[i]);
			var modifierName = this.modifierNames[modifier];
			modifiersUIName.push(modifierName);
		}
		return modifiersUIName.join(this.separatorName);
	},

	GetVKName : function (keycode) {
		var vkName = this.keyCodeToVKString[keycode];
		if (typeof(vkName) != "undefined") {
			return vkName;
		} else {
			return foxytunesGetLocaleString('unknown');
		}
	},

	GetPrettyLabel : function(modifiers, key, keycode) {
		var labelParts = new Array();

		if (modifiers) {
			labelParts.push(this.GetModifiersName(modifiers));
		} 
		if (keycode) {
			labelParts.push(this.GetKeyName(keycode));
		} 
		if (key) {
			labelParts.push(key);
		}

		var label = labelParts.join(this.separatorName);
		if (label == this.separatorName) {
			label = '';
		}
		return label;
 	}	
}

var gFoxyTunesKeyUINames = new FoxyTunesKeyUINames();

function foxytunesGetSensorEntryElem() {
	return document.getElementById('foxytunes-key-sensor');

}

function foxytunesShortcutsDialogReadOnly() {
	return 	((window.arguments.length > 1) && (window.arguments[1] == 'true'));
}

function foxytunesShortcutsCreateContents() {
	var allKeys = window.arguments[0];
	var contents = document.getElementById('foxytunes-keys-all-keys');

	for (var i = 0; i < allKeys.length; i++) {
		var key = allKeys[i];

		var newItem = document.createElement('treeitem');
		contents.appendChild(newItem);
		
		var newRow = document.createElement('treerow');
		newItem.appendChild(newRow);

		var labelCell = document.createElement('treecell');
		labelCell.setAttribute('label', key.getAttribute('label'));
		newRow.appendChild(labelCell);

		var keyCell = document.createElement('treecell');

		var modifiers 	= key.getAttribute('modifiers');
		var keyAttr 	= key.getAttribute('key');
		var keyCodeAttr = key.getAttribute('keycode');

		var label = gFoxyTunesKeyUINames.GetPrettyLabel(modifiers, keyAttr, keyCodeAttr);

		keyCell.setAttribute('label', label);

		newRow.keydata = [modifiers, keyAttr, keyCodeAttr];

		var keyEnabled = key.getAttribute('keyenabled');
		newRow.setAttribute('keyenabled', keyEnabled);
		if (keyEnabled == 'false') {
			labelCell.setAttribute('properties', 'disabled');
			keyCell.setAttribute('properties', 'disabled');
		}

		newRow.appendChild(keyCell);

	 	//dump("\n\nLabel: " + key.getAttribute('label') + "\n");	
	 	//dump("ID: " + key.id + "\n");	
	 	//dump("key: " + key.getAttribute('key') + "\n");	
	 	//dump("keycode: " + key.getAttribute('keycode') + "\n");	
	 	//dump("modifiers: " + key.getAttribute('modifiers') + "\n");	
	}
}

function foxytunesShortcutGetSelectedRow(tree) {
	var selectedIndex = tree.view.selection.currentIndex;
	if (selectedIndex == -1) {
		return null;
	}
	var selectedRow = tree.view.getItemAtIndex(selectedIndex).firstChild;
	return selectedRow;
}

function foxytunesShortcutDialogOnSelect(event) {
	if (foxytunesShortcutsDialogReadOnly()) {
		return;
	}

	var tree = event.target;
	var selectedRow = foxytunesShortcutGetSelectedRow(tree);
	if (selectedRow == null) {
		return;
	}

	var entryElem = foxytunesGetSensorEntryElem();
	entryElem.disabled = false;
	entryElem.value = selectedRow.childNodes[1].getAttribute('label');
	entryElem.keydata = [selectedRow.keydata[0], selectedRow.keydata[1], selectedRow.keydata[2]];

	// setting the focus using a timeout, 
	// because the focus is set on the tree -after- the onselect handler 
	setTimeout('foxytunesGetSensorEntryElem().focus()', 0);

	document.getElementById('foxytunes-shortcut-apply-button').setAttribute('disabled', 'false');
	document.getElementById('foxytunes-shortcut-reset-button').setAttribute('disabled', 'false');
	
	var disableButton = document.getElementById('foxytunes-shortcut-disable-button');
	disableButton.setAttribute('disabled', 'false');
	if (selectedRow.getAttribute('keyenabled') == 'true') {
			disableButton.setAttribute('label', foxytunesGetLocaleString('disable'));
	} else {
			disableButton.setAttribute('label', foxytunesGetLocaleString('enable'));
	}
}

function foxytunesShortcutDialogOnEnableDisable(event) {
	var tree = document.getElementById('foxytunes-keys-all-keys-tree');
	var selectedRow = foxytunesShortcutGetSelectedRow(tree);
	if (selectedRow == null) {
		return;
	}

	var selectedIndex = tree.view.selection.currentIndex;

	if (selectedRow.getAttribute('keyenabled') == 'true') {
		selectedRow.setAttribute('keyenabled', 'false');
		selectedRow.childNodes[0].setAttribute('properties', 'disabled');
		selectedRow.childNodes[1].setAttribute('properties', 'disabled');
		event.target.setAttribute('label', foxytunesGetLocaleString('enable'));
	} else {
		selectedRow.setAttribute('keyenabled', 'true');
		selectedRow.childNodes[0].removeAttribute('properties');
		selectedRow.childNodes[1].removeAttribute('properties');
		event.target.setAttribute('label', foxytunesGetLocaleString('disable'));
	}

	// apply the key change:
	foxytunesShortcutDialogOnApply();
}

function foxytunesShortcutDialogApplyKeyEnableDisable(keyElem) {
	if (keyElem.getAttribute('keyenabled') == 'false') {
		if (keyElem.hasAttribute('command')) {
			keyElem.setAttribute('origcommand', keyElem.getAttribute('command'));
			keyElem.removeAttribute('command');
		}

		if (keyElem.hasAttribute('oncommand')) {
			keyElem.setAttribute('origoncommand', keyElem.getAttribute('oncommand'));
			keyElem.removeAttribute('oncommand');
		}
	} else {
		if (keyElem.hasAttribute('origcommand')) {
			keyElem.setAttribute('command', keyElem.getAttribute('origcommand'));
		}

		if (keyElem.hasAttribute('origoncommand')) {
			keyElem.setAttribute('oncommand', keyElem.getAttribute('origoncommand'));
		}
	}
}

function foxytunesShortcutDialogOnApply() {
	// no apply in view-only mode
	if (foxytunesShortcutsDialogReadOnly()) {
		return;
	}

	var tree = document.getElementById('foxytunes-keys-all-keys-tree');
	var selectedRow = foxytunesShortcutGetSelectedRow(tree);
	if (selectedRow == null) {
		return;
	}

	var selectedIndex = tree.view.selection.currentIndex;

	var keydata = foxytunesGetSensorEntryElem().keydata;

    if (!keydata) return;

	var modifiers 	= keydata[0];
	var key 	= keydata[1];
	var keycode 	= keydata[2];

	selectedRow.keydata = [modifiers, key, keycode];

	selectedRow.childNodes[1].setAttribute('label', foxytunesGetSensorEntryElem().value);

	var keyElem = selectedRow.childNodes[0].getAttribute('value');

	var allKeys = window.arguments[0];
	var keyElem = allKeys[selectedIndex];

	keyElem.removeAttribute('modifiers');
	keyElem.removeAttribute('key');
	keyElem.removeAttribute('keycode');

	if (modifiers) {
		keyElem.setAttribute('modifiers', modifiers);
	} 
	if (key) {
		keyElem.setAttribute('key', key);
	}
	if (keycode) {
		keyElem.setAttribute('keycode', keycode);
	}

	// handle enabling/disabling
	keyElem.setAttribute('keyenabled', selectedRow.getAttribute('keyenabled'));
	foxytunesShortcutDialogApplyKeyEnableDisable(keyElem);

	gbFoxyTunesShortcutsDirty = true;
}

function foxytunesShortcutDialogOnReset() {
	var tree = document.getElementById('foxytunes-keys-all-keys-tree');
	var selectedRow = foxytunesShortcutGetSelectedRow(tree);
	var selectedIndex = tree.view.selection.currentIndex;
	var allKeys = window.arguments[0];
	var key = allKeys[selectedIndex];

	// restore the defaults:
	var modifiers = key.getAttribute('origmodifiers');
	var keyAttr = key.getAttribute('origkey');
	var keyCodeAttr = key.getAttribute('origkeycode');

	var label = gFoxyTunesKeyUINames.GetPrettyLabel(modifiers, keyAttr, keyCodeAttr);

	foxytunesGetSensorEntryElem().value = label;
	foxytunesGetSensorEntryElem().keydata = [modifiers, keyAttr, keyCodeAttr];

	// apply:
	foxytunesShortcutDialogOnApply();
}

function foxytunesShortcutDialogShowHelp(title, text) {
	foxytunesShowAlert(window, title, text)
}

function foxytunesShortcutDialogOnAccept(warningTitle, warningText) {
	foxytunesShortcutDialogOnApply(); 
	if (gbFoxyTunesShortcutsDirty) {
		foxytunesShowAlert(window, warningTitle, warningText);
	}
	window.close(); 
}

function foxytunesShortcutDialogAllKeysEnableDisable(bDisable) {
	var tree = document.getElementById('foxytunes-keys-all-keys-tree');
	if (bDisable) {
		tree.setAttribute('disabled', 'true');
		var keyEntryElem = document.getElementById('foxytunes-shortcut-key-entry-box');
		keyEntryElem.setAttribute('hidden', 'true');

		var allCells = tree.getElementsByTagName('treecell');
		for (var i = 0; i < allCells.length; i++) {
			allCells[i].setAttribute('properties', 'disabled');
		}
	} else {
		tree.setAttribute('disabled', 'false');
		var keyEntryElem = document.getElementById('foxytunes-shortcut-key-entry-box');
		keyEntryElem.setAttribute('hidden', 'false');

		var allRows = tree.getElementsByTagName('treerow');
		for (var i = 0; i < allRows.length; i++) {
			var currentRow = allRows[i];
			if (currentRow.getAttribute('keyenabled') == 'true') {
				currentRow.childNodes[0].removeAttribute('properties');
				currentRow.childNodes[1].removeAttribute('properties');
			} else {
				currentRow.childNodes[0].setAttribute('properties', 'disabled');
				currentRow.childNodes[1].setAttribute('properties', 'disabled');
			}
		}

	}
}

function foxytunesShortcutDialogOnToggleUseShortcuts(event) {
	var bDisableAllShortcuts = (event.target.getAttribute('checked') != 'true');
	foxytunesShortcutDialogAllKeysEnableDisable(bDisableAllShortcuts);

	if (bDisableAllShortcuts) {
		foxytunesWriteSinglePreference('use_shortcuts', 'false');
	} else {
		foxytunesWriteSinglePreference('use_shortcuts', 'true');
	}
	gbFoxyTunesShortcutsDirty = true;
	sizeToContent(); 
	if (navigator.platform.toLowerCase().indexOf('mac') != -1) {
		// on mac the window seems to jump after to resize, move it to the center
		//foxytunesCenterWindowOnScreen();
	}
}

function foxytunesShortcutDialogTweakForReadOnly() {
	var keyEntryElem = document.getElementById('foxytunes-shortcut-key-entry-box');
	keyEntryElem.setAttribute('hidden', 'true');

	var dialog = document.documentElement;
	//var helpButton = dialog.getButton('help');
	//helpButton.setAttribute('hidden', 'true');
	var cancelButton = dialog.getButton('cancel');
	cancelButton.setAttribute('hidden', 'true');

	var tree = document.getElementById('foxytunes-keys-all-keys-tree');
	tree.setAttribute('disabled', 'true');
	
	var helpElem = document.getElementById('foxytunes-shortcut-help-frame');
	helpElem.setAttribute('hidden', 'false');

	var useShortcutsFlagElem = document.getElementById('foxytunes-shortcuts-use-flag');
	useShortcutsFlagElem.setAttribute('disabled', 'true');
}

function foxytunesShortcutSense(event) {
	event.preventDefault();
	event.stopPropagation();

	var modifiers = new Array();
	if (event.altKey) 	modifiers.push("alt");
	if (event.ctrlKey) 	modifiers.push("control");
	if (event.metaKey) 	modifiers.push("meta");
	if (event.shiftKey) 	modifiers.push("shift");
	modifiers = modifiers.join(",");

	var key = ''; 
	var keycode = '';
	if (event.charCode) {
		key = String.fromCharCode(event.charCode).toUpperCase();
	} else {
		keycode = gFoxyTunesKeyUINames.GetVKName(event.keyCode);
	}

	var label = gFoxyTunesKeyUINames.GetPrettyLabel(modifiers, key, keycode);	
	event.target.value = label;
	event.target.setSelectionRange(0, 0);
	event.target.keydata = [modifiers, key, keycode];
}

function foxytunesCenterWindowOnScreen() {
	var xOffset = screen.availWidth / 2 - window.outerWidth / 2;
	var yOffset = screen.availHeight / 2 - window.outerHeight / 2; 
	
	xOffset = Math.max(xOffset, 0);
	yOffset = Math.max(yOffset, 0);
	window.moveTo( xOffset, yOffset);
}

function foxytunesShortcutDialogOnLoad() {
	gFoxyTunesKeyUINames.Init();

	foxytunesShortcutsCreateContents();

	gbFoxyTunesShortcutsDirty = false;

	// init global enabled/disabled:
	var bUseShortcuts = foxytunesReadSinglePreference('use_shortcuts', 
								foxytunesShortcutsEnabledByDefault());
	foxytunesShortcutDialogAllKeysEnableDisable(bUseShortcuts != 'true');
	var checkBox = document.getElementById('foxytunes-shortcuts-use-flag');
	checkBox.setAttribute('checked', bUseShortcuts);

	// if in view mode - change some GUI:
	if (foxytunesShortcutsDialogReadOnly()) {
		foxytunesShortcutDialogTweakForReadOnly();
	}
	sizeToContent(); 
}

// user made some changes to the configuration
gbFoxyTunesShortcutsDirty = false;

