/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function foxytunesGetTrackInfoLabelElement() {
	return document.getElementById('foxytunes-track-title-status-label');
}

function foxytunesGetCurrentTrackTitleToNothingIsPlaying() {
    var str = 'FoxyTunes';
    var playerName = foxytunesGetCurrentPlayerReadableName();
    if (playerName) { 
        str = foxytunesGetLocaleString('nothingIsPlaying', [ playerName ]);
    }
    return str;
}

function foxytunesInitTrackTitleLabel() {

	// try getting polling interval from the preferences
	try {
		gFoxyTunesTrackTitleQueryInterval = gFoxyTunesPref.getIntPref("title_poll_interval");
	} catch (e) {} 
    var str = foxytunesGetCurrentTrackTitleToNothingIsPlaying();
	foxytunesSetCurrentTrackTitleLabel(str);
		// see if we should update the track info on startup (connect to the player)
		var confItem = document.getElementById('foxytunes-configuration-update-track-title-on-startup');
		if (confItem.getAttribute('checked') == 'true') {
			foxytunesUpdateTrackTitleLabel(true);
		}

		// polling was moved to Service
        // tbd: consider changing Service polling
        // behavior according to reference count of clients
		// foxytunesTrackTitleLabelUpdater();
}

function foxytunesUpdateTrackTitleLabel(connect, bCached) {
    var title = null;
	// query player only if asked to connect (param) or already connected
	if (connect || (gFoxyTunesPlayerObj != null)) {
		title = foxytunesGetCurrentTrackTitle(bCached);
		foxytunesSetCurrentTrackTitleLabel(title);
        if (!bCached) {
            // do not update position when getting title from cache
            // this will be modified by the observer
    		foxytunesUpdateTrackPositionMarker();
        }
	} else {
		var str = foxytunesGetCurrentTrackTitleToNothingIsPlaying();
		foxytunesSetCurrentTrackTitleLabel(str);
    	var elem = foxytunesGetTrackInfoLabelElement();
        elem.progress = -1;
	}
    var labelElem = foxytunesGetTrackInfoLabelElement();
    if (!title || title == "") {
        labelElem.setAttribute('ft-has-track', 'false');
    } else {
        labelElem.setAttribute('ft-has-track', 'true');
    }
}

function foxytunesUpdateTrackPositionMarker(length, pos) {
	var elem = foxytunesGetTrackInfoLabelElement();

	if (elem.getAttribute('seekslider') != 'true') {
		// we do not want a seek slider
		elem.progress = -1;
		return;
	}

    // two modes - either get pos+length from argument
    // or query the player for real values    
    if (typeof length == 'undefined') {
    	length = foxytunesGetCurrentTrackItem('Length');
    }
    if (typeof pos == 'undefined') {
    	pos = foxytunesGetCurrentTrackItem('Position');
    }

    if ((length != null) && (pos != null)) {
		elem.progress = pos / length;
	} else {
		elem.progress = -1;
	}
}

function foxytunesTrackTitleLabelUpdater() {
    // this method isn't currently used!
	foxytunesUpdateTrackTitleLabel();

	gFoxyTunesTrackTitleDisplayTimerID = setTimeout('foxytunesTrackTitleLabelUpdater()', 
								gFoxyTunesTrackTitleQueryInterval);
}

function foxytunesSetCurrentTrackTitleLabel(title) {
        if (title == null) {
		return;
      	}

	if (title == '') {
        var str = "";
	    var isPlaying = false;
        if (gFoxyTunesPlayerObj != null) {
            try {
                isPlaying = gFoxyTunesPlayerObj.IsPlaying();
            } catch (e) {};
        }
        if (!isPlaying) {
    	    str = foxytunesGetLocaleString("idle");
            var playerName = foxytunesGetCurrentPlayerReadableName();
            if (playerName) { 
                str = foxytunesGetLocaleString('nothingIsPlaying', [ playerName ]);
            }
        }
		title = str;
	}

	var elem = foxytunesGetTrackInfoLabelElement();
	elem.label = title;
}

function foxytunesTrackTitleDisplayVisibile() {
	var trackTitleDisplay = document.getElementById('foxytunes-track-title-display-container-statusbar'); 
	var visible = (trackTitleDisplay.getAttribute('hidden') == 'false');
	return visible;
}

function foxytunesTrackInfoDisplayMouseOver() {
  // cancel all timers
  if (gFoxyTunesTrackInfoDisplayAutohideTimerID != 0) {
	clearTimeout(gFoxyTunesTrackInfoDisplayAutohideTimerID);
  }
}

function foxytunesTrackInfoDisplayMouseOut() {
  if (! foxytunesTrackTitleAutoHideIsOn()) {
  	// auto-hide is off, nothing to do
	return;
  }

  var arrowButton = document.getElementById('foxytunes-toggle-track-title-display-visibility'); 
  var trackTitleDisplay = document.getElementById('foxytunes-track-title-display-container-statusbar'); 
  var bHidden = (trackTitleDisplay.getAttribute('hidden') == 'true');

  if (bHidden) {
	// already hidden, nothing to do
	return;
  }

  // stop the previous timers if exist
  if (gFoxyTunesTrackInfoDisplayAutohideTimerID != 0) {
	clearTimeout(gFoxyTunesTrackInfoDisplayAutohideTimerID);
  }

  // schedule auto-hide
  gFoxyTunesTrackInfoDisplayAutohideTimerID = 
			setTimeout(foxytunesToggleObjectVisibilityWithArrow, gFoxyTunesAutoHideTimeout, arrowButton, trackTitleDisplay, true);
}


function foxytunesTrackTitleAutoHideIsOn() {
  var autoHideMenuItem = document.getElementById('foxytunes-track-info-panel-auto-hide-toggle');
  return (autoHideMenuItem.getAttribute('checked') == 'true');
}

function foxytunesToggleTrackTitleDisplayVisibility(event, bHovering) {
	var arrowButton = event.target; 
	var trackTitleDisplay = document.getElementById('foxytunes-track-title-display-container-statusbar'); 
        var bHidden = (trackTitleDisplay.getAttribute('hidden') == 'true');

  	// if hovering and the panel is already visible, or auto-hide is off, return
	if (bHovering && (!bHidden || !foxytunesTrackTitleAutoHideIsOn())) {	
		return;
	}

	foxytunesToggleObjectVisibilityWithArrow(arrowButton, trackTitleDisplay, true);

	// stop any old timers
	if (gFoxyTunesTrackTitleDisplayTimerID != 0) {
		clearTimeout(gFoxyTunesTrackTitleDisplayTimerID);
	}

	var labelElem = foxytunesGetTrackInfoLabelElement();

	// start updater if needed
	if (foxytunesTrackTitleDisplayVisibile()) {
		// refresh title, connect if needed
		foxytunesUpdateTrackTitleLabel(true);

		// polling was moved to Service
        // tbd: consider changing Service polling
        // behavior according to reference count of clients
		//foxytunesTrackTitleLabelUpdater();

		// start scrolling if needed
		labelElem.refreshScroll(); 
	} else {
		// stop (pause) the scrolling
		labelElem.stopScroll(); 
	}
}


function foxytunesTrackTitleCopyToClipBoard() {
	var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"];
	clipboardHelper = clipboardHelper.getService(Components.interfaces.nsIClipboardHelper);

	var labelElem = foxytunesGetTrackInfoLabelElement();
     	clipboardHelper.copyString(labelElem.getAttribute('label'));
}

function foxytunesTrackTitleToggleScrolling() {
	var labelElem = foxytunesGetTrackInfoLabelElement();
	var scrolling;
	if (labelElem.getAttribute('scrolling') == 'true') {
		scrolling = 'false';
	} else {
		scrolling = 'true';
	}

	labelElem.setAttribute('scrolling', scrolling);
	labelElem.scrolling = (scrolling == 'true');

	labelElem.resetPosition();

	// start/stop scrolling if needed
	labelElem.refreshScroll(); 
}

function foxytunesTrackToggleSeekSlider() {
	var labelElem = foxytunesGetTrackInfoLabelElement();
	var seekslider;
	if (labelElem.getAttribute('seekslider') == 'true') {
		seekslider = 'false';
	} else {
		seekslider = 'true';
	}

	labelElem.setAttribute('seekslider', seekslider);

	foxytunesUpdateTrackPositionMarker();
}

function foxytunesTrackTitleToggleAlignment() {
	var labelElem = foxytunesGetTrackInfoLabelElement();
	var align;
	if (labelElem.getAttribute('ftalign') == 'right') {
		align = 'left';
	} else {
		align = 'right';
	}

	labelElem.setAttribute('ftalign', align);

	labelElem.resetPosition();
}

function foxytunesTrackTitleDisplayResizeDown(event) {
	if (gbFoxyTunesTrackPressing) {
		// sanity check
		dump("\nMouse down event when already pressing\n");
		return;
	}

	gbFoxyTunesTrackPressing = true;
	gFoxyTunesResizeStartX = event.screenX;
	var labelElem = foxytunesGetTrackInfoLabelElement();
	gFoxyTunesResizeInitialWidth = labelElem.getAttribute('width');
	addEventListener("mouseup", foxytunesTrackTitleDisplayResizeUp, false);
	addEventListener("mousemove", foxytunesTrackTitleDisplayResizeMove, false);
	//dump("Down: " + gFoxyTunesResizeStartX + "\n");
}

function foxytunesTrackTitleDisplayResizeUp(event) {
	if (gbFoxyTunesTrackPressing) {
		gbFoxyTunesTrackPressing = false;
		removeEventListener("mouseup", foxytunesTrackTitleDisplayResizeUp, false);
		removeEventListener("mousemove", foxytunesTrackTitleDisplayResizeMove, false);
		var labelElem = foxytunesGetTrackInfoLabelElement();
		labelElem.resetPosition();
		foxytunesUpdateTrackPositionMarker();
	}
	//dump("Up\n");
}

function foxytunesTrackTitleDisplayResizeMove(event) {
	if (gbFoxyTunesTrackPressing) {
		var offset = gFoxyTunesResizeStartX - event.screenX;
		var newWidth = gFoxyTunesResizeInitialWidth*1 + offset*1;
		//dump("move " + offset + ": ");
		//dump("gFoxyTunesResizeInitialWidth " + gFoxyTunesResizeInitialWidth + " - ");
		//dump("newWidth " + newWidth + "\n");
		if (newWidth*1 > 10) {
			var labelElem = foxytunesGetTrackInfoLabelElement();
			labelElem.setAttribute('width', newWidth);	
		}
	}
}

var foxytunesTrackTitleDragStartObserver =
{
  onDragStart: function (event, transferData, action) {
	var labelElem = foxytunesGetTrackInfoLabelElement();
     	var label = labelElem.getAttribute('label');

	foxytunesInstallDragDropObservers();
	transferData.data = new TransferData();
	transferData.data.addDataForFlavour("text/unicode", label);
  }
}

// the time ID for the query player for title polling:
var gFoxyTunesTrackTitleDisplayTimerID = 0;

// track title polling interval:
var gFoxyTunesTrackTitleQueryInterval = 5000;

// mouse pressed flag (resizer):
var gbFoxyTunesTrackPressing = false; 

// starting X coordinate for the resizer:
var gFoxyTunesResizeStartX = false; 

// the initial track title label width (for the resizer):
var gFoxyTunesResizeInitialWidth = false; 

// time ID for auto-hide:
var gFoxyTunesTrackInfoDisplayAutohideTimerID = 0;
