/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

const MAX_TRACKS_IN_HISTORY_MENU = 20;

function InfoBox() {
     this._currentTrackData = null;
     this.displayedTrackData = null;
     this._bDetached = false;
     this._bDetachedBeforeSearchPopup = false;

     this._coverGetter = null;
     this._foxytunesService = Components.classes['@foxytunes.org/FoxyTunesEngine/FoxyTunesService;1'].
                                    getService(Components.interfaces.IFoxyTunesService);
     this._historyTracks = this._foxytunesService.GetPlaybackHistory().wrappedJSObject;
     this._historyCurrentIndex = this._historyTracks.length ? this._historyTracks.length - 1 : -1;
     this._bIsInHistoryMode = false;

     this._lastDisplayedAlbum = null;

     this._platform = foxytunesGetPlatform();

     this.dateService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"].
			getService(Components.interfaces.nsIScriptableDateFormat);

     this._trackPositionTimerId = null;

     this._bIsPlaying = false;
     this._lastKnownTrackPos = 0;
     this._prevKnownTrackPos = 0;
     this._lastKnownTrackLength = 0;

     this._observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);

    this._unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                getService(Components.interfaces.nsIScriptableUnicodeConverter);
     this._prefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefBranch);

    this._foxytunesUtils = null;
    const kUtilsCID = "@foxytunes.org/FoxyTunesEngine/FoxyTunesUtils;1";
    if (typeof Components.classes[kUtilsCID] != 'undefined') {
        this._foxytunesUtils = Components.classes[kUtilsCID]
                    .getService(Components.interfaces.IFoxyTunesUtils);        
    }

    
     var me = this;
     this.albumCoverReceived = {
        handle : function(coverData) {
            // if failed to find in amazon - we check if we have it in other sources
            if (!coverData) {
                coverData = me.getAlbumCoverFromOtherSources(me.displayedTrackData);
            }
            me.setAlbumCover(coverData);
            // update album name if needed
            if (coverData && coverData.title && me.displayedTrackData && !me.displayedTrackData.album) {
                me.displayedTrackData.album = coverData.title;
                me.setAAT(me.displayedTrackData);
                if (!me.isDetached()) {
                    sizeToContent();
                    me.makeSureIsFullyOnscreen();
                }
            }
        }
     }
}

InfoBox.prototype = {
    onLoad : function(event) {
        foxytunesInitMenupopups();

        this.artistNameElem = document.getElementById('artist-name');
        this.trackNameElem = document.getElementById('track-name');
        this.albumNameElem = document.getElementById('album-name');

        this.artistBoxElem = document.getElementById('artist-name-box');
        this.trackIconElem = document.getElementById('track-icon');
        this.albumBoxElem = document.getElementById('album-name-box');

        this.albumCoverElem = document.getElementById('album-cover');
        this.albumCoverLink = null;

        this.statusBarDeckElem = document.getElementById('status-bar-deck');
        this.statusBarTextElem = document.getElementById('status-bar-text');
        this.statusBarBoxElem = document.getElementById('status-bar-box');

        this.progressBarElem = document.getElementById('progress-bar');
        this.progressLabelBoxElem = document.getElementById('track-position-label-box');
        this.progressLabelElem = document.getElementById('track-position-label');

        this.contentBoxElem = document.getElementById('content-box');

        this.historyBackElem = document.getElementById('history-back');
        this.historyForwardElem = document.getElementById('history-forward');
        this.historyListElem = document.getElementById('history-list-button');
        this.moreInfoButtonElem = document.getElementById('more-infobox-button');

        this.playerIconElem = document.getElementById('player-icon');

        this._currentTrackData = window.arguments[0];
        if (this._currentTrackData && this._currentTrackData.originalTitle) {
            this.setInfoBoxData(this._currentTrackData);
        } else {
            this.setInfoBoxData(null);
        }

        //if detached
        if (window.arguments[1]) {
            this.setDetached(true);
        }

	    if (this._platform == "mac") {
		    this.setDetached(true);
	    }

        this.setPlayerIcon();

        this.updateHistoryUIElements();

        this.setCurrentTrackTime();

        //dump('----> loading \n');
        var me = this;
        setTimeout(function () { me.delayedInit() }, 0);

        this._observerService.addObserver(this, "foxytunes-internal-playback-history", false);

        //this.flashStatusText('testing...');
	window.document.documentElement.setAttribute('ftplatform', this._platform);
    },

    onUnload : function(event) {
        this._observerService.removeObserver(this, "foxytunes-internal-playback-history");

        if (this._coverGetter) {
            this._coverGetter.abort();
        }

        // Added to solve an issue with detached mode on Mac
        this.closeSelf();
    },

    onMouseOver : function(event) {
        if (window.opener) {
            window.opener.foxytunesCancelHideTrackInfoBox();
        }
    },

    onClick : function(event) {
        if (window.opener) {
            window.opener.foxytunesCancelHideTrackInfoBox();
        }
    },


    onMouseOut : function(event) {
        // don't close on mouse out when detached
        if (this._bDetached) return;
        this.triggerCloseSelf();
    },

    onPopupShowing: function(event) {
	event.target._ftpopupshown=0;
    },

    onPopupShown: function(event) {
	event.target._ftpopupshown=1;
    },

    onPopupHiding: function(event) {
	if (this._platform != 'linux') return;
	if (event.target._ftpopupshown === 0) return false;
    },

    triggerCloseSelf : function() {
        if (window.opener) {
            try {
                window.opener.foxytunesTriggerHideTrackInfoBox();
            } catch(e) {
                // our opener died?
                close();
            }
        }
    },

    closeSelf : function() {
        this._bDetached = false;
        if (window.opener) {
            try {
                window.opener.foxytunesHideTrackInfoBox();
            } catch(e) {
                // our opener died?
                close();
            }
        } else {
                close();
        }
    },

    delayedInit : function() {
        sizeToContent();
        
        if (window.opener) {
	    var height = window.outerHeight;
            if (this._platform == 'mac') {
		//var diff = window.opener.outerHeight - window.opener.innerHeight;
		//height += diff;
		height += 30;
        }
            window.opener.gFoxytunesInfoBoxHeight = height;
	   
        }
    
           this.makeSureIsFullyOnscreen();
    },

    onMovedWithTitlebar : function(event) {
        this.setDetached(true);
    },

    setDetached : function(bDetached) {
        this._bDetached = bDetached;
        document.documentElement.setAttribute('detached', bDetached ? 'true' : 'false');
    },

    isDetached : function() {
        return this._bDetached;
    },

    setPlayerIcon : function() {

        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefBranch);
        var player = null;
        try {
            player = foxytunesTryGetFoxyTunesPlayerFromContractID(prefs.getCharPref("foxytunes.player_class"));
        } catch(e) { dump(e)}
    },

    setInfoBoxData : function(trackData) {
        if (trackData) {
            this.displayedTrackData = trackData;
            this.trackNameElem.removeAttribute('nolink');
            this.setAAT(trackData);
            if (this._lastDisplayedAlbum && trackData.album && 
                    (this._lastDisplayedAlbum == trackData.album)) {
                // we haven't changed an album, do nothing
            } else {
                this._lastDisplayedAlbum = trackData.album;
                this.fetchAlbumCover(trackData);
            }

            var bPlayerFound = false;
            if (trackData.player) {
          	    var player = foxytunesTryGetFoxyTunesPlayerFromContractID(trackData.player); 
                if (player) {
                    this.playerIconElem.setAttribute('playername', player.toLowerCase());            
                    this.playerIconElem.setAttribute('tooltiptext', player);            
                    this.playerIconElem.setAttribute('hidden', 'false');
                    bPlayerFound = true;
                }
            }
            if (!bPlayerFound) {
                this.playerIconElem.setAttribute('hidden', 'true');
            }
        } else {
            this.playerIconElem.setAttribute('hidden', 'true');

            this.displayedTrackData = null;

            this.trackNameElem.value = 'FoxyTunes';
            this.trackNameElem.setAttribute('nolink', 'true');

            this.artistNameElem.value = '';
            this.albumNameElem.value = '';

            this.artistBoxElem.hidden = true;
            this.trackIconElem.hidden = true;
            this.albumBoxElem.hidden = true;

            this._lastDisplayedAlbum = null;
            this.setDefaultCoverImage();
        }

        this.setExploreMenuData(trackData);
    },

    setExploreMenuData: function(trackData) {
        var artistMenuItem = document.getElementById('explore-artist-menuitem');

        if (!trackData || !trackData.artist) {
            artistMenuItem.setAttribute('disabled', true);
            return;
        } 

        var exploreMenuTooltip = foxytunesGetLocaleString('exploreOnThePlanet', ['"' + trackData.title + '"']);
        var exporeArtistLabel = foxytunesGetLocaleString('exploreArtistDefault');
        if (trackData && trackData.artist) {
            exporeArtistLabel = foxytunesGetLocaleString('exploreArtist', ['"' + trackData.artist + '"']);
            artistMenuItem.setAttribute('disabled', false);
            exploreMenuTooltip = foxytunesGetLocaleString('exploreOnThePlanet', ['"' + trackData.artist + '"']);
        }
        
        artistMenuItem.setAttribute('label', exporeArtistLabel);
        artistMenuItem.setAttribute('tooltiptext', exporeArtistLabel);

        var exploreMenu = document.getElementById('explore-button');
        exploreMenu.setAttribute('tooltiptext', exploreMenuTooltip);
    },

    setAAT : function(trackData) {  
        this.artistNameElem.value = trackData.artist;
        this.artistNameElem.setAttribute('tooltiptext',foxytunesGetLocaleString('openOnPlanet', ['"' + trackData.artist + '"']));
        if (trackData.artist) {
            this.artistBoxElem.hidden = false;
        } else {
            this.artistBoxElem.hidden = true;
        }

        var title = trackData.title ? trackData.title : trackData.originalTitle;
        this.trackNameElem.value = title;
        this.trackNameElem.setAttribute('tooltiptext',foxytunesGetLocaleString('openOnPlanet', ['"' + title + '"']));
        if (title) {
            this.trackIconElem.hidden = false;
        } else {
            this.trackIconElem.hidden = true;
        }

        this.albumNameElem.value = trackData.album;
        this.albumNameElem.setAttribute('tooltiptext', trackData.album);
        if (trackData.album) {
            this.albumBoxElem.hidden = false;
        } else {
            this.albumBoxElem.hidden = true;
        }
    },

    convertURLToSource : function(url) {
         var ioService = Components.classes["@mozilla.org/network/io-service;1"].
			              getService(Components.interfaces.nsIIOService);
         var uri = null;
         try {
                uri = ioService.newURI(url, null, null);
         } catch(e) {
            return null;
         }

         var host = uri.host;
         var source = host;
         if (host.match(/^www\.(.*)/)) {
             source  = RegExp.$1;
         }

         // make the first letter upper case
         source = source.substr(0,1).toUpperCase() + source.substr(1);
         return source;
    },

    goToAlbumPage : function(event) {
        if (this.albumCoverLink) {
            foxytunesGoToURL(this.albumCoverLink);
        }
    },

    setCoverImageLink : function(coverData, tooltip) {
        var albumCoverElem = this.albumCoverElem;
        if (coverData && coverData.link) {
            this.albumCoverLink = foxytunesSanitizeURL(coverData.link);

           // albumCoverElem.addEventListener('click', function() { foxytunesGoToURL(link); }, false );

            albumCoverElem.setAttribute('link', 'true');

            albumCoverElem.setAttribute('tooltiptext', coverData.title + " - " + this.convertURLToSource(coverData.link));
        } else {
            albumCoverElem.removeAttribute('link');
            albumCoverElem.removeAttribute('tooltiptext');
            albumCoverElem.removeAttribute('onclick');
            this.albumCoverLink = null;
        }
    },

    setAlbumCover : function(coverData) {
        var albumCoverElem = this.albumCoverElem;
        if (coverData) {
            albumCoverElem.src = coverData.imagePath;
            if (coverData.link) {
                this.setCoverImageLink(coverData);
            }
        } else {
            this.setDefaultCoverImage();
        }
    },

    setPleaseWaitCoverImage : function() {
        var albumCoverElem = this.albumCoverElem;
        albumCoverElem.src = 'chrome://foxytunes/skin/infobox/pleasewait.gif';
        this.setCoverImageLink(null);
    },

    setDefaultCoverImage : function() {
        var albumCoverElem = this.albumCoverElem;
        albumCoverElem.src = 'chrome://foxytunes/skin/infobox/genericAlbum.png';
        this.setCoverImageLink(null);
    },

    setCurrentTrackTime : function(tick) {
        if (!tick) tick = 0;

        var pos;
        var bRealPoll = (tick < 5) || ((tick % 5) == 0);
        if (bRealPoll) {
            this._prevKnownTrackPos = this._lastKnownTrackPos;
            
            // get current track length from cache (performance)
            var trackData = this._foxytunesService.GetCurrentTrackData(true); 
            if (trackData.wrappedJSObject) trackData = trackData.wrappedJSObject; 
            this._lastKnownTrackLength = trackData.length;
            //this._lastKnownTrackLength = this._foxytunesService.GetCurrentTrackLength();

            // get position from the player itself, getting from cache
            // might cause _bIsPlaying to be false
            this._lastKnownTrackPos = this._foxytunesService.GetCurrentTrackPosition();

            pos = this._lastKnownTrackPos;            

            this._bIsPlaying = (this._lastKnownTrackPos != this._prevKnownTrackPos);
        } else {
            // this is a trick. even though we get the real position
            // every 5 secs, increment it here by 1
            pos = this._lastKnownTrackPos;
            if (this._bIsPlaying) {
                pos += (tick % 5);
            }         
        }

        length = this._lastKnownTrackLength;
        if (pos > length) {
            pos = length;
        }

        var posStr = foxytunesSeconds2TimeStr(pos);
        var lengthStr = foxytunesSeconds2TimeStr(length);

        var bPosZero    = (!posStr || (posStr == '0:00'));
        var bLengthZero = (!lengthStr || (lengthStr == '0:00'));

        var value = 0;
        if (!bPosZero && !bLengthZero) {              
            value = Math.round(100*(pos/length));
        } else if (bLengthZero) {
            // hide scrollbar
            value = -1;
        }

        this.progressLabelBoxElem.hidden = (value == -1);
        this.progressLabelElem.value = posStr + ' / ' + lengthStr;

        if (bRealPoll) {
            // the progress bar is only updated with real values
            this.progressBarElem.setAttribute('value', value + '%');
            this.progressBarElem.setAttribute('track-position', pos);
            this.progressBarElem.setAttribute('track-length', length);
        }

        var me = this;
        clearTimeout(this._trackPositionTimerId);
        this._trackPositionTimerId = 
            setTimeout(function() { me.setCurrentTrackTime(tick+1) } , 1000);
    },

    getAlbumCoverFromOtherSources : function(trackData) {
        if (!trackData) {
            return null;
        }

        // we check if we got it from the player:
        var coverData = this.getAlbumCoverFromTrackData(trackData);
        if (!coverData) {
            // we try to look in the track's dir for Folder.jpeg :
            coverData = this.findLocalAlbumCover(trackData);
        }
        return coverData;
    },

    getAlbumCoverFromTrackData : function(trackData) {

        if (!trackData.albumCoverURL) {
            return null;
        }

        var albumCoverData = new AlbumCoverData(trackData.albumCoverURL, null, null, null, trackData.album);
        return albumCoverData;
    },

    findLocalAlbumCover : function(trackData) {

        if (!trackData.location) return null;

        var albumCoverData = null;
        try {
       
            var charset = null;
            if (this._foxytunesUtils) {
                charset = this._foxytunesUtils.GetCurrentCharset();
            }

            var path = trackData.location

            if (charset) {
                this._unicodeConverter.charset = charset;
                path = foxytunesEscapeNonAsciiText(this._unicodeConverter.ConvertFromUnicode(trackData.location) );
            }

            var lcPath = path.toLowerCase();
            var ios = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);
            var fileHandler = ios.getProtocolHandler("file")
                     .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
            var file = null;
            // checking if file is local
            if (lcPath.indexOf("file:") == 0 ) {
                file = fileHandler.getFileFromURLSpec(path);
            } else if (lcPath.match(/^[a-z]:/) || lcPath.match(/^\//) ){
                file = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
                file.initWithPath(path);
            } else {
                return null;
            }

            if (!file) return null;

//            search for album art files name like: ["Folder.jpg", "frontcover.jpg", "frontcover.gif", "front.jpg", "jacket.jpg"];
            var possibleImagesNameList = ['Folder.jpg'];
            try {
                possibleImagesNameList = this._prefs.getCharPref("foxytunes.supported_album_art_file_names").split(",");
            } catch(e) { dump(e)}

            var bFound = false;
            
            var rootFolder = file.parent;
            for (var i=0; i<possibleImagesNameList.length; i++) {
                var possibleImageName = possibleImagesNameList[i];
                file = rootFolder.clone();
                file.append(possibleImageName);
                if (file.exists()) {
                    bFound = true;
                    break;
                }
            }
            if (!bFound) {
                return null;
            }

            var url = fileHandler.getURLSpecFromFile(file);
            var albumCoverData = new AlbumCoverData(url, null, null, null, trackData.album);
        } catch(e) {
            dump('findLocalAlbumCover: ' + e + '\n');
        } 

        return albumCoverData;

    },

    fetchAlbumCover : function(trackData) {
        var country = null;
        try {
            country = this._prefs.getCharPref("foxytunes.amazon.country");
        } catch(e) {dump("Failed to read pref:" + e + "\n")}

        if (this._coverGetter) {
            // abort the previous getter
            this._coverGetter.abort();
        }

		var bFailed = true;
		if (country != "None") {
        	this._coverGetter = new AlbumCoverGetter(trackData.title,
                        						trackData.album,
                                                trackData.artist,   
                                                country,
												"small",
                                                this.albumCoverReceived);
        
        	if (this._coverGetter.init()) {   
	            // start anim
    	        try {
        	        this._coverGetter.get();
            	    this.setPleaseWaitCoverImage(); 
                    bFailed = false;
            	} catch(e) {
                	dump('fetchAlbumCover: ' + e + '\n');
            	}
	        }
        }

        if (bFailed) {
            var coverData = this.getAlbumCoverFromOtherSources(trackData);
            this.setAlbumCover(coverData)
        }

            
    },

    makeSureIsFullyOnscreen : function() {
        var outsideTheScreenPixels = window.screenX + window.outerWidth - window.screen.width;
        if (outsideTheScreenPixels > 0) {
            moveBy(-outsideTheScreenPixels, 0);
        }
    },

    enterHistoryModeIfNeeded : function() {
        if (!this._bIsInHistoryMode) {
            // TBD: do we want to detach when entering history mode?
            // this.setDetached(true);

          // removed history resizing - N.B.
           // window.resizeTo(400, window.outerHeight);
            this.makeSureIsFullyOnscreen();
            this._bIsInHistoryMode = true;
        }
    },

    updateHistoryUIElements : function() {
        this.historyBackElem.disabled = this._historyCurrentIndex <= 0;
        this.historyForwardElem.disabled = this._historyCurrentIndex >= this._historyTracks.length - 1;

        var bHaveHistory = this._historyTracks.length > 0;
        this.historyListElem.hidden = !bHaveHistory;
        this.moreInfoButtonElem.hidden = (this.displayedTrackData == null) || 
                                                (this.displayedTrackData.originalTitle == null);

        var bIsInHistoricTrack = (this._historyCurrentIndex != this._historyTracks.length - 1);
        document.documentElement.setAttribute('historic-track', bIsInHistoricTrack ? 'true' : 'false');
        //this.contentBoxElem.setAttribute('disabled', bIsInHistoricTrack);

        if (!bIsInHistoricTrack && this.displayedTrackData) {
              // refresh the progress view and start the timer
              this.progressBarElem.hidden = false;
              this.progressLabelBoxElem.hidden = false;
              this.setCurrentTrackTime();
        } else {
              // stop timer in history view
              clearTimeout(this._trackPositionTimerId);
              this.progressBarElem.hidden = true;
              this.progressLabelBoxElem.hidden = true;
        }
    },


    historyGoto : function(historyIndex) {
        if ((historyIndex < 0) || (historyIndex >= this._historyTracks.length)) {
            dump('wrong index supplied\n');
            return;
        }

        this._historyCurrentIndex = historyIndex;

        var trackData = this._historyTracks[this._historyCurrentIndex];
        this.setInfoBoxData(trackData);
        this.enterHistoryModeIfNeeded(); 

        this.updateHistoryUIElements();
    },

    historyBack : function() {
        this.setDetached(true);

        this._historyCurrentIndex--;

        var trackData = this._historyTracks[this._historyCurrentIndex];
        this.setInfoBoxData(trackData);
        this.enterHistoryModeIfNeeded();

        this.updateHistoryUIElements();
    },

    historyForward : function() {
        this._historyCurrentIndex++;

        var trackData = this._historyTracks[this._historyCurrentIndex];
        this.setInfoBoxData(trackData);

        this.updateHistoryUIElements();
    },

    initSearchPopup : function(event) {
        // Prevent the function from being called
        // from sub-menus popups
        if (event.target != event.originalTarget) {
            return;
        }

        this._bDetachedBeforeSearchPopup = this._bDetached;
        this.setDetached(true);

        var searchPopup = document.getElementById('search-popup'); 
        searchPopup.trackData = this.displayedTrackData;
    },

    executeSearch : function(event) {
        foxytunesExecuteSearch(event);
        
        // don't close window when detached
        if (this._bDetachedBeforeSearchPopup) return;

        // closing the InofBox sometimes causes crashes on Mac
        if (this._platform != 'mac') {
            this.closeSelf();
        }
    },

    getReadableTimeString : function(date) {
        var now = new Date();
        var milli1 = date.getTime();
        var milli2 = now.getTime();
        var deltaMilli = milli2 - milli1;
        var deltaSeconds = deltaMilli / 1000;
        var deltaMinutes = deltaSeconds / 60;
        var deltaHours = deltaMinutes / 60;
        var deltaDays = deltaHours / 24;
       
        var deltaStr = null; 
        if (deltaDays > 1) {
            deltaStr = null; 
        } else if (deltaHours > 1) {
            // Hours
            deltaHours = Math.round(deltaHours);
            if (deltaHours == 1) {
                 deltaStr =  foxytunesGetLocaleString('oneHourAgo');
            } else {
                 deltaStr =  foxytunesGetLocaleString('hoursAgo', [deltaHours]);
            }
        } else if (deltaMinutes > 1) {
            // Minutes
            deltaMinutes = Math.round(deltaMinutes);
            if (deltaMinutes == 1) {
                 deltaStr =  foxytunesGetLocaleString('oneMinuteAgo');
            } else {
                 deltaStr =  foxytunesGetLocaleString('minutesAgo', [deltaMinutes]);
            }
        } else if (deltaSeconds > 1) {
            deltaSeconds = Math.round(deltaSeconds);
            if (deltaSeconds == 1) {
                 deltaStr =  foxytunesGetLocaleString('oneSecondAgo');
            } else {
                 deltaStr =  foxytunesGetLocaleString('secondsAgo', [deltaSeconds]);
            }
        } 

        var time = this.dateService.FormatDateTime("", 
							this.dateService.dateFormatShort, 
							this.dateService.timeFormatNoSeconds, 
							date.getFullYear(), 
							date.getMonth() + 1, 
							date.getDate(), 
							date.getHours(), 
							date.getMinutes(),
                            0);

        var displayTime = deltaStr ? 
                            foxytunesGetLocaleString('itervalAndTime', [deltaStr, time]) : time;
        return displayTime;
    },

    onFoxyTunesClicked : function(event) {
        foxytunesGoToURL('http://www.foxytunes.com');
        if (!this.isDetached()) {
            this.closeSelf();
        }
    },

    launchWebPage : function(target, sID) {
        var trackData = this.displayedTrackData;
//        if (!trackData) return;

        foxytunesGotoPlanet(target, sID, trackData);
    },


    setStatusText  : function(str) {
        this.statusBarTextElem.value = str;
        if (str) {
            this.statusBarDeckElem.selectedIndex = 1;
        } else {
            this.statusBarDeckElem.selectedIndex = 0;
        }
    },

    fadeStatusText : function(value, offset) {
        value += offset;
        value = Math.max(0, value);
        this.statusBarBoxElem.style.opacity = value / 100;
        if (value == 0) {
            this.statusBarBoxElem.style.opacity = 1;
            this.setStatusText(null);
        } else {
            var me = this;
            setTimeout(function() { me.fadeStatusText(value, -5) }, 40);
        }
    },


    flashStatusText : function(str) {
        this.setStatusText(str);
        var me = this;
        setTimeout(function() { me.fadeStatusText(99, -1) }, 2000);
    },

    observe: function(subject, topic, data){
        if (topic != "foxytunes-internal-playback-history") {
            return;
        }
        var bUpdateHistoryUIElements = false;
        switch (data) {
            case "track-added-internal":
                bUpdateHistoryUIElements = true;
                break;
            case "track-skipped-internal":
                break;
            default:
                dump("Unkown data from " + topic + " -> " + data + "\n");
                return;
        }

        var bIsInHistoricTrack = (document.documentElement.getAttribute('historic-track') == 'true') ? true : false;
        if (!bIsInHistoricTrack) {
            this._currentTrackData = this._historyTracks[this._historyTracks.length-1];
            this._historyCurrentIndex = this._historyTracks.length-1;

            if (this._currentTrackData && this._currentTrackData.originalTitle) {
                this.setInfoBoxData(this._currentTrackData);
            } else {
                this.setInfoBoxData(null);
            }
        }

        if (bUpdateHistoryUIElements) {
            this.updateHistoryUIElements();
        }

    }
}

function MoreInfoBox(infoBox) {
    this._infoBox = infoBox;
    this._bParentDetached = false;
}

MoreInfoBox.prototype = {
    onShowing : function(event) {
        this._bParentDetached = this._infoBox.isDetached();
        if (!this._bParentDetached) {
            this._infoBox.setDetached(true);
        }
       
        this.setData(this._infoBox.displayedTrackData);
    },

    onHiding : function(event) {
        // wasn't detached when we first appeared, re-attach
        if (!this._bParentDetached) {
            // remain in a detached state
            // this._infoBox.setDetached(false);
        }
    },

    setData : function(trackData) {
        this._longestItem = '';

        if (!trackData) {
            this.setItemData('artist', null);            
            this.setItemData('track', null);            
            this.setItemData('album', null);            
            this.setItemData('time', null);            
            this.setItemData('time', null);            
            this.setItemData('player', null);            
            this.setItemData('location', null);            
            return; 
        }

        this.setItemData('artist', trackData.artist);       
        this.setItemData('track', trackData.title);       
        this.setItemData('album', trackData.album);       

        if (trackData.player) {
          	var player = foxytunesTryGetFoxyTunesPlayerFromContractID(trackData.player); 
            this.setItemData('player', player);            
        } else {
            this.setItemData('player', null);            
        }

        if (trackData.playTime) {
            var date = new Date(trackData.playTime);
            var displayTime = this._infoBox.getReadableTimeString(date);
            this.setItemData('time', displayTime);       
        } else {
            this.setItemData('time', null);       
        }

        if (trackData.location) {
            var location = trackData.location;
            location = location.replace(/^file:\/\/\//, '');
            location = location.replace(/^file:\/\//, '');
            this.setItemData('location', location);       
        } else {
                this.setItemData('location', null);       
        }

        // why do we need the 'width-corrector'?
        // if we have labels as data elements, the popup is shown 
        // having enough width to show the longest label.
        // when we replace labels with textboxes, the popup
        // is to narrow and the text is clipped. 
        // solution - create an invisible label having the value
        // of our longest item. it will make our popup wide enough.
//        this.setItemData('width-corrector', this._longestItem);       
        document.getElementById('more-info-width-corrector').value = this._longestItem;
    },

    setItemData : function(itemName, data) {
        var itemElem = document.getElementById('more-info-' + itemName);
        var itemRowElem = document.getElementById('more-info-' + itemName + '-row');

        // do not update fields without ui representation
        if (!itemElem || !itemRowElem) return;

        if (data) {
            if (data.length > this._longestItem.length) {
                this._longestItem = data;
            }
            itemRowElem.hidden = false;
            itemElem.value = data;
        } else {
            itemRowElem.hidden = true;
        }
    }
}


function HistoryListBox(infoBox) {
    this._infoBox = infoBox;
    this._bParentDetached = false;
}

HistoryListBox.prototype = {
    onShowing : function(event) {
        this._bParentDetached = this._infoBox.isDetached();
        if (!this._bParentDetached) {
            this._infoBox.setDetached(true);
        }

        this.setData();
    },

    onHiding : function(event) {
        // wasn't detached when we first appeared, re-attach
        if (!this._bParentDetached) {
            // remain in a detached state
            //this._infoBox.setDetached(false);
        }
    },

    setData : function() {
        var parent = document.getElementById('history-list');
        while (parent.firstChild && parent.firstChild.nextSibling) {
            parent.removeChild(parent.firstChild.nextSibling);
        }
        var nAdded = 0;
        for (var i = this._infoBox._historyTracks.length - 1; i >= 0 ; i--) {
            var trackData = this._infoBox._historyTracks[i];
            var menuitem = document.createElement('menuitem');
            menuitem.value = i;
            menuitem.setAttribute('label', trackData.normalizedOriginalTitle);
            parent.appendChild(menuitem);
            nAdded++;
            if (nAdded >= MAX_TRACKS_IN_HISTORY_MENU) break;
        }
    },

    onCommand : function(event) {
        if (event.target && (event.target.value >= 0)) {
            this._infoBox.historyGoto(event.target.value);
        }
    }
}

function ProgressBarListener() {
     this._foxytunesService = Components.classes['@foxytunes.org/FoxyTunesEngine/FoxyTunesService;1'].
                                    getService(Components.interfaces.IFoxyTunesService);
     this._bMouseIsDown = false;

     var me = this;
     this.mouseUpEventListener = function(event) { 
            me.onMouseUp(event); 
     }

     this.mouseMoveEventListener = function(event) { 
            me.onMouseMove(event); 
     }

     this._currentTarget = null;
}

ProgressBarListener.prototype = {
    onMouseOver : function(event) {
        var target = event.target;
        this.updateTooltip(target);
    },

    onMouseOut : function(event) {
    },

    onMouseDown : function(event) {
        if (this._bMouseIsDown) return;

        window.addEventListener('mouseup', this.mouseUpEventListener, true);
        window.addEventListener('mousemove', this.mouseMoveEventListener, true);
        this._bMouseIsDown = true;
        this._currentTarget = event.target;
        this.setProgress(event);

        gInfoBox.setDetached(true);
    },

    onMouseUp : function(event) {
        if (!this._bMouseIsDown) return;
        this._bMouseIsDown = false;
        window.removeEventListener('mouseup', this.mouseUpEventListener, true);
        window.removeEventListener('mousemove', this.mouseMoveEventListener, true);
    },

    onMouseMove : function(event) {
        if (!this._bMouseIsDown) return;

        this.setProgress(event);
    },

    setProgress : function(event) {
        var target = this._currentTarget;
        var width = target.boxObject.width;
        var offset = event.screenX - target.boxObject.screenX;
        var percentage = offset/width;
        target.setAttribute('value', Math.round(100*(percentage)) + '%');

        var length = target.getAttribute('track-length');
        var newPos = Math.round(percentage*length);
        target.setAttribute('track-position', newPos);
        this.updateTooltip(target);

        this._foxytunesService.SetCurrentTrackPosition(newPos);       

        // update the global object (so the label is displayed correctly):
        gInfoBox._lastKnownTrackPos = newPos;
    },

    updateTooltip : function(target) {
        var pos = target.getAttribute('track-position');
        var length = target.getAttribute('track-length');

        var progressLabelElem = document.getElementById('track-position-label');

        var posStr = foxytunesSeconds2TimeStr(pos);
        var lengthStr = foxytunesSeconds2TimeStr(length);
        if (posStr && lengthStr &&
            (posStr != '0:00') && (lengthStr != '0:00')) {
            
           	var progress = (pos / length);
		    var persentage =  Math.round(progress * 100);

            progressLabelElem.value = posStr + ' / ' + lengthStr;

            var tooltip = posStr + ' / ' + lengthStr + ' - ' + persentage + '%';
            target.setAttribute('tooltiptext', tooltip);    
        } else {
            target.removeAttribute('tooltiptext');    
        }
   }
}

var gInfoBox = new InfoBox();
var gMoreInfoBox = new MoreInfoBox(gInfoBox);
var gHistoryListBox = new HistoryListBox(gInfoBox);
var gProgressBarListener = new ProgressBarListener();
