/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function MiniBrowser() {
}

MiniBrowser.prototype = {
    init : function() {
        var me = gMiniBrowser;
    	var contextMenu = document.getElementById("contentAreaContextMenu");
    	contextMenu.addEventListener("popupshowing", me.onContextMenuShowing, false);

        var openerType = null;
        if (window.opener) {
            openerType = window.opener.document.documentElement.getAttribute('windowtype');
        }

        if (openerType.match(/minimode/i)) {
            //setTimeout(function() { me.dockToMiniMode(window.opener) }, 0);
        }

        addEventListener('unload', function(event) { me.onClosing(event) }, false);
        addEventListener('resize', function(event) { me.onResize(event) }, false);

        var defaultBrowserButtonElem = document.getElementById('open-in-default-browser-button');
        var defaultBrowserIcon = foxytunesGetDefaultBrowserIcon();
        if (defaultBrowserButtonElem && defaultBrowserIcon) {
             defaultBrowserButtonElem.setAttribute('image', defaultBrowserIcon);
        }
        
        me.setWindowIconToFoxyTunesIcon();

    },

    unload : function() {
        // Destroy the MiniBrowser icon (after chaning it to FoxyTunes icon on init)
        // Currently only works in Windows
        try {
            var baseWindow = foxytunesGetBaseWindow(window);
            var windowController = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesWindowController;1"]
                                 .getService(Components.interfaces.IFoxyTunesWindowController);
            windowController.DestroyWindowIcon(baseWindow);
        } catch (e) {};
    },

    setWindowIconToFoxyTunesIcon : function() {
        // Change the MiniBrowser icon into FoxyTunes icon 
        // Currently only works in Windows
        try {
            var foxytunesIconPath = foxytunesGetIconPath();
            var baseWindow = foxytunesGetBaseWindow(window);
            var windowController = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesWindowController;1"]
                                 .getService(Components.interfaces.IFoxyTunesWindowController);
            windowController.SetWindowIcon(baseWindow, foxytunesIconPath);
        } catch (e) {};
    },

    dockToMiniMode : function(dockToWin) {
        var bDockBelow = dockToWin.screenY < window.screen.height/2;
        var targetTop = dockToWin.screenY;
        var targetBottom = targetTop + dockToWin.outerHeight;
        var targetRight = dockToWin.screenX + dockToWin.outerWidth;

        var myHeight = window.outerHeight;
        var myWidth = window.outerWidth;

        var wantedX = targetRight - myWidth;
        var wantedY = bDockBelow ? targetBottom : targetTop - myHeight;

        window.moveTo(wantedX, wantedY);
    },

    saveCurrentSize : function() {
        var prefs = new FTJSLib.Prefs();
        prefs.setInt('foxytunes.minibrowser.width', window.outerWidth);
        prefs.setInt('foxytunes.minibrowser.height', window.outerHeight);
    },

    onClosing : function(event) {
        this.saveCurrentSize();
    },

    onResize : function(event) {
        this.saveCurrentSize();
    },

    onContextMenuShowing : function(event) {
        gContextMenu.showItem("context-open-external-browser", 
                    !( gContextMenu.isContentSelected || 
                       gContextMenu.onLink || 
                       gContextMenu.onImage || 
                       gContextMenu.onTextInput ) );
    },

    openInDefaultBrowser : function(url) {
        var ioService = Components.classes["@mozilla.org/network/io-service;1"].
                           getService(Components.interfaces.nsIIOService); 
        var uri = ioService.newURI(url, null, null);
        var externalProtocolService = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].
                                        getService(Components.interfaces.nsIExternalProtocolService);         

        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefBranch);

        var protocols = ['http', 'https', 'about'];
        var bSavedWarnValues = {};
        for (var i in protocols) {
            var protocol = protocols[i];
            bSavedWarnValues[protocol] = -1;
            try {
                bSavedWarnValues[protocol] = prefs.getBoolPref("network.protocol-handler.warn-external." + protocol);
            } catch(e) {}
            prefs.setBoolPref("network.protocol-handler.warn-external." + protocol, false);
        }
        externalProtocolService.loadUrl(uri);         

        function restoreHandlerPrefs() {
            for (var i in protocols) {
                var protocol = protocols[i];
                if (bSavedWarnValues[protocol] != -1) {
                    prefs.setBool('network.protocol-handler.warn-external.' + protocol, bSavedWarnValues[protocol]); 
                } else {
                    prefs.clearUserPref('network.protocol-handler.warn-external.' + protocol);
                }
            }
        } // restoreHandlerPrefs

        // looks like loadUrl is called async., so restoring the prefs right away won't help
        // remove the warnings   
        setTimeout(restoreHandlerPrefs, 1000); 
    },

    openLinkInDefaultBrowser : function() {
        this.openInDefaultBrowser(gContextMenu.linkURL);
    },

    openCurrentPageInDefaultBrowser : function() {
        var browser =  document.getElementById('content');
        var url = 'about:blank';
        const browsers = browser.browsers;
        if (browsers && (browsers.length > 1)) {
            url = browser.selectedBrowser.webNavigation.currentURI.spec;
        } else {
            url = browser.webNavigation.currentURI.spec;
        }
        this.openInDefaultBrowser(url);
    }
}

var gMiniBrowser = new MiniBrowser();

addEventListener("load", gMiniBrowser.init, false);
addEventListener("unload", gMiniBrowser.unload, false);
