/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function FoxyTunesMiniMode() {
    this._observerService = Components.classes["@mozilla.org/observer-service;1"]
                              .getService(Components.interfaces.nsIObserverService);

    this._trayService = null;
    this._windowController = null;

    try {
        this._trayService = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesTrayService;1"]
                                     .getService(Components.interfaces.IFoxyTunesTrayService);
    } catch(e) {}

    try {
        this._windowController = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesWindowController;1"]
                                     .getService(Components.interfaces.IFoxyTunesWindowController);
    } catch(e) {}

    this._minimizedToTray = false;
    this._mouseIsInTray = false;
    this._trayX = -1;
    this._trayY = -1;
    this._bIsUIReady = false;
}

FoxyTunesMiniMode.prototype = {
    init : function() {
       this._bIsUIReady = true;
       try {
           var skinURI = 'chrome://foxytunes-skin/content/foxytunesOverlay.xul';
           var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                                     .getService(Components.interfaces.nsIIOService);
           var uri = ioService.newURI(skinURI, null, null);

           var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
                            .getService(Components.interfaces.nsIChromeRegistry);
           try {
               chromeRegistry.convertChromeURL(uri);
               document.loadOverlay(skinURI, null);
           } catch(e) {}

        } catch(e) {}

        this._baseWindow = foxytunesGetBaseWindow(window);
        if (this.isOnWindows()) {
            this._windowController.RemoveWindowFromTaskBar(this._baseWindow);
        }

    /*
       var resizer = document.getElementById('foxytunes-track-title-status-panel-resizer');
       resizer.addEventListener('mouseup', 
                function(event) { FoxyTunesMinimodeResizerHandler.onMouseUp(event) }, false);
       resizer.addEventListener('mousedown', 
                function(event) { FoxyTunesMinimodeResizerHandler.onMouseDown(event) }, false);
       resizer.addEventListener('mousemove', 
                function(event) { FoxyTunesMinimodeResizerHandler.onMouseMove(event) }, false);
    */

       var resizer = document.getElementById('foxytunes-minimode-resizer');
       resizer.addEventListener('mouseup', 
                function(event) { FoxyTunesMinimodeResizerHandler.onMouseUp(event) }, false);
       resizer.addEventListener('mousedown', 
                function(event) { FoxyTunesMinimodeResizerHandler.onMouseDown(event) }, false);
       resizer.addEventListener('mousemove', 
                function(event) { FoxyTunesMinimodeResizerHandler.onMouseMove(event) }, false);

       window.addEventListener('mouseup', 
                function(event) { FoxyTunesMinimodeMoveHandler.onMouseUp(event) }, false);
       window.addEventListener('mousedown', 
                function(event) { FoxyTunesMinimodeMoveHandler.onMouseDown(event) }, false);
       window.addEventListener('mousemove', 
                function(event) { FoxyTunesMinimodeMoveHandler.onMouseMove(event) }, false);

       this._observerService.addObserver(this,"domwindowclosed", false);

       this.readPrefs();

       if (this.prefs['borderless']) {
            document.documentElement.setAttribute('borderless', 'true');
       }

        
       var me = this;
       setTimeout(function() { me.delayedInit() }, 1);
       // make sure our bottom-right corner is on-screen
       setTimeout(function() { me.ensureIsOnScreen(); }, 4);
    },

    unload : function() {
        this._observerService.removeObserver(this, "domwindowclosed");
        if (this.isOnWindows()) {
            this._trayService.DeleteTrayIcon();
            this._windowController.ResetRemovedFromTaskBarWindow(false); 
        }
    },

    delayedInit : function() {
        this.adjustSize(null, true);
        
        // this solved some additional layout problems (messed up borders around close/minimize buttons)
        resizeBy(1, 0);
        setTimeout('foxytunesUpdateTrackTitleLabel(true)', 50);

        // the following fixes a nasty Mozilla bug, in which the window opens with 
         // extra areas visible (the removed title bar?)
        if (!this.prefs['alwaysontop']) {
           var width = window.outerWidth;
           window.resizeTo(width, 16);
           this.adjustSize();
        }
    
        if (this.isOnWindows()) {
            this.initTrayIcon();
        }
    },

    isOnWindows : function() {
        return (foxytunesGetPlatform() == 'windows');
    },

    readPrefs : function() {
        function fetchOptionWithDefault(key, defaultVal) {
           try {
             var val = pref.getBoolPref(key);
             return val;   
           } catch (e) {
             return defaultVal; 
           }
        }

        this.prefs = new Object();

        var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
        var pref = prefService.getBranch("foxytunes.minimode.options."); 
        this.prefs['alwaysontop'] = fetchOptionWithDefault('alwaysontop', true);
        this.prefs['borderless']  = fetchOptionWithDefault('borderless', false);
    },

    writePrefs : function() {
        var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
        var pref = prefService.getBranch("foxytunes.minimode.options."); 
        for (var option in this.prefs) {
               try {
                   pref.setBoolPref(option, this.prefs[option]); 
               } catch(e) { 
                   dump("Couldn't save pref: " + option + '\n');
               }
        }
        try {
        	prefService.savePrefFile(null);
        } catch(e) { dump("\nSaving preferences failed: " + err + "\n"); }
    },

    ensureIsOnScreen : function() {

        // make sure that our bottom right corner is on-screen

        var currentWidth = window.outerWidth;
        var screenWidth = window.screen.availWidth;

        var currentX = window.screenX;
        var currentY = window.screenY;
        var currentHeight = window.outerHeight;
        var right = currentX + currentWidth; 
        var bottom = currentY + currentHeight;
        var screenHeight = window.screen.availHeight;

        var left = currentX;
        if (right >  screenWidth) {
            left = left - (right -  screenWidth);
        }

        var top = currentY;
        if (bottom >  screenHeight) {
            top = top - (bottom -  screenHeight);
        }        
        moveTo(left, top);
    },

    ensureFitsScreen : function() {
        var currentWidth = window.outerWidth;
        var screenWidth = window.screen.availWidth;

        if (currentWidth > screenWidth) {
            var panelElem  = document.getElementById('foxytunes-track-title-status-label');
            var oldTitleWidth = panelElem.getAttribute('width');
            if (!oldTitleWidth) {
                oldTitleWidth = 0;
            }
            var newTitleWidth = oldTitleWidth - ( currentWidth - screenWidth);
            if (newTitleWidth < 0) {
                newTitleWidth = 0;
            }
            panelElem.setAttribute('width', newTitleWidth);
        }            
        
    },

    adjustSize : function(event, firstTime) {
        if (!this._bIsUIReady) return;
        
        var currentX = window.screenX;
        var currentY = window.screenY;
        var currentWidth = window.outerWidth;
        var right = currentX + currentWidth; 
        sizeToContent();

        var newWidth = window.outerWidth;
        var left = right - newWidth;
        
        this.ensureFitsScreen();
        moveTo(left, currentY);
        
        var me = this;
        if (firstTime) {
            // doing sizeToContent once often leaves the minimode in incorrect state
            // let's do it one more time
            setTimeout(function() { me.adjustSize(event); }, 100);
        }
    },

    restartSelf : function() {
        // delay this (crashes if called directly)
        setTimeout(function() { foxytunesOpenMinimode(true); close(); }, 10);
    },

    setFlavor : function(event) {
        var flavor = event.target.getAttribute('flavor');
        if (flavor) {
            document.documentElement.setAttribute('flavor', flavor);
        }
        this.restartSelf();
    },

    toggleOption : function(event) {
        var option = event.target.getAttribute('minimodeoption');
        if (option) {
            this.prefs[option] = !this.prefs[option];
            this.writePrefs();
        }
        this.restartSelf();
    },


    selectFlavorInUI : function(event) {
        var target = event.target;
        var flavor = document.documentElement.getAttribute('flavor');
        for (var i=0; i < target.childNodes.length; i++) {
            if (target.childNodes[i].getAttribute('flavor') == flavor) {
                target.childNodes[i].setAttribute('checked', 'true');
            } else {
                target.childNodes[i].setAttribute('checked', 'false');
            }
        }
    },

    setOptionValueInMenu : function(event) {
        var target = event.target;
        for (var i=0; i < target.childNodes.length; i++) {
            var elem = target.childNodes[i]; 
            if (elem.hasAttribute('minimodeoption')) {
                var option = elem.getAttribute('minimodeoption');
                if (typeof this.prefs[option] != 'undefined') {
                    elem.setAttribute('checked', this.prefs[option] ? 'true' : 'false');
                }
            }
        }
    },

    setupOptionsMenu : function(event) {
        this.selectFlavorInUI(event);
        this.setOptionValueInMenu(event);
    },

    onMoved : function(event) {
      //  foxytunesAdjustPopupPositions();
    },

    onDOMWindowClosed : function() {
        // When we'll have Mini for Mac+Linux we'll need to think about this feature
        if (!this.isOnWindows()) {
            return;
        }

        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
        var nWindows = 0;
        var enumerator = wm.getEnumerator(null);
        while (enumerator.hasMoreElements()) {
              var win = enumerator.getNext();
              var winType = win.document.documentElement.getAttribute('windowtype');
              if ((winType == 'foxytunes:infobox') || (winType == 'foxytunes:minibrowser')) {
                continue;
              }
              nWindows++;
        }

        if (nWindows == 1) {
            var IFoxytunesTrayService =  Components.interfaces.IFoxyTunesTrayService;

            var appName = foxytunesGetAppName();
            appName = appName.charAt(0).toUpperCase() + appName.substr(1).toLowerCase();
            var balloonHelpTitle = foxytunesGetLocaleString('minimodeRemainsAloneBalloonWarningTitle', [appName]);
            var balloonHelpText  = foxytunesGetLocaleString('minimodeRemainsAloneBalloonWarningText', [appName]);
            var balloonHelptimeout = 4000;
            var ret = this._trayService.ShowBalloon(balloonHelpTitle, balloonHelpText, 
                                IFoxytunesTrayService.ICON_TYPE_WARNING, balloonHelptimeout);
        
            // User has old shell32.dll -> showing balloon from xul
            if (!ret) {
                var x = new Object();
                var y = new Object();
                this._trayService.GetTrayXY(x, y);
                foxytunesShowBalloonHelp(x.value, y.value, balloonHelpTitle, balloonHelpText, balloonHelptimeout);
            }
        }
    },

    // TBD: really need to rewrite this :)
    simulateSearchEnterPress : function(event) {

        if (event.originalTarget.localName != 'toolbarbutton') return;

        var myEvent = {}; 
        myEvent.keyCode = 13;
        myEvent.target = document.getElementsByTagName('foxytunessearchbar')[0];
        this.doGeneralSearch(myEvent);
    },

    doGeneralSearch : function(event) {
        if (event.keyCode != 13) return;

        var target = event.target;
        var searchEngine = target.currentSearchEngine;

        if (!searchEngine.url) {
            // Handle Firefox search engine
            var submission = searchEngine.getSubmission(target.value, null);
            if (submission) {
                url = submission.uri.spec;
            }
        } else {
            var searchParameters = new Object();
            searchParameters.url = searchEngine.url;
            searchParameters.process = searchEngine.process;
            searchParameters.encoding = searchEngine.encoding;
            searchParameters.query = target.value;

            var searchEngine = new FoxyTunesSearchEngine(searchParameters, this.trackData);
	        var url = searchEngine.getURL();
        }
        foxytunesGoToURL(url);
    },

    minimize : function() {
        if (!this.isOnWindows()) {
            minimize();
            return;
        }

        this._trayService.DrawAnimatedRectsFromTray(this._baseWindow);
        this._windowController.HideWindow(this._baseWindow);
        this._minimizedToTray = true;
    },

    openTrayIconMainMenu : function(cursorX, cursorY) {
        var popup = document.getElementById('foxytunes-tray-main-menu-popup');
        popup.showPopup(
            document.documentElement,   
            cursorX,                    
            cursorY,                    
            "context",                  
            "",                         
            "");                        
    },

    restoreWindow : function() {
        this._trayService.DrawAnimatedRectsToTray(this._baseWindow);
        this._windowController.ShowWindow(this._baseWindow);
        this._minimizedToTray = false;
    },

    focusWindow : function () {
        if (this.isOnWindows() && this.isMinimizedToTray()) {
            this.restoreWindow();
        } else {
            focus();
        }
        
        this.adjustSize();
        this.ensureIsOnScreen();
    },

    initTrayIcon : function () {
        var me = this;
        var trayCallback = {
            // IFoxyTunesTrayServiceCallback
            Notify : function(eventType, keyMask, cursorX, cursorY, trayPos) {
                var IFoxyTunesTrayServiceCallback = Components.interfaces.IFoxyTunesTrayServiceCallback;
                switch (eventType) {
                    case IFoxyTunesTrayServiceCallback.EVENT_MOUSEUP_LEFT:
                        foxytunesCancelShowTrackInfoPopup();
                        setTimeout(function() {me.focusWindow();}, 20);
                        break;
                    case IFoxyTunesTrayServiceCallback.EVENT_MOUSEMOVE:
                            // Ignore move if x,y didn't change
                            if (cursorX == me._trayX && cursorY == me._trayY) {
                                return;
                            }

                            var popup = document.getElementById('foxytunes-tray-main-menu-popup');
                            
                            // Don't show infobox if popup is openned
                            if (popup.popupBoxObject.width != 0) {
                                return;
                            }

                            me._mouseIsInTray = true; 

                            me._trayX = cursorX;
                            me._trayY = cursorY;

                            // Ugly hack to open infobox above our icon
                            if (trayPos == IFoxyTunesTrayServiceCallback.FT_ABE_RIGHT) {
                                me._trayY -= 8;
                            }

                            // simulate dummy event for foxytunesTriggerShowTrackInfoPopup
                            var event = new Object();
                            event.relatedTarget = true;
                            event.bInfoBoxFromTray = true
                            foxytunesTriggerShowTrackInfoPopup(event);
                        break;
                    case IFoxyTunesTrayServiceCallback.EVENT_MOUSELEAVE:
                        // Reset trayX, trayY to make sure that in the next 
                        // EVENT_MOUSEMOVE they wont be equal to cursorX, cursorY
                        me._trayX = -1;
                        me._trayY = -1;

                        me._mouseIsInTray = false;
                        foxytunesTriggerHideTrackInfoBox(null);
                        break;
                    case IFoxyTunesTrayServiceCallback.EVENT_MOUSEUP_RIGHT:
                        foxytunesHideTrackInfoBox(null);
                        foxytunesTriggerHideTrackInfoBox(null);

                        setTimeout(function() {me.openTrayIconMainMenu(cursorX, cursorY);}, 20);
                        break;
                }   
            },

            QueryInterface : function(iid) {
                if( iid.equals(Components.interfaces.IFoxyTunesTrayServiceCallback) ) {
                    return this;
                }
                throw Components.results.NS_NOINTERFACE;
            }     
        };

        var foxytunesIconPath = foxytunesGetIconPath();
        this._trayService.AddTrayIcon(foxytunesIconPath, trayCallback);
    },

    isMinimizedToTray : function () {
        return this._minimizedToTray;
    },

    isMouseInTray : function () {
        return this._mouseIsInTray;
   },

    getTrayX : function () {
        return this._trayX;
    },

    getTrayY : function () {
        return this._trayY;
    },

    observe: function(subject,topic,data) {
        if (topic == 'domwindowclosed') {
            var win = subject;
            if (win == window) { 
                // we're not interested in ourselves closing
                return; 
            }
            var id = win.document.documentElement.id;
            var type = win.document.documentElement.getAttribute('windowtype');
            if ((id == 'commonDialog') || (type && type.match('foxytunes'))) {
                // ignore alerts and other prompts
                return;
            }

            var me = this;
            setTimeout(function() { me.onDOMWindowClosed(); }, 100);
        }
    }

}

var FoxyTunesMinimodeResizerHandler = {
    bResizing : false,
    startWindowWidth : null,
    startPanelWidth : null,
    panelElem : null,

    onMouseMove : function(event) {
        if (!this.bResizing) return;
        var currentWindowWidth = window.outerWidth;
        var deltaX = currentWindowWidth - this.startWindowWidth;
    
        var newWidth = 1*this.startPanelWidth + deltaX;
        if (newWidth > 10) {
            this.panelElem.setAttribute('width', newWidth);
        } 
    },

    onMouseUp : function(event) {
        if (!this.bResizing) return;
        this.bResizing = false;
        gFoxyTunesMiniMode.adjustSize()
		var labelElem = foxytunesGetTrackInfoLabelElement();
		labelElem.resetPosition();
		foxytunesUpdateTrackPositionMarker();
    },

    onMouseDown : function(event) {
        if (this.bResizing) return;
        this.bResizing = true;
        this.startWindowWidth = window.outerWidth;
        this.panelElem  = document.getElementById('foxytunes-track-title-status-label');
        this.startPanelWidth = this.panelElem.getAttribute('width');
    }


/*
    onMouseMove : function(event) {
        if (!this.bResizing) return;
        var delta = event.target.boxObject.screenX - window.screenX;
        if (delta < 10) {
            window.resizeBy(50, 0);
            window.moveBy(-50, 0);
            return;
        }
    },

    onMouseUp : function(event) {
        if (!this.bResizing) return;
        this.bResizing = false;
        var delta = event.target.boxObject.screenX - window.screenX;
        delta -= 5;
        if (delta > 0) {
            window.resizeBy(-delta, 0);
            window.moveBy(delta, 0);
        }
    },

    onMouseDown : function(event) {
        if (this.bResizing) return;
        this.bResizing = true;
        window.resizeBy(200, 0);
        window.moveBy(-200, 0);
    }
*/
}

var FoxyTunesMinimodeMoveHandler = {
    bMoving : false,
    startX : -1,
    startY : -1,
    stickyWindow : null,
    bDocked : false,
    bShouldDockTop : false,
    bShouldDockBottom : false,

    setDockMark : function(bTop, bShow) {
        return; // TBD
        if (bTop) {
            if (bShow) {
                window.document.documentElement.style.borderTopColor = 'red !important';
            } else {
                window.document.documentElement.style.borderTopColor = '';
            }
        } else {
            if (bShow) {
                window.document.documentElement.style.borderBottomColor = 'red !important';
            } else {
                window.document.documentElement.style.borderBottomColor = '';
            }
        }
    },

    onMouseMove : function(event) {
        if (!this.bMoving) return;

        var currentX = window.screenX;
        var currentY = window.screenY;
        var deltaX = currentX - this.startX;
        var deltaY = currentY - this.startY;
        this.startX = currentX;
        this.startY = currentY;

        if (this.bDocked) {
            this.stickyWindow.moveBy(deltaX, deltaY);
        } else if (this.stickyWindow) {
            var myTop    = window.screenY;
            var myBottom = myTop + window.outerHeight;
            var stickyWindowTop = this.stickyWindow.screenY;
            var stickyWindowBottom = stickyWindowTop + this.stickyWindow.outerHeight;
            var bShouldDockBottomNow = ((myTop - stickyWindowBottom) > 0) && ((myTop - stickyWindowBottom) < 5);
            var bShouldDockTopNow = ((stickyWindowTop - myBottom) > 0) && ((stickyWindowTop - myBottom) < 5);
            if (bShouldDockBottomNow) {
                //jitter: setTimeout(function() { window.moveTo(window.screenX, stickyWindowBottom); }, 0);
                this.setDockMark(true, true);
            } else if (this.bShouldDockBottom) {
                this.setDockMark(true, false);
            }
            this.bShouldDockBottom = bShouldDockBottomNow;

            if (bShouldDockTopNow) {
                this.setDockMark(false, true);
            } else if (this.bShouldDockTop) {
                this.setDockMark(false, false);
            }
            this.bShouldDockTop = bShouldDockTopNow;

        }


//        dump('moving ' + deltaX + ' - ' + deltaY + '\n');
    },

    onMouseUp : function(event) {
        if (!this.bMoving) return;
        this.bMoving = false;
    },

    onMouseDown : function(event) {
        if (event.target.tagName != 'titlebar') return;
        if (this.bMoving) return;
        this.bMoving = true;
        this.startX = window.screenX;
        this.startY = window.screenY;

        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);
        this.stickyWindow = wm.getMostRecentWindow('foxytunes:minibrowser');
        if (!this.stickyWindow) {
            this.stickyWindow = wm.getMostRecentWindow('foxytunes:infobox');
        } 

        if (!this.stickyWindow) return;
        
        
        var myTop    = window.screenY;
        var myBottom = myTop + window.outerHeight;
        var stickyWindowTop = this.stickyWindow.screenY;
        var stickyWindowBottom = stickyWindowTop + this.stickyWindow.outerHeight;
      
        dump( myTop - stickyWindowBottom + '\n');
        dump( stickyWindowTop - myBottom + '\n');
        var bDockedBottom = ((myTop - stickyWindowBottom) > -3) && ((myTop - stickyWindowBottom) < 5);
        var bDockedTop = ((stickyWindowTop - myBottom) > -3) && ((stickyWindowTop - myBottom) < 5);
        this.bDocked = bDockedBottom || bDockedTop;
        if (this.bDocked) {
            this.stickyWindow.focus();
        } else {
            this.bShouldDockTop = false;
            this.bShouldDockBottom = false;
        }
    }
}


function foxytunesInitMinimode() {

}

var gFoxyTunesMiniMode = new FoxyTunesMiniMode();

addEventListener("load", function(event) { gFoxyTunesMiniMode.init(); }, false);
addEventListener("unload", function(event) { gFoxyTunesMiniMode.unload(); }, false);

