/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function ManualMusicUpdates() {
    this.musicUpdatesService = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxytunesMusicUpdatesService;1"]
                                              .getService().wrappedJSObject; 

    this.lastFreeText = null;
    this.lastTemplateText = {};

    this.window_params = null;
    if (window.arguments.length > 0) {
         this.window_params = window.arguments[0];
    }

    this.lastTemplateId = null;

    if (this.window_params == "tell-your-friends") {
        this.currentTrackData = null;
        this.bNothingIsPlaying = true;
    } else {
    this.currentTrackData = this.musicUpdatesService.getCurrentTrackData(); 
    this.bNothingIsPlaying = ((this.currentTrackData.originalTitle == null) ||
                              (this.currentTrackData.originalTitle == ""));
    }

    var me = this;
    this.musicUpdatesConfiguration = new MusicUpdatesConfiguration();
    this.musicUpdatesConfiguration.onCheckboxToggle( function() { me.updateManualModeDialogState(); } );

    this.bCloseWindowAfterSettings = false;
}

ManualMusicUpdates.prototype = {
    init: function() {
        this.mainText = document.getElementById('twittytunes-main-text');
        this.mainTextLink = document.getElementById('twittytunes-main-text-link');
        this.charsAvail = document.getElementById('chars-available');
        this.charsAvailBox = document.getElementById('chars-available-box');
        this.templateMenu = document.getElementById('twittytunes-update-template');
        this.autoUpdatesStateLabel = document.getElementById('auto-updates-state');

        var updateTemplateId = foxytunesReadSinglePreference('music_updates.manual.template', 'listeningTo');
        if (this.bNothingIsPlaying) {
            updateTemplateId = 'freeText';
            this.templateMenu.setAttribute('disabled', 'true');
            document.getElementById('url-hbox').setAttribute('hidden', 'true');
        }
        var templateMenuitem = document.getElementById(updateTemplateId);
        this.templateMenu.selectedItem = templateMenuitem;

        this.musicUpdatesConfiguration.populateConsumers("manual"); 
        this.updateTemplate(updateTemplateId);

        if (this.musicUpdatesService.getNumberOfActivatedConsumers("manual") == 0) {
            this.collapseManualServicePanel(false);
        } else {
            var collapseManualServicePanelState = foxytunesReadSinglePreference('music_updates.manual.service_panel.collapse', 'false');
            this.collapseManualServicePanel((collapseManualServicePanelState == 'true'));
        }

        this.setAutoUpdatesStateLabel(); 

        setTimeout( function() { window.sizeToContent(); }, 50);
    },

    onSend: function() {
        var formattedText = '';
        var bHasLink = (this.mainTextLink.value != "");
        if (this.templateMenu.selectedItem.id == 'freeText') {
            if (!bHasLink) {
                formattedText = this.mainText.value;
            } else {
                formattedText = '<a href="' + this.mainTextLink.value + '">' + this.mainText.value + '</a>';
            }
        } else {
            formattedText = this.mainText.getAttribute('ft-formatted-text');
        }
            
        // Currently using hard-coded html format
        this.musicUpdatesService.sendMusicUpdate(this.currentTrackData, this.mainText.value, formattedText, (bHasLink ? this.mainTextLink.value : null));

        // Save configuration
        foxytunesWriteSinglePreference('music_updates.manual.template', 
                       this.templateMenu.selectedItem.id);
        var bCollapse = document.getElementById('send-updates-to-caption').getAttribute('ft-collapsed');
        foxytunesWriteSinglePreference('music_updates.manual.service_panel.collapse', bCollapse); 

        return true;
    },

    updateTemplate: function(updateTemplateId) {
        var trackLink = this.bNothingIsPlaying ? '' : foxytunesGetTrackDataLink(this.currentTrackData);
        this.mainTextLink.value = this.bNothingIsPlaying ? '' : trackLink;

        if (this.lastTemplateId) {
            this.lastTemplateText[this.lastTemplateId] = this.mainText.value;
        }

        if (this.lastTemplateText[updateTemplateId]) {
            this.mainText.value = this.lastTemplateText[updateTemplateId];
        } else {
        if (updateTemplateId == 'freeText') {
                var text = '';
                if (this.window_params == 'tell-your-friends') {
                    text = foxytunesGetLocaleString('twittyTunesTellYourFriends', [ 'http://www.foxytunes.com/firefox/download/' ]);
                } else if (!this.bNothingIsPlaying) {
                    text =  gFoxyTunesSanitizer.replaceTriangularBrackets(this.currentTrackData.originalTitle);
                }
                this.mainText.value = text;
            } else {
            // Curentlly hard-coded to %artist-track%
            var updateTemplate = foxytunesGetLocaleString(updateTemplateId + 'Template', [ '%artist-track%' ]);

            var updateText = this.musicUpdatesService.getMusicUpdateText(this.currentTrackData, trackLink, 'html', updateTemplate);
            this.mainText.value = updateText[0];
            this.mainText.setAttribute('ft-formatted-text', updateText[1]);
            }
        }

        this.updateManualModeDialogState();
        this.updateAvailableChars();
        this.lastTemplateId = updateTemplateId;
    },

    updateAvailableChars: function() {
        if (this.musicUpdatesService.getNumberOfActivatedConsumers("manual") == 0) {
            this.charsAvailBox.setAttribute('hidden', 'true');
            return;
        }

        this.maxAvailChars = this.musicUpdatesService.getCurrentMaxAllowedCharacters();
        this.charsAvailBox.setAttribute('hidden', (this.maxAvailChars < 0));

        if (this.maxAvailChars < 0) return;

        var val = this.maxAvailChars - this.mainText.value.length;
        linkLength = this.mainTextLink.value.length;
        // Currently only true for Twitter (url's are limited by 40 cause they use tinyurl)
        if (linkLength > 40) linkLength = 40;
        val -= linkLength;

        // -1 because of the space before the URL
        if (linkLength > 0) {
            val -= 1;
        }

        if (val < 0) {
            this.charsAvail.setAttribute('no-chars-left', 'true');
            this.disableDialogButton('accept', true);
        } else {
            this.charsAvail.setAttribute('no-chars-left', 'false');
            this.disableDialogButton('accept', false);
        }

        this.charsAvail.setAttribute('value', val);
    },

    updateManualModeDialogState: function() {
        this.disableDialogButton('accept', (this.musicUpdatesService.getNumberOfActivatedConsumers("manual") == 0));
        this.updateAvailableChars();
    },

    disableDialogButton: function(buttonId, bDisable) {
        var dialog = document.documentElement;
        if (bDisable) {
            dialog.getButton(buttonId).setAttribute('disabled', 'true');
        } else {
            dialog.getButton(buttonId).setAttribute('disabled', 'false');
        }
    },

    toggleManualServicePanelCollapsing: function() {
        var panel = document.getElementById('manual-consumers-grid');
        var bHidden = (panel.getAttribute('hidden') == 'true');
        this.collapseManualServicePanel(!bHidden);
    },

    collapseManualServicePanel: function(bCollapse) {
        document.getElementById('manual-consumers-grid').setAttribute('hidden', bCollapse);
        document.getElementById('send-updates-to-caption').setAttribute('ft-collapsed', bCollapse);
         // Fix xul bug - sizeToContent doesn't always work if xul thinks that no change is needed
        window.outerHeight = window.outerHeight+1;
        window.sizeToContent();
    },

    setAutoUpdatesStateLabel: function() {
        var autoUpdatesStateOnLabel = foxytunesGetLocaleString('twittytunesAutoUpdatesStateOn');
        var autoUpdatesStateOffLabel = foxytunesGetLocaleString('twittytunesAutoUpdatesStateOff');

        var autoUpdatesStateLabel = (this.musicUpdatesService.getNumberOfActivatedConsumers("auto") == 0) ? autoUpdatesStateOffLabel : autoUpdatesStateOnLabel;
        this.autoUpdatesStateLabel.setAttribute('value', autoUpdatesStateLabel);
    },

    openSettingsDialog: function() {
        window.openDialog('chrome://foxytunes/content/music-updates/musicUpdatesConfiguration.xul', '',
						  'chrome,centerscreen, modal, toolbar');
        if (this.bCloseWindowAfterSettings) close();
        this.setAutoUpdatesStateLabel();
    },

    // Use to enable the settings window to close the manual updates window
    // in case we want to close the main window after settings is closed 
    closeWindowAfterSettings: function() {
        this.bCloseWindowAfterSettings = true;
    }
}
