/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function MusicUpdatesConfiguration() {
    this.musicUpdatesService = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxytunesMusicUpdatesService;1"]
                                              .getService().wrappedJSObject; 
    this.consumersDetails = this.musicUpdatesService.getConsumersDetails();
    this.onCheckboxToggleCB = null;
}

MusicUpdatesConfiguration.prototype = {

    onCheckboxToggle: function(callback) {
        this.onCheckboxToggleCB = callback;
    },

    toggleUpdatesActivation: function(event, mode) {
        var target = event.target;
        var consumerName = target.getAttribute('consumername');
        var bChecked = (target.getAttribute('checked') == 'true');
        var bNeedAuth = (target.getAttribute('ftneedauth') == 'true');

        if (bChecked && bNeedAuth && 
            !this.musicUpdatesService.isConsumerHasAuthDetails(consumerName)) {
            bChecked = this.openUserAuthDialog(consumerName, mode);
        }
        target.setAttribute('checked', bChecked);

        if (bNeedAuth) {
            this.updateConsumerAuthDetails(consumerName, mode);
        }

        this.musicUpdatesService.setConumerUpdatesActiveState(consumerName, mode, bChecked);

        if (bChecked) {
            window.sizeToContent();
        }

        if (this.onCheckboxToggleCB) this.onCheckboxToggleCB();
    }, 

    openUserAuthDialog :function(consumerName, mode) {
        var res = false;
        var me = this;
        var authCallback = function(username, password) {
            me.musicUpdatesService.removeConsumerAuthDetails(consumerName);
            me.musicUpdatesService.setConsumerAuthDetails(consumerName, username, password);
            me.musicUpdatesService.sendAuthDetailsToConsumer(consumerName);
            me.updateConsumerAuthDetails(consumerName, mode);
            res = true;
        };

        var params = {};
        params.consumerName = consumerName;
        params.callback = authCallback;

        var auth = this.musicUpdatesService.getConsumerAuthDetails(consumerName);  
        params.username = auth ? auth[0] : '';
        params.password = auth ? auth[1] : '';

        var win = this.consumersDetails[consumerName].authWindowURL;
        if (!win) return false;
        window.openDialog(win, "_blank", "centerscreen,chrome,modal", params);
        return res;
    },

    updateConsumerAuthDetails:function (consumerName, mode) {
        var authDetails = this.musicUpdatesService.getConsumerAuthDetails(consumerName);

        var consumerCheckbox = document.getElementById(consumerName + '-checkbox-' + mode);
        if (!consumerCheckbox) return;

        var consumerRow = document.getElementById(consumerName + '-row-' + mode);
        var usernameLabel = document.getElementById(consumerName + '-username-' + mode);
        var changeLabel = document.getElementById(consumerName + '-change-' + mode);
        var bChecked = (consumerCheckbox.getAttribute('checked') == 'true');

        if (authDetails && authDetails[0]) {
            usernameLabel.setAttribute('value', authDetails[0]);
        }

        usernameLabel.setAttribute('hidden', !bChecked);
        changeLabel.setAttribute('hidden', !bChecked);
    },

    populateConsumers: function(mode) {
        var me = this;
        var rows = document.getElementById(mode + '-consumers-rows');
        var newElement;
        var consumersCheckboxes = new Array();
        for (var consumerName in this.consumersDetails) {
            if ((mode == "auto") && !this.musicUpdatesService.isConsumerSupportAutoUpdates(consumerName)) continue;
            if ((mode == "manual") && !this.musicUpdatesService.isConsumerSupportManualUpdates(consumerName)) continue;
                
            var consumer = this.consumersDetails[consumerName];
            newElement = document.createElement('checkbox');
            newElement.consumerName = consumer.readableName;
            // Ugly patch - use two different names for the same consumer
            var consumerNameLabel = newElement.consumerName;
            if ((mode == "manual") && (consumer.readableNameManual)) {
                consumerNameLabel = consumer.readableNameManual
            }
            if ((mode == "auto") && (consumer.readableNameAuto)) {
                consumerNameLabel = consumer.readableNameAuto
            }
            newElement.setAttribute('id', consumerName + '-checkbox-' + mode); 
            newElement.setAttribute('label', consumerNameLabel); 
            newElement.setAttribute('consumername', consumerName);
            newElement.setAttribute('tooltiptext', newElement.consumerName);
            newElement.setAttribute('value', consumer.classContract);
            newElement.setAttribute('ftneedauth', consumer.needAuth);
            newElement.setAttribute('class', 'music-updates-consumer-item'); 
            newElement.addEventListener('command', function(event) { me.toggleUpdatesActivation(event, mode); }, false);
          
            if (consumer.bIsUpdatesActivated[mode]) {
                newElement.setAttribute('checked', 'true');
            }

            consumersCheckboxes.push(newElement);
        }

        var sortFunc = function (a, b) { 	
            a = a.consumerName.toLowerCase(); 
		    b = b.consumerName.toLowerCase(); 
		    if ( a < b ) return -1;
		    if ( a > b ) return 1;
		    return 0;   
        }

        consumersCheckboxes.sort(sortFunc);

        for (var i=0; i<consumersCheckboxes.length; i++) {
            var element = consumersCheckboxes[i];
            var consumerName = element.getAttribute('consumername');
            var newRow = document.createElement('row');
            newRow.setAttribute('id', consumerName + '-row-' + mode);
            newRow.setAttribute('class', 'consumer-details-row');
            newRow.setAttribute('ft-consumer-name', consumerName);
            newRow.appendChild(element);
            rows.appendChild(newRow);

            var bNeedAuth = (element.getAttribute('ftneedauth') == 'true');
            if (bNeedAuth) {
                var usernameLabel = document.createElement('label');
                usernameLabel.setAttribute('id', consumerName + '-username-' + mode);
                usernameLabel.setAttribute('class', 'username-label');
                usernameLabel.setAttribute('hidden', 'true');
                usernameLabel.setAttribute('crop', 'end');
                newRow.appendChild(usernameLabel);

                var changeLabel = document.createElement('label');
                changeLabel.setAttribute('id', consumerName + '-change-' + mode);
                changeLabel.setAttribute('value', foxytunesGetLocaleString('twittytunesChangeLabel'));
                changeLabel.setAttribute('class', 'url');
                changeLabel.setAttribute('hidden', 'true');
                changeLabel.onclick = function(name) {
                    return function() { 
                        me.openUserAuthDialog(name, mode);
                    };
                }(consumerName);
                newRow.appendChild(changeLabel);

                this.updateConsumerAuthDetails(consumerName, mode);
           }
        }
    }
}
