/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function FoxytunesYMPPageAnalyzer() {
    var me = this;
    this.domContentLoadedCallback =  function(event) { me.onDOMContentLoad(event); }

     var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/htmlTooltip.js");
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
    }

    var tooltipText = foxytunesGetLocaleString('ympFirstTimeBalloonText');
    this.tooltip = new FoxyTunesHTMLTooltip();
    this.tooltipInfo = new FoxyTunesTooltipInfo();
    this.tooltipInfo.text = tooltipText;
    this.tooltipInfo.position = 'top-left';
    this.tooltipInfo.showOnce = true;
    this.tooltipInfo.persistOnRefersh = true;
    this.tooltipInfo.learnMoreLink = 'http://www.foxytunes.com/mediaplayers/yahoo-media-player';
    this.tooltipInfo.id = 'foxytunes-goose-tooltip';

    this.MAX_LINKS_NUM = 1000;

    this.mp3LinkRegExp = new RegExp('^[^\?]+\\.mp3$');
}

FoxytunesYMPPageAnalyzer.prototype = {
    init: function() {
        var appcontent = document.getElementById("appcontent");   
        if (appcontent) {
            appcontent.addEventListener("DOMContentLoaded", this.domContentLoadedCallback, true);
        }
    },

    unInit: function() {
        var appcontent = document.getElementById("appcontent");   
        if (appcontent) {
            appcontent.removeEventListener("DOMContentLoaded", this.domContentLoadedCallback, true);
        }
    },

    isDocContainMedia: function(doc) {
        var links = doc.getElementsByTagName('a');
        for (var i=0; i<Math.min(links.length, this.MAX_LINKS_NUM); i++) {
            var currentLink = links[i];
            if (!currentLink.hasAttribute('href')) continue;
            var href = currentLink.getAttribute('href');
            if (href.match(this.mp3LinkRegExp)) {
                return true;
            }
        }
        return false;
    },

    isRTLPage: function(doc) {
        var direction = 'ltr';
        try {
            var body = doc.getElementsByTagName('body')[0];
            direction = doc.defaultView.getComputedStyle(body, "").getPropertyValue("direction");
        } catch (e) {};
        return (direction == 'rtl') ? true : false;
    },

    onYMPInit: function(doc, numOfTries) {
        if (numOfTries > 25) return;
        var elem1 = doc.getElementById('ymwp-player-overlay');
        var elem2 = doc.getElementById('ymp-player');
        var elem = elem1 ? elem1 : elem2;
        if (!elem) { 
            var me = this;
            setTimeout(function() { me.onYMPInit(doc, numOfTries+1); }, 200);
            return;
        }
        this.tooltip.showTooltip(doc, elem, this.tooltipInfo);
    },

    onDOMContentLoad: function(event) {
        if (!gFoxytunesPreferenceManager.getBoolPref('add_media_player_to_page', 'true')) return;

        if (event.originalTarget.nodeName != "#document") { return; }
        var doc = event.originalTarget || document;
        if (!doc) return;
        if (!doc.location || !doc.location.protocol) return;
        // Avoid adding Goose to https pages
        if (doc.location.protocol.indexOf('https') > -1) return;

        // Don't add YMP to rtl pages
        if (this.isRTLPage(doc)) return;
        // Only add YMP if there are media links in the page
        if (!this.isDocContainMedia(doc)) return;

        var asyncLoadScript = '';
        asyncLoadScript += 'if (YAHOO.music) {';
        asyncLoadScript += '  setTimeout(function(){YAHOO.music.WebPlayer.asyncLoadPlayer()}, 100)';
        asyncLoadScript += '} else {';
        asyncLoadScript += ' setTimeout(function(){YAHOO.mediaplayer.loadPlayerScript()}, 100)';
        asyncLoadScript += '}';

        var ympScript = '';
        ympScript += 'if (typeof ympparams == "undefined") {' ;
        ympScript += '  var ympparams={};';
        ympScript += '}';
        ympScript += 'if (typeof YMPParams == "undefined") {' ;
        ympScript += '  var YMPParams={};';
        ympScript += '}';
        ympScript += '(function (){';
        ympScript += 'var scriptTags = document.getElementsByTagName("script");';
        ympScript += 'var bYMPAlreadyInPage = false;';
        ympScript += 'for (var i=0; i<Math.min(scriptTags.length,' + this.MAX_LINKS_NUM + '); i++) {';
        ympScript +=    'if (!scriptTags[i].hasAttribute("src")) continue;';
        ympScript +=    'if (!scriptTags[i].getAttribute("src").match(/mediaplayer.yahoo.com/) && !scriptTags[i].getAttribute("src").match(/webplayer.yahooapis.com/)) continue;';
        ympScript +=    'bYMPAlreadyInPage = true;';
        ympScript +=    'break;';
        ympScript += '}';
        ympScript += 'if (!bYMPAlreadyInPage && !(YAHOO && YAHOO.MediaPlayer)) {';
        ympScript +=   'ympparams["injected-by"] = \'foxytunes-ff\';';
        ympScript +=   'YMPParams["injected-by"] = \'foxytunes-ff\';';
        ympScript +=   'var script = document.createElement("script");';
        ympScript +=   'script.setAttribute("src", "http://mediaplayer.yahoo.com/js");';
        ympScript +=   'script.setAttribute("type", "text/javascript");';
        ympScript +=   'var asyncLoadScript = document.createElement("script");';
        ympScript +=   'asyncLoadScript.setAttribute("type", "text/javascript");';
        ympScript +=   'asyncLoadScript.innerHTML = "' + asyncLoadScript + '";';
        ympScript +=   'document.getElementsByTagName("head")[0].appendChild(script);';
        ympScript +=   'document.getElementsByTagName("head")[0].appendChild(asyncLoadScript);';
        ympScript +=   'var event = document.createEvent("Events");';
        ympScript +=   'event.initEvent("FoxyTunesYMPInit", true, false);';
        ympScript +=   'document.dispatchEvent(event);';
        ympScript += '}';
        ympScript += '})();';

        var me = this;
        doc.addEventListener("FoxyTunesYMPInit", function(event) { me.onYMPInit(doc, 1); }, true);

        var script = doc.createElement('script');
        script.setAttribute('type', 'text/javascript');
        script.innerHTML = ympScript;
        doc.getElementsByTagName('head')[0].appendChild(script);
    }
}



