/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function FTPlayerConfig() {
    this._prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);
    // Load utils if needed
    if (typeof FoxyTunesLoginManager == "undefined") {
        var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
        if (scriptLoader) {
            scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
        }
    }
}

FTPlayerConfig.prototype = {
    getReadableName : function(playerName) {
        var branch = this._prefs.getBranch("foxytunes.player_readable_name.");
        var name = '';
        try {
            name = branch.getCharPref(playerName);           
        } catch(e) {}
        return name;
    },

    needsConfig : function(playerName) {
        var branch = this._prefs.getBranch("foxytunes.player_needs_config.");
        var needs = false;
        try {
            needs = branch.getBoolPref(playerName);
        } catch(e) {}
        return needs;             
    },

    wasConfigured : function(playerName) {
      var branch = this._prefs.getBranch("foxytunes.player_was_configured.");
      var wasConfigured = false;
      try {
        wasConfigured = branch.getBoolPref(playerName);
      } catch(e) {}
      return wasConfigured;
    },

    setWasConfigured : function(playerName) {
      var branch = this._prefs.getBranch("foxytunes.player_was_configured.");
      branch.setBoolPref(playerName, true);
    },

    getConfigURL : function(playerName) {
      var branch = this._prefs.getBranch("foxytunes.player_config_url.");
      var url = '';
      try {
          url = branch.getCharPref(playerName);
      } catch(e) {}
      return url;
    },

    getSupportedExtensions : function(playerName) {
      var branch = this._prefs.getBranch("foxytunes.supported_extensions.");
      var ext = '';
      try {
          ext = branch.getCharPref(playerName);
      } catch(e) {}
      return ext;
    },

    getSupportsSubscribeToPodcast : function(playerName) {
      var branch = this._prefs.getBranch("foxytunes.supports_podcasts.");
      var supports = false;
      try {
          supports = branch.getBoolPref(playerName);
      } catch(e) {}
      return supports;
    },

    getSupportsDynamicMp3Stream : function(playerName) {
      var supportedExtensionsRegExp = gFTPlayerConfig.getSupportedExtensions(playerName);
      if (supportedExtensionsRegExp && supportedExtensionsRegExp == "PLAY_URI_NOT_SUPPORTED") {	
        return false;
	  }
      var branch = this._prefs.getBranch("foxytunes.supports_dynamic_mp3_stream.");
      var supports = true;
      try {
          supports = branch.getBoolPref(playerName);
      } catch(e) {}
      return supports;
    },

    getSupportPlayFile : function(playerName) {
      var branch = this._prefs.getBranch("foxytunes.supports_play_file.");
      var supports = false;
      try {
          supports = branch.getBoolPref(playerName);
      } catch(e) {}
      return supports;
    },

    openConfigDialog : function(playerName) {
      var url = this.getConfigURL(playerName);
      if (!url) return;

      window.openDialog(url, "_blank", "centerscreen,chrome,modal,resizable");
    },

    getPlayerConfigVariable : function(playerName, varName) {
         var branch = this._prefs.getBranch("foxytunes.player_config." + playerName + ".");
         var val = '';
         try {
            val = branch.getCharPref(varName);
         } catch(e) {}
         return val;
    },

    setPlayerConfigVariable : function(playerName, varName, varValue) {
         var branch = this._prefs.getBranch("foxytunes.player_config." + playerName + ".");
         branch.setCharPref(varName, varValue);
    },

    deletePlayerConfigVariables: function(playerName) {
        var branch = this._prefs.getBranch("foxytunes.player_config.");
        branch.deleteBranch(playerName);
    },

    clearPlayerConfigVariable: function(playerName, varName) {
        var branch = this._prefs.getBranch("foxytunes.player_config." + playerName + ".");
        branch.clearUserPref(varName);
    },

    getPlayerLoginDetails: function(playerName) {
        if (typeof this._loginManager == "undefined") this._loginManager = new FoxyTunesLoginManager();
        return this._loginManager.getLogin('http://' + playerName.toLowerCase() + '.players.foxytunes.com');
    },

    setPlayerLoginDetails: function(playerName, userName, password) {
        if (typeof this._loginManager == "undefined") this._loginManager = new FoxyTunesLoginManager();
        this._loginManager.addLogin('http://' + playerName.toLowerCase() + '.players.foxytunes.com', userName, password);
    },

    getPlayerConfigCommandLine : function(playerName) {
         var branch = this._prefs.getBranch("foxytunes.player_config." + playerName + ".");
         var children = branch.getChildList("", {});
         var commandLine = '';
         for (var i=0; i < children.length; i++) {
                var variable = children[i];
                var val = this.getPlayerConfigVariable(playerName, variable);              
                if (val) {
                    commandLine += ' -' + variable + ' ' + val;
                }
         }
         var loginDetails = this.getPlayerLoginDetails(playerName);
         if (loginDetails) {
             commandLine += ' -username ' + loginDetails.username + ' -password ' + loginDetails.password;
         }
         return commandLine;
    }
}


var gFTPlayerConfig = new FTPlayerConfig();

