/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function foxytunesInitMusicSearchMenuPopup(event) {
    // Prevent the function from being called
    // from sub-menus popups
    if (event.target != event.originalTarget) {
        return;
    }
    event.target.trackData = foxytunesGetCurrentTrackData();
}

function foxytunesExecuteSearch(event) {
    var target = event.currentTarget;
    var trackData = target.trackData;
    var searchExecuter = new FoxyTunesSearchExecuter(trackData);
    searchExecuter.executeSearch(event.originalTarget);
}

function foxytunescontextMenuExecuteSearch(event) {
    var target = event.target;

    // If no selection use the foxytunesExecuteSearch
    if (!target.selection) {
        foxytunesExecuteSearch(event);
        return;
    }

    // find the current search type
    var element = event.originalTarget;
    while (!element.hasAttribute('searchtype')) {
        element = element.parentNode;
    }

    if (!element) {
        dump("Can't find searchtype\n");
        return;
    }

    // build custom trackData with selection as the
    // relevant search type (e.g. artist=selection)
    var trackData = new Object();
    switch (element.getAttribute('searchtype')) {
        case 'artist':
            trackData.artist = target.selection;
            break;
        case 'album':
            trackData.album = target.selection;
            break;
        case 'track':
            trackData.title = target.selection;
            break;
        case 'planet':
        case 'info':
        case 'web':
        case 'videos':
        case 'tickets':
        case 'images':
        case 'general':
        case 'lyrics':
            trackData.artist = target.selection
            trackData.album = target.selection
            trackData.title = target.selection
            break;
        default:
            dump('Unknown search type');
            return;
    } 

    var searchExecuter = new FoxyTunesSearchExecuter(trackData);
    searchExecuter.executeSearch(event.originalTarget);
}

function FoxyTunesSearchEngineUtils() {
    this.rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                            .getService(Components.interfaces.nsIRDFService);
    var dataSourcePath = this.getCurrentDataSourceFile();
    this.dataSource = this.rdfService.GetDataSource(dataSourcePath);
}

FoxyTunesSearchEngineUtils.prototype = {
    getCurrentDataSourceFile: function() {
        var firefoxLocale = foxytunesGetFirefoxLocale();
        var searchLocale = foxytunesReadSinglePreference("search.locale", "default");
        var locale = (searchLocale != "default") ? searchLocale : firefoxLocale;
        var rdfFilePath = 'chrome://foxytunes/content/search/config/foxytunesSearchEngines.' + locale + '.rdf';

        FTJSLib.include(FTJSLib.jslib_chromefile);
        var cf = new FTJSLib.ChromeFile(rdfFilePath);
        if (!(cf.exists())) { 
            rdfFilePath = 'chrome://foxytunes/content/search/config/foxytunesSearchEngines.en-US.rdf';
        }

        return rdfFilePath;
    },

    getRDFLiteral : function(resource, predicate) {
        var literal = this.dataSource.GetTarget(resource, predicate, true);
        if (!literal) return null;
        literal.QueryInterface(Components.interfaces.nsIRDFLiteral);
        return literal.Value;
    },

    getSearchEngineFromDescription: function(desc) {
                searchEngine = new Object();
                var urlResource = this.rdfService.GetResource("http://www.foxytunes.org/rdf#url");
        searchEngine.url = this.getRDFLiteral(desc, urlResource);
        searchEngine.name = this.getRDFLiteral(desc, this.rdfService.GetResource("http://www.foxytunes.org/rdf#name"));
                var processResource = this.rdfService.GetResource("http://www.foxytunes.org/rdf#process");
        if (this.dataSource.hasArcOut(desc, processResource)) {
            searchEngine.process = this.getRDFLiteral(desc, processResource);
                }

                var encodingResource = this.rdfService.GetResource("http://www.foxytunes.org/rdf#encoding");
        if (this.dataSource.hasArcOut(desc, encodingResource)) {
            searchEngine.encoding = this.getRDFLiteral(desc, encodingResource);
                }

        return searchEngine;
    },

    getDefaultSearchEngineByType: function(type) {
        var searchEngine = null;
        var children = this.dataSource.GetAllResources();
        while (children.hasMoreElements()) {
            var child = children.getNext();
            child.QueryInterface(Components.interfaces.nsIRDFResource);
            var resource = this.rdfService.GetResource("http://www.foxytunes.org/rdf#default");
            var defaultType = this.getRDFLiteral(child, resource);
            if (!defaultType) continue;
            defaultTypeArr = defaultType.split(',');
            for (var i=0; i<defaultTypeArr.length; i++) {
                if (type == defaultTypeArr[i]) {
                    searchEngine = this.getSearchEngineFromDescription(child);         
                    return searchEngine;
                }
            }
        }
        return searchEngine;
    }
                }

function FoxyTunesSearchExecuter(trackData) {
    this.trackData = trackData;

    this.customEngineURL = null;
    // If custom web search URL exists use it instead of the default search engine
    var bUseCustomEngine = foxytunesReadSinglePreference("search.use_custom_web_engine", "");
    if (bUseCustomEngine == 'true') {
        var customURL = foxytunesReadSinglePreference("search.custom_web_engine_URL", "");

        // our internal general search uses '%query%', we expose custom query 
        // as '%s' - to be compatible with Opera and Firefox keyword search conventions
        customURL = customURL.replace(/%s/g,  '%query%');
        this.customEngineURL = customURL;
                }
        }

FoxyTunesSearchExecuter.prototype = {

    getSearchURL : function(menuitem) {
        var searchParameters = new Object();
        var searchType = menuitem.getAttribute('searchtype');

        if (searchType && !menuitem.getAttribute('url')) {
            var engineUtils = new FoxyTunesSearchEngineUtils();
            searchParameters = engineUtils.getDefaultSearchEngineByType(searchType);
        } else {
	        searchParameters.url = menuitem.getAttribute('url');
	        searchParameters.process = menuitem.getAttribute('process');
            searchParameters.encoding = menuitem.getAttribute('encoding');
        }

        if (((searchType == 'web') || (searchType == 'lyrics')) &&
              this.customEngineURL) {
            searchParameters.url = this.customEngineURL;
            delete searchParameters.process;
            delete searchParameters.encoding; 
        }
	    
        searchParameters.query = menuitem.getAttribute('query');
        var searchEngine = new FoxyTunesSearchEngine(searchParameters,this.trackData);
	    var url = searchEngine.getURL();
        return url;
    },

    executeSearch : function(target) {
        var urlList = new Array();
        var url = null;

        if (target.hasAttribute('openintabs')) {
            var menupopup = target.parentNode;
            for (var iMenuItem=0; iMenuItem < menupopup.childNodes.length; iMenuItem++) {
                var menuitem = menupopup.childNodes[iMenuItem];
                if (menuitem.hasAttribute('url') && (menuitem != target)) {
                    url = this.getSearchURL(menuitem); 
                    if (url) {
                        urlList.push(url);
                    }
                }
            }
        } else {
            url = this.getSearchURL(target);
            if (url) {
                urlList.push(url);
            }
        }

	    foxytunesOpenInTabs(urlList);   
    }
}

function FoxyTunesSearchTermsBuilder() {
}

FoxyTunesSearchTermsBuilder.prototype = {

    artistQuoted : null,
    artistQuotedIfNeeded : null,

    albumQuoted : null,
    albumQuotedIfNeeded : null,

    trackQuoted : null,
    trackQuotedIfNeeded : null,

    discographySearch : null,
    biographySearch : null,

    selectionQuoted : null,
    selectionQuotedIfNeeded : null,
    selectionLyricsSearch : null,
    selectionChordsSearch : null,
    selectionTabsSearch : null,

    lyricsSearch : null,
    chordsSearch : null,
    tabsSearch : null,

    generalSearchURL : null,

    set trackData(trackData) {
        this._trackData = trackData;
        this.hasArtist = (trackData && trackData.artist) ? true : false;
        this.hasAlbum  = (trackData && trackData.album)  ? true : false;
        this.hasTrack  = (trackData && trackData.title)  ? true : false;
        this.setSearchTerms();
    },

    get trackData() {
        return this._trackData;
    },

    set selection(selection) {
        this._selection = selection;
        this.setSelectionSearchTerms();
    },

    get selection() {
        return this._selection;
    },

    setSearchTerms : function() {

        if (!this._trackData) {
            return;
        }

        var trackData = this._trackData;
	    if (this.hasArtist) {
            if (FoxyTunesSearchEngine.StringProcessors.needQuoting(trackData.artist)) {
	            this.artistQuoted = FoxyTunesSearchEngine.StringProcessors.quote(trackData.artist);
            } else {
                this.artistQuoted = null;
            }
		    this.artistQuotedIfNeeded = FoxyTunesSearchEngine.StringProcessors.quoteIfNeeded(trackData.artist);
		    this.biographySearch = this.artistQuotedIfNeeded + ' ' + foxytunesGetLocaleString('biography');
		    this.discographySearch = this.artistQuotedIfNeeded + ' ' + foxytunesGetLocaleString('discography');
    	}
	    if (this.hasAlbum) {
            if (FoxyTunesSearchEngine.StringProcessors.needQuoting(trackData.album)) {
                this.albumQuoted = FoxyTunesSearchEngine.StringProcessors.quote(trackData.album);
            } else {
                this.albumQuoted = null;
            }
    		this.albumQuotedIfNeeded = FoxyTunesSearchEngine.StringProcessors.quoteIfNeeded(trackData.album);
        }	    
	    if (this.hasTrack) {
            if (FoxyTunesSearchEngine.StringProcessors.needQuoting(trackData.title)) {
                this.trackQuoted = FoxyTunesSearchEngine.StringProcessors.quote(trackData.title);	
            } else {
                this.trackQuoted = null;
            }
		    this.trackQuotedIfNeeded = FoxyTunesSearchEngine.StringProcessors.quoteIfNeeded(trackData.title);
	    }    
	    if (this.hasArtist && this.hasTrack) {
	        var artistPlusTrack = this.artistQuotedIfNeeded + ' ' + this.trackQuotedIfNeeded;
	        this.lyricsSearch = artistPlusTrack + ' ' + foxytunesGetLocaleString('lyrics');
	        this.chordsSearch = artistPlusTrack + ' ' + foxytunesGetLocaleString('chords');
	        this.tabsSearch   = artistPlusTrack + ' ' + foxytunesGetLocaleString('tabs');
	    } else {
		    if (this.hasTrack) {
		        this.lyricsSearch = this.trackQuotedIfNeeded + ' ' + foxytunesGetLocaleString('lyrics');
		        this.chordsSearch = this.trackQuotedIfNeeded + ' ' + foxytunesGetLocaleString('chords');
		        this.tabsSearch   = this.trackQuotedIfNeeded + ' ' + foxytunesGetLocaleString('tabs');
		    }
	    }
    },

    setSelectionSearchTerms : function() {
        if (FoxyTunesSearchEngine.StringProcessors.needQuoting(this.selection)) {
            this.selectionQuoted = FoxyTunesSearchEngine.StringProcessors.quote(this.selection);
        } else {
            this.selectionQuoted = null;
        }
        this.selectionQuotedIfNeeded = FoxyTunesSearchEngine.StringProcessors.quoteIfNeeded(this.selection);
        this.selectionLyricsSearch = this.selectionQuotedIfNeeded + ' ' + foxytunesGetLocaleString('lyrics');
        this.selectionChordsSearch = this.selectionQuotedIfNeeded + ' ' + foxytunesGetLocaleString('chords');
        this.selectionTabsSearch = this.selectionQuotedIfNeeded + ' ' + foxytunesGetLocaleString('tabs');
    },

    appendHeaderItem : function(menupopup, text) {
        var menuitem;
	    menuitem = document.createElement('menuitem');
        menuitem.setAttribute('label', text);
	    menuitem.setAttribute('tooltiptext', text);
	    menuitem.setAttribute('disabled', 'true');
  	    menuitem.setAttribute('class', 'foxytunes-header-menuitem');
	    menupopup.appendChild(menuitem);
    },

    appendSearchTerm : function(menupopup, text, disabled, isLastWordClipped) {
        if (typeof disabled == 'undefined') {
            disabled = 'false';
        }
        if (typeof isLastWordClipped == 'undefined') {
            isLastWordClipped = 'true';
        }

        var menuitem;
	    menuitem = document.createElement('menuitem');
        if (isLastWordClipped) {
            menuitem.setAttribute('label', foxytunesGetClippedText(text,30));
        } else {
            var splitText = text.split(' ');
            var lastWord = splitText[splitText.length-1];
            splitText.pop();
            var clippedText = splitText.join(' ');
            menuitem.setAttribute('label', foxytunesGetClippedText(clippedText,30 - lastWord.length - 1) + ' ' + lastWord);
        }
	    menuitem.setAttribute('tooltiptext', text);
	    menuitem.setAttribute('query', text);
	    menuitem.setAttribute('searchtype', 'web');
	    menuitem.setAttribute('disabled', disabled);
	    menupopup.appendChild(menuitem);
    },

    appendSeparator : function(menupopup) {
        var menuitem = document.createElement('menuseparator');
	    menupopup.appendChild(menuitem);
    },

    appendArtistSearchTerms : function(menupopup) {
        if (!this.hasArtist) {
            return;
        }

	    this.appendHeaderItem(menupopup, foxytunesGetLocaleString('artistColon'));
	    this.appendSearchTerm(menupopup, this._trackData.artist);
	    if (this.artistQuoted) {
	        this.appendSearchTerm(menupopup, this.artistQuoted);
	    }
	    this.appendSearchTerm(menupopup, FoxyTunesSearchEngine.StringProcessors.tagify(this._trackData.artist));
	    this.appendSearchTerm(menupopup, this.biographySearch);
	    this.appendSearchTerm(menupopup, this.discographySearch);
    },

    appendAlbumSearchTerms : function(menupopup) {
        if (!this.hasAlbum) {
            return;
        }

	    this.appendHeaderItem(menupopup, foxytunesGetLocaleString('albumColon'));
	    this.appendSearchTerm(menupopup, this._trackData.album);

	    if (this.albumQuoted) {
	        this.appendSearchTerm(menupopup, this.albumQuoted);
	    }
        
        if (this.hasArtist && this.hasAlbum) {
	        this.appendSearchTerm(menupopup, this.artistQuotedIfNeeded + ' ' + this.albumQuotedIfNeeded);
	    }	
    },

    appendTrackSearchTerms : function(menupopup) {
        if (!this.hasTrack) {
            return;
        }

	    this.appendHeaderItem(menupopup, foxytunesGetLocaleString('trackColon'));
	    this.appendSearchTerm(menupopup, this._trackData.title);
	    if (this.trackQuoted) {
	        this.appendSearchTerm(menupopup, this.trackQuoted);
	    }

        if (this.hasArtist && this.hasTrack) {  
	        this.appendSearchTerm(menupopup, this.artistQuotedIfNeeded + ' ' + this.trackQuotedIfNeeded);
	    }
    },

    appendGeneralSearchTerms : function(menupopup) {

	    if (this.hasTrack) {  
            this.appendHeaderItem(menupopup, foxytunesGetLocaleString('extrasColon')); 
	        this.appendSearchTerm(menupopup, this.lyricsSearch, false, false);
	        this.appendSearchTerm(menupopup, this.chordsSearch, false, false);
	        this.appendSearchTerm(menupopup, this.tabsSearch, false, false);
	    }
    },

    appendSelectionSearchTerms : function(menupopup) {
        if (!this._selection) {
            return;
        }  
        this.appendHeaderItem(menupopup,foxytunesGetLocaleString('selectionColon'));
        this.appendSearchTerm(menupopup, this._selection);
        if (this.selectionQuoted) {
            this.appendSearchTerm(menupopup, this.selectionQuoted);
        }
        this.appendSeparator(menupopup);
        this.appendHeaderItem(menupopup, foxytunesGetLocaleString('extrasColon')); 
        this.appendSearchTerm(menupopup, this.selectionLyricsSearch, false, false);
        this.appendSearchTerm(menupopup, this.selectionChordsSearch, false, false);
        this.appendSearchTerm(menupopup, this.selectionTabsSearch, false, false);
    },
    
    appendClipboardText : function(menupopup) {
        var clipboardText = foxytunesGetTextFromClipboard();
        clipboardText = foxytinesTrimString(clipboardText);
        this.appendHeaderItem(menupopup, foxytunesGetLocaleString('clipboardColon')); 
        if (clipboardText && (clipboardText.length < 40)) {
            this.appendSearchTerm(menupopup, clipboardText); 
            if (FoxyTunesSearchEngine.StringProcessors.needQuoting(clipboardText)) {
                this.appendSearchTerm(menupopup, FoxyTunesSearchEngine.StringProcessors.quote(clipboardText)); 
            }
        } else {
            this.appendSearchTerm(menupopup, foxytunesGetLocaleString('empty'), 'true'); 
        }
    },

    buildSearchTheWebPopup : function(menupopup) {
		if (this.hasArtist) {
		    this.appendArtistSearchTerms(menupopup);
		    this.appendSeparator(menupopup);
		}
		if (this.hasAlbum) {
		    this.appendAlbumSearchTerms(menupopup);
		    this.appendSeparator(menupopup);
		}
		if (this.hasTrack) {
		    this.appendTrackSearchTerms(menupopup);
		    this.appendSeparator(menupopup);
		    this.appendGeneralSearchTerms(menupopup);
		    this.appendSeparator(menupopup);
		}

        this.appendClipboardText(menupopup);
    },

    getLyricsSearchQuery : function() {
        return this.lyricsSearch
    },

    getArtistQuotedIfNeeded : function () {
        return this.artistQuotedIfNeeded;
    },
    
    buildSelectionSearchTheWebPopup : function(menupopup) {
        this.appendSelectionSearchTerms(menupopup);
        this.appendSeparator(menupopup);
        this.appendClipboardText(menupopup); 
    },

    getSelectionLyricsSearchQuery : function() {
        return this.selectionLyricsSearch;
    },
    
    getSelectionQuotedIfNeeded : function () {
        return this.selectionQuotedIfNeeded;
    }

}

function FoxyTunesSearchEngine(searchParameters, trackData) {
    this.trackData = trackData;
    this.searchParameters = searchParameters;
    this.textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"].
			                    getService(Components.interfaces.nsITextToSubURI);

    if (!this.searchParameters) {
        dump('no search params found!!!\n');
        throw 'no search params found!!!';
    }

    if (!this.trackData) {
        this.trackData = new Object();
    }
    if (!this.trackData.artist) {
        this.trackData.artist = '';
    }
    if (!this.trackData.album) {
        this.trackData.album = '';
    }
    if (!this.trackData.title) {
        this.trackData.title = '';
    }
}

FoxyTunesSearchEngine.StringProcessors = {
    doNothing : function(str) {
	    return str;
	},

	tagify : function(str) {
	    str = str.replace(/ /g, '');
	    return str.toLowerCase();
	},

    quote : function(str) {
        return '"' + str + '"';
    },

    quoteIfMultiWord : function(str) {
        if (str.match(' ')) {
	        return '"' + str + '"';
        } else {
	        return str;
        }
    },

    quoteIfNeeded : function(str) {
        if (this.needQuoting(str)) {
	        return this.quote(str);
        } else {
	        return str;
        }
    },

    urlEncode : function(str) {
        str = str.replace(/ /g, "+"); 
        str = encodeURIComponent(str);
        return str;
    },
    
    iTunesEncode : function(str) {
        str = FoxyTunesSearchEngine.StringProcessors.urlEncode(str);
        // Encode apostrophe (encodeURIComponent ignores it)
        // We need to encode it 2 times otherwise the link doesn't work
        str = str.replace(/'/g, '%2527');
        str = str.replace(/%26/g, '%2526');
        return str;
    },

    eBayEncode : function(str) {
        var  textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"].
			                    getService(Components.interfaces.nsITextToSubURI);
        str = textToSubURI.ConvertAndEscape('ISO-8859-1', str);
        return str;
    },

    ampEncode : function(str) {
        str = str.replace(/&/g, '%26');
        return str;
    },

    normalizeAccents : function(str) {
        var tmpStr = '[' + String.fromCharCode(192,193,194,195,196,197,198,224,225,226,227,228,229,230) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'a');

        tmpStr = '[' + String.fromCharCode(199,231) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'c');

        tmpStr = '[' + String.fromCharCode(200,201,202,203,232,233,234,235) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'e');

        tmpStr = '[' + String.fromCharCode(204,205,206,207,236,237,238,239) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'i');

        tmpStr = '[' + String.fromCharCode(208,240) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'d');

        tmpStr = '[' + String.fromCharCode(209,241) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'n');
 
        tmpStr = '[' + String.fromCharCode(210,211,212,213,214,216,242,243,244,245,246,248) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'o');

        tmpStr = '[' + String.fromCharCode(215,247) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'x');

        tmpStr = '[' + String.fromCharCode(217,218,219,220,249,250,251,252) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'u');

        tmpStr = '[' + String.fromCharCode(221,253,255) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'y');

        tmpStr = '[' + String.fromCharCode(222,254) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'p');

        tmpStr = '[' + String.fromCharCode(223) + ']';
        str = str.replace(new RegExp(tmpStr,'g'),'b');

        return str;
    },

    replaceEmptyWithHypen: function(str) {
        if (str == '') {
            str = '-';
        }
        return str;
    },

    needQuoting : function(str) {
        return str.match(' ');
    }
}

FoxyTunesSearchEngine.prototype = {
    getURL : function() {
        var process = FoxyTunesSearchEngine.StringProcessors.doNothing;
        if (this.searchParameters.process && 
            FoxyTunesSearchEngine.StringProcessors[this.searchParameters.process]) {
                process = FoxyTunesSearchEngine.StringProcessors[this.searchParameters.process];
        }

    	var url = this.searchParameters.url;

        var artist = process(gFoxyTunesSanitizer.replaceTriangularBrackets(this.trackData.artist));
        var album  = process(gFoxyTunesSanitizer.replaceTriangularBrackets(this.trackData.album));
        var title  = process(gFoxyTunesSanitizer.replaceTriangularBrackets(this.trackData.title));
        var query  = process(gFoxyTunesSanitizer.replaceTriangularBrackets(this.searchParameters.query));

        if (!this.searchParameters.encoding) {
    	    artist = encodeURIComponent(artist);
    	    album  = encodeURIComponent(album);
    	    title  = encodeURIComponent(title);
            query  = encodeURIComponent(query);
        } else {
            artist = this.textToSubURI.ConvertAndEscape(this.searchParameters.encoding, artist);
            album  = this.textToSubURI.ConvertAndEscape(this.searchParameters.encoding, album);
            title  = this.textToSubURI.ConvertAndEscape(this.searchParameters.encoding, title);
            query  = this.textToSubURI.ConvertAndEscape(this.searchParameters.encoding, query);
        }

	    url = url.replace(/%artist%/g, artist);
	    url = url.replace(/%album%/g,  album);
	    url = url.replace(/%title%/g,  title);
	    url = url.replace(/%query%/g,  query);
        url = url.replace(/%client%/g, foxytunesGetAppName());
	    url = url.replace(/\%20/g, "+");
	    return url;
    }
}
