/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function FoxytunesSignaturesSiteHandler(handlerData, doc) {
    this.doc = doc;
    this.handlerData = handlerData; 
    this.monitorId = null;
    this.monitorTimeout = 1000;

    // The signature button details defaults
    this.ButtonDetailsDefaults = function() {
        this.node = null;
        this.signatureInsertNode = null;
        this.loc = 'rightbottom';
        this.bAttachToBody = false;
        this.containerXpath = '';
        this.signatureType = '';
        this.xOffset = 0;
        this.yOffset = 0;
        this.type = '';
        this.buttonId = '';
    },

    this.initButtonDetails();
}

FoxytunesSignaturesSiteHandler.prototype = {

    getHandlerData: function() {return this.handlerData;},

    startMonitoring: function() {
        var me = this;
        this.stopMonitoring();
        this.monitorId = setInterval(function() { me.handleSite(); }, this.monitorTimeout);
        this.printTrace('start monitoring.... ' + this.monitorId + '\n');
    },

    stopMonitoring: function() {
        if (this.monitorId == null) return;

        clearInterval(this.monitorId);
        this.printTrace('Stop monitoring.... ' + this.monitorId + '\n');
        this.monitorId = null;
    },

    setButtonDetailsData: function(data, buttonDetails) {
        if (data['button-location']) {
            buttonDetails.loc = data['button-location'];
        }
        if (data['container-xpath']) {
            buttonDetails.containerXpath = data['container-xpath'];
        }
        if (data['signature-type']) {
            buttonDetails.signatureType = data['signature-type'];
        }
        if (data['attach-button-to-body'] && 
            (data['attach-button-to-body'] == 'true')) {
            buttonDetails.bAttachToBody = true;
        }
        if (data['x-offset']) {
            buttonDetails.xOffset = parseInt(data['x-offset']);
        }
        if (data['y-offset']) {
            buttonDetails.yOffset = parseInt(data['y-offset']);
        }
        if (data['type']) {
            buttonDetails.type = data['type'];
        }
    },

    initButtonDetails: function() {
        this.buttonDetails = new Array();
        for (var i=0; i<this.handlerData.nodes.length; i++) {
            this.buttonDetails[i] = new this.ButtonDetailsDefaults();
            // Get data from the site 
            this.setButtonDetailsData(this.handlerData, this.buttonDetails[i]);
            // Get data from the node (can override the site definitions) 
            this.setButtonDetailsData(this.handlerData.nodes[i], this.buttonDetails[i]);
        }
    },

    getNodes: function(xpath) {
        var xpathArray = xpath.split('####');
        var currentDoc = this.doc;
        var currentNode = new Array();
        for (var i=0; i<xpathArray.length; i++) {
            currentNode[i] = currentDoc.evaluate(xpathArray[i], currentDoc, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null).snapshotItem(0);
            if (!currentNode[i]) return null;
            if ((i < xpathArray.length) && (currentNode[i].localName.toLowerCase() == 'iframe')) {
                currentDoc = currentNode[i].contentDocument;
            } 
        }
	    return {
		    node : currentNode[0],
			signatureInsertNode : currentNode[xpathArray.length-1],
	    };
    },

    removeGhostSignatunesButton: function(node, id) {
        var signatureButton = this.doc.getElementById(id);
        if (signatureButton) {
            signatureButton.parentNode.removeChild(signatureButton);
        }   
        try {
            node.removeAttribute('foxytunes-signature-button-id');
        } catch(e) {};
    },

    handleSite: function() {
        if (!this.doc.defaultView) {
            this.stopMonitoring();
            return;
        }

        for (var iNode=0; iNode<this.handlerData.nodes.length; iNode++) {
            var nodeData = this.handlerData.nodes[iNode];
            if (nodeData['add-button'] && (nodeData['add-button'] == 'false')) continue;
            var nodes = this.getNodes(nodeData.xpath);

            if (nodes && nodes.node && nodes.signatureInsertNode) {

                // If node changed -> remove button from old node
                if ((this.buttonDetails[iNode].node != null) && 
                    (this.buttonDetails[iNode].node != nodes.node)) {
                        var id = this.buttonDetails[iNode].node.getAttribute('foxytunes-signature-button-id');
                        this.removeGhostSignatunesButton(this.buttonDetails[iNode].node, id);
                }

                this.buttonDetails[iNode].node = nodes.node;
                this.buttonDetails[iNode].signatureInsertNode = nodes.signatureInsertNode;

                // Handle this node - add signature button if needed or adjust its position
                gFoxytunesSignatures.handleNode(this, this.buttonDetails[iNode], this.doc);
            } else {
                // If the node disappeared -> remove the signatunre button
                if (this.buttonDetails[iNode].node != null) {
                    this.removeGhostSignatunesButton(this.buttonDetails[iNode].node, this.buttonDetails[iNode].buttonId);
                    this.buttonDetails[iNode].node = null;
                } 
            }
        }
    },

    printTrace : function(message) {
        // dump(message + '\n');
    }
}

function FoxytunesSignatures() {
    this.XULNS = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
    this.HTMLNS = 'http://www.w3.org/1999/xhtml';
    this.observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);
    this.observerService.addObserver(this, "foxytunes-internal-signatures", false);
    this.sitesXMLFilename = 'signatureSites.xml';
    this.sitesXMLFile = null;
    this.siteHandlers = null;

    this.bAutoMode = new Object();

    var me = this;
    this.domContentLoadedCallback =  function(event) { me.onDOMContentLoad(event); }

    this.foxytunesService = Components.classes['@foxytunes.org/FoxyTunesEngine/FoxyTunesService;1'].
                                    getService(Components.interfaces.IFoxyTunesService);
    this.historyTracks = this.foxytunesService.GetPlaybackHistory().wrappedJSObject;
    this.MAX_TRACKS_FROM_HISTORY = 20;

    // Balloon messages
    this.showBalloonMessage = false;
    this.balloonMessage = new Object();
    this.balloonMessage.messageID = 1;
    this.balloonMessage.text = foxytunesGetLocaleString('signatunesBalloonMessage1');
    this.balloonMessage.showLernMore = true;
    this.balloonMessage.learnMoreLink = 'http://www.foxytunes.com/signatunes/news/1';

    // First run ballon
    this.firstRun = false;
    this.firstRunBallonInfo = new Object();
    this.firstRunBallonInfo.text = foxytunesGetLocaleString('signatunesFirstRunBalloonText');
    this.firstRunBallonInfo.showLernMore = true;
    this.firstRunBallonInfo.learnMoreLink = 'http://www.foxytunes.com/signatunes/';
}

FoxytunesSignatures.prototype = {

    init: function() {
        // Used in order to show the first-run ballon
        if (foxytunesReadSinglePreference('signature.first-run', 'true') == 'true') {
            this.firstRun = true;
        }
        // Used in order to show the balloon message
        if (foxytunesReadSinglePreference('signature.messeges.' + this.balloonMessage.messageID, 'true') == 'true') {
            this.showBalloonMessage = true;
        }

        this.readPreferences();
        this.observePreferences(true);

        // Override the profile site-xml in case that the jar contains a newer file
        var bForceXmlFileInit = false; 

        // Compare inner xml version only if FoxyTunes version changed
        var bCompareXMLVersionIsneeded = this.isFoxytunesVersionChanged();

        try {
          if (bCompareXMLVersionIsneeded) {
        FTJSLib.include(FTJSLib.jslib_chromefile);
        var cf = new FTJSLib.ChromeFile('chrome://foxytunes/content/signatures/' + this.sitesXMLFilename);
        if (cf.isJarFile) {
            var domParser = new DOMParser();
            var xmlFileInJarDom = domParser.parseFromString(cf.read(), "text/xml");
        } else {
            // In case the FT extension is open
            var xmlFileInJarDom = this.readSitesXML(cf.nsIFile);
        }
        var xmlFileInJarVersion = xmlFileInJarDom.getElementsByTagName('version')[0].firstChild.nodeValue;
        var xmlFileInProfileVersion = foxytunesReadSinglePreference('signatures.sitelist.version', '0');
        if (xmlFileInJarVersion > xmlFileInProfileVersion) {
            bForceXmlFileInit = true;
        }
          }
        } catch (e) {};
        
        this.initSitesXMLFileIfNeeded(bForceXmlFileInit);
        this.initSiteHandlers();

        var appcontent = document.getElementById("appcontent");   
        if (appcontent) {
            appcontent.addEventListener("DOMContentLoaded", this.domContentLoadedCallback, true);
        }
    },

    unInit: function() {
        this.observePreferences(false);
        this.observerService.removeObserver(this, "foxytunes-internal-signatures");
        var appcontent = document.getElementById("appcontent");   
        if (appcontent) {
            appcontent.removeEventListener("DOMContentLoaded", this.domContentLoadedCallback, true);
        }
    },

    isFoxytunesVersionChanged: function() {
        var currrentFoxytunesVersion = foxytunesGetFoxyTunesVersion();
        if (!currrentFoxytunesVersion) return false;
        var signatunesFoxytunesVersion = foxytunesReadSinglePreference('signatures.foxytunes.version', '---');
        // Foxytunes version changed
        if (signatunesFoxytunesVersion != currrentFoxytunesVersion) {
            foxytunesWriteSinglePreference('signatures.foxytunes.version', currrentFoxytunesVersion);
            return true;
        }
        return false;
    },

    checkForUpdates: function(bManuallCheck) {
        if (typeof bManuallCheck == 'undefined') bManuallCheck = false;
        var params = null;
        if (bManuallCheck) {
            params = {};
            params.callback =  {
                notify: function(bUpdateFound) {
                    var title = 'FoxyTunes';
                    var text;
                    if (bUpdateFound) {
                        body = foxytunesGetLocaleString('siteDefinitionsUpdated');
                    } else {
                        body = foxytunesGetLocaleString('noUpdatesFound');
                    }
                    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                                           getService(Components.interfaces.nsIPromptService);
                    promptService.alert(null, title, body);
                }
            }
            params.foxytunesVersion = foxytunesGetFoxyTunesVersion();
            params.wrappedJSObject = params;
        }
        var foxytunesService = Components.classes['@foxytunes.org/FoxyTunesEngine/FoxyTunesService;1'].
                                 getService(Components.interfaces.IFoxyTunesService);
        foxytunesService.SignatunesCheckForUpdates(params);
    },

    initSiteHandlers: function() {
        var sitesXML = this.readSitesXML(this.sitesXMLFile);        
        // If the file is coruppted - force init
        if (!sitesXML) {
            this.printTrace('corrupted file - reset\n');
            this.initSitesXMLFileIfNeeded(true);
            sitesXML = this.readSitesXML(this.sitesXMLFile);        
        }
        this.parseSitesXML(sitesXML);
    },

    initSitesXMLFileIfNeeded: function(bForce) {
        var xmlFileFolder = Components.classes["@mozilla.org/file/directory_service;1"].
                            getService(Components.interfaces.nsIProperties).
                            get("ProfD", Components.interfaces.nsIFile);
        xmlFileFolder.append("FoxyTunes");
        this.sitesXMLFile = xmlFileFolder.clone();
        this.sitesXMLFile.append(this.sitesXMLFilename);

        FTJSLib.include(FTJSLib.jslib_fileutils);
        var fileUtils = new FTJSLib.FileUtils;
        if (fileUtils.exists(this.sitesXMLFile.path) && bForce) {
            fileUtils.remove(this.sitesXMLFile.path);
        }
        if (!fileUtils.exists(this.sitesXMLFile.path)) {
            FTJSLib.include(FTJSLib.jslib_dir);
            var foxytunesDirInProfile = new FTJSLib.Dir(xmlFileFolder.path);
            if (!foxytunesDirInProfile.exists()) {
                foxytunesDirInProfile.create(0777);
            }
            FTJSLib.include(FTJSLib.jslib_chromefile);
            var cf = new FTJSLib.ChromeFile('chrome://foxytunes/content/signatures/' + this.sitesXMLFilename);
            var text;
            if (cf.isJarFile) {
                text = cf.read();
            } else {
                text = this.readFile(cf.nsIFile)
            }

            var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                              createInstance( Components.interfaces.nsIFileOutputStream );
            outputStream.init(this.sitesXMLFile, 0x04 | 0x08 | 0x20,  0777, 0);
        	outputStream.write(text, text.length);
	        outputStream.close();

            // Writing current version before checking for update 
            // so that update will know the xml version in case of fresh installation
            var xmlFile = this.readSitesXML(this.sitesXMLFile)
            var version = xmlFile.getElementsByTagName('version')[0];
            if (version) {
                version = version.firstChild.nodeValue;
                foxytunesWriteSinglePreference('signatures.sitelist.version', version);
            }

            // After copying the xml from the jar file check for updates in cast this xml is old
            var params = {};
            params.foxytunesVersion = foxytunesGetFoxyTunesVersion();
            params.signatunesXMLVersion = version;
            params.initialUpdateCheck = true;
            params.wrappedJSObject = params;
            var foxytunesService = Components.classes['@foxytunes.org/FoxyTunesEngine/FoxyTunesService;1'].
                                 getService(Components.interfaces.IFoxyTunesService);
            foxytunesService.SignatunesCheckForUpdates(params);
        }
    },

    getObjFromDOM: function(dom) {
        var obj = new Object;
        var childNodes = dom.childNodes;
        for (var i=0; i<childNodes.length; i++) {
            if (childNodes[i].nodeType != 1) continue;
            var name  = childNodes[i].localName;
            var value = childNodes[i].firstChild.nodeValue;
            obj[name] = value; 
        }
        return obj;
    },

    parseSitesXML: function(sitesXML) {
        this.siteHandlers = new Array();

        var version = sitesXML.getElementsByTagName('version')[0];
        if (version) {
            version = version.firstChild.nodeValue;
            foxytunesWriteSinglePreference('signatures.sitelist.version', version);
        }
        var sites = sitesXML.getElementsByTagName('site');
        for (var i=0; i<sites.length; i++) {
            var handler = this.getObjFromDOM(sites[i]);
            handler.nodes = new Array();
            var nodes = sites[i].getElementsByTagName('node');
            for (var iNode=0; iNode<nodes.length; iNode++) {
                var node = this.getObjFromDOM(nodes[iNode]);
                handler.nodes.push(node);
            }
            handler.domains = new Array();
            handler.domainsRegExp = new Array();
            var domains = sites[i].getElementsByTagName('domain');
            for (var iDomain=0; iDomain<domains.length; iDomain++) {
                var domain = domains[iDomain].firstChild.nodeValue; 
                handler.domains.push(domain);
                handler.domainsRegExp.push(new RegExp(domain));
            }
            handler.adjustments = new Array();
            var adjustments = sites[i].getElementsByTagName('adjustment'); 
            for (var iMod=0; iMod<adjustments.length; iMod++) {
                var adjustment = this.getObjFromDOM(adjustments[iMod]);
                handler.adjustments.push(adjustment);
            }
            this.siteHandlers.push(handler);
        }
    },

    readFile: function(file) {
        var fiStream = Components.classes['@mozilla.org/network/file-input-stream;1']
                            .createInstance(Components.interfaces.nsIFileInputStream);
        var siStream = Components.classes['@mozilla.org/scriptableinputstream;1']
                            .createInstance(Components.interfaces.nsIScriptableInputStream);
        fiStream.init(file, 1, 0, false);
        siStream.init(fiStream);
        var data = siStream.read(-1);
        siStream.close();
        fiStream.close();
        return data;
    },

    readSitesXML: function(xmlFile) {
        var data = this.readFile(xmlFile);
        var domParser = new DOMParser();
        var sitesXML = domParser.parseFromString(data, "text/xml");
        if ((sitesXML.documentElement.tagName == "parserError") ||
            (sitesXML.documentElement.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml")){
            return null;  
        }
        
        return sitesXML;
    },

    observePreferences: function(bAdd) {
        var prefBranch2 = gFoxyTunesPref.QueryInterface(Components.interfaces.nsIPrefBranch2);
        if (bAdd) {
            prefBranch2.addObserver("", this, false);  
        } else {
            prefBranch2.removeObserver("", this);  
        }
    },

    observe: function(subject, topic, data) {
        if (topic == "nsPref:changed") {
                 this.readPreferences();
        }
        if (topic == "foxytunes-internal-signatures") {
            if (data == "foxytunes-signatures-sites-xml-updated") {
                this.initSiteHandlers();
            }
        }
    },

    readPreferences: function() {
	    this.bAutoMode['all'] = (foxytunesReadSinglePreference('signatures.automode', 'false') == 'true') ? true : false;
	    this.bAutoMode['mail'] = (foxytunesReadSinglePreference('signatures.automode.mail', 'false') == 'true') ? true : false;
	    this.bAutoMode['blog'] = (foxytunesReadSinglePreference('signatures.automode.blog', 'false') == 'true') ? true : false;
	    this.bAutoMode['forum'] = (foxytunesReadSinglePreference('signatures.automode.forum', 'false') == 'true') ? true : false;
		this.bDisableSignatures = (foxytunesReadSinglePreference('signatures.disable', 'false') == 'true') ? true : false;
		this.bUsePostedWithFoxyTunes = (foxytunesReadSinglePreference('signatures.use-posted-with-foxytunes', 'true') == 'true') ? true : false;
        var defaultFormat = foxytunesGetLocaleString('nowPlaying', [ '%artist-track%' ]);
        this.nowPlayingFormat = foxytunesReadSingleUnicharPreference("signatures.now-playing-format", defaultFormat); 
    },

    getElementsByProperty: function(doc, tagName, attribute, value) {
        var res = new Array();
        var nodes = doc.getElementsByTagName(tagName);
        for (var i=0; i<nodes.length; i++) {
            var node = nodes[i];
            if (node.getAttribute(attribute) == value) {
                res.push(node);
            }
        }
        return res;
    },

  getNowPlayingHTML : function(trackTitle, trackLink, player, type) {
        var nowPlaying = '<br><br>' + 
                         '----------------' + 
                         '<br>' + 
                         this.nowPlayingFormat;
        if (this.bUsePostedWithFoxyTunes) {  
               var foxytunesTitle = foxytunesGetLocaleString('webOfMusicAtYourFingertips');
               var postedWithLink = '<a style="color:#666" href="http://www.foxytunes.com/signatunes/" title="' + 
                                foxytunesTitle +
                                '">FoxyTunes</a>';
               var postedWithText;
               if (type == 'blog') {
                   postedWithText = foxytunesGetLocaleString('via', [ postedWithLink ]);
               } else {
                   postedWithText = foxytunesGetLocaleString('postedWith', [ postedWithLink ]);               
               }
               nowPlaying += '<br><span style="color: rgb(153, 153,153); font-style: italic; font-size: 10px;">' +
                             postedWithText + 
                             '</span>&nbsp;&nbsp;&nbsp;&nbsp;';
        }
        var title = foxytunesGetLocaleString('openOnPlanet', [ "'" + trackTitle + "'" ]); 
        var artistTrack = '<a href="' + trackLink + '" title="' +
                          title + 
                          '">' + trackTitle + '</a>';
        nowPlaying = nowPlaying.replace(/%artist-track%/g, artistTrack);
        nowPlaying = nowPlaying.replace(/%player%/g, player);
        return nowPlaying;
    },

    getNowPlayingMinimalHTML: function(trackTitle, trackLink, player, type) {
        var nowPlaying = '\n\n' +
                         '----------------' + 
                         '\n' + 
                         this.nowPlayingFormat;
        if (this.bUsePostedWithFoxyTunes) {  
               var postedWithLink = '<a href="http://www.foxytunes.com/signatunes/">FoxyTunes</a>';
               var postedWithText = foxytunesGetLocaleString('via', [ postedWithLink ]);
               nowPlaying += '\n' + postedWithText;  
        }
        var artistTrack = '<a href="' + trackLink + '">' + trackTitle + '</a>';
        nowPlaying = nowPlaying.replace(/%artist-track%/g, artistTrack);
        nowPlaying = nowPlaying.replace(/%player%/g, player);
        return nowPlaying;
    },

    getNowPlayingMinimalHTMLRaw: function(trackTitle, trackLink, player, type) {
        var nowPlaying = '<br><br>' +
                         '----------------' + 
                         '<br>' + 
                         this.nowPlayingFormat;
        if (this.bUsePostedWithFoxyTunes) {  
               var postedWithLink = '<a href="http://www.foxytunes.com/signatunes/">FoxyTunes</a>';
               var postedWithText = foxytunesGetLocaleString('via', [ postedWithLink ]);
               nowPlaying += '<br>' + postedWithText;  
        }
        var artistTrack = '<a href="' + trackLink + '">' + trackTitle + '</a>';
        nowPlaying = nowPlaying.replace(/%artist-track%/g, artistTrack);
        nowPlaying = nowPlaying.replace(/%player%/g, player);
        return nowPlaying;
    },

    getNowPlayingBBCode: function(trackTitle, trackLink, player, type) {
        var nowPlaying = '\n\n' +
                         '----------------' + 
                         '\n' + 
                         this.nowPlayingFormat;
        if (this.bUsePostedWithFoxyTunes) {  
               var postedWithLink = '[url=http://www.foxytunes.com/signatunes/]FoxyTunes[/url]';
               var postedWithText = foxytunesGetLocaleString('via', [ postedWithLink ]);
               nowPlaying += '\n' + postedWithText;  
        }
        var artistTrack = '[url=' + trackLink + ']' + trackTitle + '[/url]';
        nowPlaying = nowPlaying.replace(/%artist-track%/g, artistTrack);
        nowPlaying = nowPlaying.replace(/%player%/g, player);
        return nowPlaying;
    },

    getNowPlayingPlainText : function(trackTitle, trackLink, player, type) {
        var nowPlaying =  '\n\n' + 
                          '----------------' + 
                          '\n' + 
                          this.nowPlayingFormat;
/*
        if (this.bUsePostedWithFoxyTunes) {
            var postedWithText;
            if (type == 'blog') {
                postedWithText = foxytunesGetLocaleString('via',[ 'http://foxytunes.com/signatunes/' ]);
            } else {
                postedWithText = foxytunesGetLocaleString('postedWith', [ 'http://foxytunes.com/signatunes/' ]);
            }
            nowPlaying += '\n' + postedWithText;
        }
*/
        nowPlaying = nowPlaying.replace(/%artist-track%/g, trackTitle + '\n' + trackLink);
        nowPlaying = nowPlaying.replace(/%player%/g, player);
        return nowPlaying;
    },

    // In case of auto-mode and video - return null
    // type - mail/blog/etc...
    getNowPlayingAudio: function(bIsAutoMode, type, signatureType, customTrackData) {
      if (typeof customTrackData == 'undefined') {
      var trackData = foxytunesGetCurrentTrackData();
      } else {
          var trackData = customTrackData;
      }
      if (bIsAutoMode &&  (trackData.isVideo != 'no')) {
        this.printTrace('Signatures - detected video playing, ignoring auto mode\n');
        return null;
      }
      var trackTitle = gFoxyTunesSanitizer.replaceTriangularBrackets(trackData.originalTitle);
      var trackLinkHTML = foxytunesGetTrackDataLink(trackData, false);
      var trackLinkText = foxytunesGetTrackDataLink(trackData, true);
      var player = foxytunesTryGetFoxyTunesPlayerFromContractID(gFoxyTunesCurrentPlayerClass);
      var readablePlayerName = gFTPlayerConfig.getReadableName(player.toLowerCase());         
	  readablePlayerName = readablePlayerName ? readablePlayerName : player;

      var signature = null;
      switch (signatureType) {
        case 'full-html': signature = this.getNowPlayingHTML(trackTitle, trackLinkHTML, readablePlayerName, type);
                          break;
        case 'minimal-html': signature = this.getNowPlayingMinimalHTML(trackTitle, trackLinkHTML, readablePlayerName, type);
                            break;
        case 'minimal-html-raw': signature = this.getNowPlayingMinimalHTMLRaw(trackTitle, trackLinkHTML, readablePlayerName, type);
                        break;
        case 'bbcode': signature = this.getNowPlayingBBCode(trackTitle, trackLinkHTML, readablePlayerName, type);
                        break;
        case 'txt': signature = this.getNowPlayingPlainText(trackTitle, trackLinkText, readablePlayerName, type);
                    break;
      }
      return signature;
    },

    getCaretPosition: function(editor) {
        var editorWindow = editor.defaultView;
        var selection = editorWindow.getSelection();
        var range = selection.getRangeAt(0);
        
        var containerParent = range.startContainer.parentNode;
        var containerIndex = 0;
        while ((containerParent.childNodes[containerIndex] != range.startContainer) &&
               (containerIndex != containerParent.childNodes.length)) {
            containerIndex++;
        }

        var nodes = editor.getElementsByTagName(containerParent.nodeName);
        for (var i=0; i<nodes.length; i++) {
            if (nodes[i] == containerParent) {
                var containerParentIndex = i;
                break;
            }
        }

       	return {
		    containerParentTagName : containerParent.nodeName,
			containerParentIndex : containerParentIndex,
			containerIndex : containerIndex,
			startOffset : range.startOffset,
	    };
    },

    moveCaretTo: function(editor, caretPosition) {
        var editorWindow = editor.defaultView;
        var selection = editorWindow.getSelection();
        var range = editor.createRange();

	    var nodes = editor.getElementsByTagName(caretPosition.containerParentTagName);
        if (nodes.length > caretPosition.containerParentIndex) {
		    range.setStart(nodes[caretPosition.containerParentIndex].childNodes[caretPosition.containerIndex], 
                            caretPosition.startOffset);
		    range.setEnd(nodes[caretPosition.containerParentIndex].childNodes[caretPosition.containerIndex], 
                            caretPosition.startOffset);
        }

		selection.removeAllRanges();
		selection.addRange(range);
    },

    addNowPlayingHTML: function(node, signature) {
        var doc = node.contentWindow.document;
        if (!doc.designMode || !(doc.designMode == 'on')) return;
        try {
            var caretPosition = this.getCaretPosition(doc);
        } catch(e) {};
        doc.execCommand("inserthtml", 
                        false, 
                        signature);
        try {
            this.moveCaretTo(doc, caretPosition);
        } catch(e) {};
    //    doc.defaultView.focus();
    },

    addNowPlayingText: function(node, signature) {
     //   node.focus();

        if (node.selectionStart || node.selectionStart == '0') {
            var startPos  = node.selectionStart;
            var endPos    = node.selectionEnd;
        	var scrollTop = node.scrollTop;

            node.value = node.value.substring(0, startPos) +
		                 signature + 
        	             node.value.substring(endPos, node.value.length);

	        node.selectionStart = startPos;
	        node.selectionEnd = startPos;
	        node.scrollTop = scrollTop;
        } else {
        	node.value += signature;
        }

    //    node.focus();
    },

    // bIsAutoMode - in case of auto mode if we're playing video the signature wont be added
    addNowPlayingAudio:function(buttonDetails, bIsAutoMode, customTrackData) {
        var node = buttonDetails.signatureInsertNode;
        var signatureType = buttonDetails.signatureType;

        if (typeof bIsAutoMode == 'undefined') bIsAutoMode = false;
        var nodeType = node.localName.toLowerCase();
        var signatureType =  buttonDetails.signatureType;
        if ((nodeType == 'iframe') && (signatureType == '')) {
            signatureType = 'full-html';
        }
        if ((nodeType == 'textarea') && (signatureType == '')) {
            signatureType = 'txt';
        }
        var signature = this.getNowPlayingAudio(bIsAutoMode, buttonDetails.type, signatureType, customTrackData);
        if (!signature) return;
        this.printTrace("Adding now playing...\n");
        switch (nodeType) {
            case 'iframe': this.addNowPlayingHTML(node, signature);
                          break; 
            case 'textarea': this.addNowPlayingText(node, signature);
                          break;
        }
        
        // If not auto mode set focus on node
        if (!bIsAutoMode) {
            if (nodeType == 'iframe') {
                node.contentWindow.document.defaultView.focus();
            }
            if (nodeType == 'textarea') {
                node.focus();
            }
        }
    },

    removeSignatureButtons: function(doc) {
        var buttons = this.getElementsByProperty(doc, 'img', 'class', 'foxytunes-signature-button')
        for (var i=0; i<buttons.length; i++) {
            var signatureButton = buttons[i];
            signatureButton.parentNode.removeChild(signatureButton);
        }
    },

    isMusicPlaying: function() {
         var trackTitle = foxytunesGetCurrentTrackTitle();
         var bIsMusicPlaying = trackTitle ? true : false;
         return bIsMusicPlaying;
    },

    menuPopupExecCommand: function(event) {
        if (event.target.localName != 'menuitem') return;
        var popup = document.getElementById('foxytunes-signature-menu');
        var buttonDetails = popup.buttonDetails;
        var id = event.target.getAttribute('id');
        var signatunesObject = foxytunesIsInThunderbird() ? gFoxytunesThunderbirdSignatures : this;
        switch (id) {
            case 'signatunes-insert-now-playing': 
                             signatunesObject.addNowPlayingAudio(buttonDetails, false);
                             break;
            case 'signatunes-insert-now-playing-from-history':
                             signatunesObject.addNowPlayingAudio(buttonDetails, false, event.target.value);
                             break;
            case 'signatunes-insert-custom-now-playing':
                            var dialogTitle = foxytunesGetLocaleString('signatunesTrackChooseTitle');
                            var me = this;
                            window.openDialog("chrome://foxytunes/content/utils/trackChooser.xul", 
                            "_blank", 
                            "chrome, resizable, modal, centerscreen",
                            function(trackData) { signatunesObject.addNowPlayingAudio(buttonDetails, false, trackData); },
                            dialogTitle);
                             break;
            case 'foxytunes-signatures-auto-mode':
                            var value = event.target.getAttribute("checked");
                            foxytunesWriteSinglePreference('signatures.automode', value);
                            foxytunesWriteSinglePreference('signatures.automode.mail', value);
                            foxytunesWriteSinglePreference('signatures.automode.blog', value);
                            foxytunesWriteSinglePreference('signatures.automode.forum', value);
                            break;
            case 'signatunes-now-playing-configuration': 
                            foxytunesOpenSignatuensConfigurationWindow();
                            break;
        }
    },

    handlePopupShowing: function(event) {
        if (event.target.id != 'foxytunes-signature-menu') return;

        var bDisable = 'false'
        if (!this.isMusicPlaying()) {
            var bDisable = 'true';
        }

        var popup = event.target;
        var menuitem = popup.getElementsByAttribute('id', 'signatunes-insert-now-playing')[0];
        menuitem.setAttribute('disabled', bDisable);

        menuitem = popup.getElementsByAttribute('id', 'foxytunes-signatures-auto-mode')[0];
        var autoMode = (this.bAutoMode['all'] || this.bAutoMode['mail'] || this.bAutoMode['blog'] || this.bAutoMode['forum']) ? 'true' : 'false'
        menuitem.setAttribute('checked', autoMode);

        var bIsVideo = false;
        if (bDisable != 'true') {
        var trackData = foxytunesGetCurrentTrackData();
        if (trackData.isVideo != 'no') {
            bIsVideo = true;
            }
        }
        var autoModeMenuItem = popup.getElementsByAttribute('id', 'foxytunes-signatures-auto-mode')[0];
        var autoModeDisabledMenuItem = popup.getElementsByAttribute('id', 'foxytunes-signatunes-auto-mode-disabled')[0];
        autoModeMenuItem.hidden = bIsVideo;
        autoModeDisabledMenuItem.hidden = !bIsVideo; 

        var historyMenu = popup.getElementsByAttribute('id', 'signatunes-insert-now-playing-history-menu')[0];
        if (this.historyTracks.length == 0) {
            historyMenu.setAttribute('disabled', 'true');
        } else {
            historyMenu.setAttribute('disabled', 'false');
        }
    },

    buildRecentHistoryMenu: function(event) {
        var menuPopup = event.target;
        var menu = menuPopup.parentNode;
        
        foxytunesClearMenupopup(menuPopup);
        var nAdded = 0;
        for (var i = this.historyTracks.length - 1; i >= 0 ; i--) {
            var trackData = this.historyTracks[i];
            var menuitem = document.createElement('menuitem');
            menuitem.value = trackData;
            menuitem.setAttribute('label', trackData.normalizedOriginalTitle);
            menuitem.setAttribute('id', 'signatunes-insert-now-playing-from-history');
            menuPopup.appendChild(menuitem);
            nAdded++;
            if (nAdded >= this.MAX_TRACKS_FROM_HISTORY) break;
        } 
        menuPopup.appendChild(menuitem);
    },

    // Get node x,y calculated by adding the node offsetLeft, offsetTop with its ancestors left and top
    // until the containingElem is reached
    getNodePosition: function(node, containingElem) {
    	var curleft = curtop = 0;
    	if (node.offsetParent) {
    		curleft = node.offsetLeft;
	    	curtop = node.offsetTop;
    		while ((node = node.offsetParent) && (node != containingElem)) {
    			curleft += node.offsetLeft;
	    		curtop += node.offsetTop;
    		}
	    }
    	return [curleft,curtop];
    },

    // adjust the ballon position according to the signature button top/left
    adjustFirstRunBalloonPosition: function(firstRunBallon, top, left, signatureButtonLocation) {
        var newTop = top -97;
        var newLeft;
        if ((signatureButtonLocation == 'leftbottom') || (signatureButtonLocation == 'leftup')) {
            newLeft = left-8;
        } else {
            newLeft = left-250;
        }
        if (newTop < 0) newTop = 0;
        if (newLeft < 0) newLeft = 0;        
        firstRunBallon.style.top = newTop + 'px';
        firstRunBallon.style.left = newLeft + 'px'; 
    },

    // adjust the signature button position to match the node position
    adjustButtonPosition: function(button, buttonDetails) {
        var node = buttonDetails.node;
        var loc = buttonDetails.loc;
        var xOffset = buttonDetails.xOffset;
        var yOffset = buttonDetails.yOffset;

        if (!this.isNodeDisplayed(node)) return;

        // Calculating the coords up to the signature button offsetParent 
        // because the absolute position of the button is relative to its offsetParent
        var coords = this.getNodePosition(node, button.offsetParent);
        var left = coords[0];
        var top  = coords[1];
        
        switch (loc) {
            case 'rightbottom': left += Math.max(1, node.offsetWidth - 30);
                                top += node.offsetHeight;
                                break;
            case 'righttop': left += Math.max(1, node.offsetWidth - 30);
                             top -= (button.offsetHeight);
                             break;
            case 'leftbottom': top += node.offsetHeight;
                                break;
            case 'lefttop': top -= (button.offsetHeight);
                             break;
        }

        top += yOffset;
        left += xOffset;

        if (top < 0) top = 0;

        leftStyle = left + 'px';
        topStyle  = top + 'px';
        if (button.style.left != leftStyle) { 
            button.style.left = leftStyle; 
        }
        if (button.style.top != topStyle) { 
            button.style.top = topStyle; 
        }

        // adjust firstRunBallon position
        var firstRunBallon = node.ownerDocument.getElementById('foxytunes-signatures-first-run-ballon');
        // If no balloon or balloon doesn't belong to signatune button (in case of more than one button we don't wont the ballon jumping back and forth)
        if (!firstRunBallon || 
            (firstRunBallon.getAttribute('foxytunes-signature-button-id') != button.id)) return;
        this.adjustFirstRunBalloonPosition(firstRunBallon, top, left, buttonDetails.loc);
    },

    showMenuPopup: function(x, y, buttonDetails) {
        var selectedBrowser = gBrowser.selectedBrowser;
        var browserX = selectedBrowser.boxObject.screenX;
        var browserY = selectedBrowser.boxObject.screenY;

        var popup = document.getElementById('foxytunes-signature-menu');
        popup.buttonDetails = buttonDetails;
        
        document.popupNode = null;
        popup.showPopup(
           document.documentElement,   
           browserX + x,
           browserY + y,
           "context",                  
           "",                         
           "");      
    },

    showNoMusicAlert: function() {
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                                getService(Components.interfaces.nsIPromptService);
        var noMusicAlertText = foxytunesGetLocaleString('signatunesNoMusicAlert');
        promptService.alert(null, 'FoxyTunes', noMusicAlertText);
    },

    handleSignatureButtonClick: function(event, buttonDetails) {
        var node = buttonDetails.node;
        var signatureButton = event.target;
        // Getting the button position in order to calculate if the click was on the button or
        // on the dropmarker
        var coords = this.getNodePosition(signatureButton, node.ownerDocument.documentElement);
        var doc = node.ownerDocument;

        // close the first time balloon if open
        this.closeFirstRunBallon(doc, signatureButton);

        // Scroll amount of the html - could be in the body or in the documentElement
        var scrollOffsetTop = doc.body.scrollTop ? doc.body.scrollTop : doc.documentElement.scrollTop;
        var scrollOffsetLeft = doc.body.scrollLeft ? doc.body.scrollLeft : doc.documentElement.scrollLeft;
        var buttonX = coords[0] - scrollOffsetLeft;
        var buttonY = coords[1] - scrollOffsetTop;

        // Calculating if the click was on the dropmarker or on the main button
        if ((event.clientX - buttonX) > 18) {
            this.showMenuPopup(buttonX, buttonY + signatureButton.offsetHeight, buttonDetails);
        } else {
            if (this.isMusicPlaying()) {
                this.addNowPlayingAudio(buttonDetails, false);
            } else {
                this.showNoMusicAlert();
            }
        }
    },

    signatureButtonSetOpacityLevel: function(signatureButton, opacityLevel) {
        if (opacityLevel == 'low') {
            signatureButton.style.opacity = '0.7';
        } else {
            signatureButton.style.opacity = '1';
        }
    },

    signatureButtonOnMouseHover: function(signatureButton) {
        signatureButton.setAttribute('src', 'chrome://foxytunes-pbulic/content/signatures/signature-button-on-hover.png');
        this.signatureButtonSetOpacityLevel(signatureButton, 'high');
        signatureButton.style.borderColor = 'ThreeDHighlight ThreeDShadow ThreeDShadow ThreeDHighlight';
        signatureButton.style.borderStyle = 'solid';
        signatureButton.style.borderWidth = '1px';
    },

    signatureButtonOnMouseOut: function(signatureButton) {
        signatureButton.setAttribute('src', 'chrome://foxytunes-public/content/signatures/signature-button.png');
        this.signatureButtonSetOpacityLevel(signatureButton, 'low');
        signatureButton.style.border = '1px solid transparent';
    },

    signatureButtonOnMouseDown: function(signatureButton) {
        signatureButton.style.borderColor = 'ThreeDShadow ThreeDHighlight ThreeDHighlight ThreeDShadow';
        signatureButton.style.borderStyle = 'solid';
        signatureButton.style.borderWidth = '1px';
    }, 

    signatureButtonOnMouseUp: function(signatureButton) {
        this.signatureButtonOnMouseHover(signatureButton);
    },

    // Currently looking only on node and node->parent
    isNodeDisplayed: function(node) {
        if (node.style.display == 'none') return false;
        if (node.parentNode && node.parentNode.style.display == 'none') return false;
        if (node.parentNode.parentNode && node.parentNode.parentNode.style.display == 'none') return false;
        return true;
    },

    // Add the button if needed or adjust is position if it's already exists
    addSignatureButton: function(buttonDetails, doc) {
        var loc = buttonDetails.loc;
        var node = buttonDetails.node;
        var container = null;
        if (buttonDetails.containerXpath != '') {
            container = doc.evaluate(buttonDetails.containerXpath, doc, null, 
                                            XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null).snapshotItem(0);
        } 

        // If this node already has button attached to it just adjust the position
        if (node.hasAttribute('foxytunes-signature-button-id')) {
            var id = node.getAttribute('foxytunes-signature-button-id');
            var signatureButton = doc.getElementById(id);
            this.adjustButtonPosition(signatureButton, buttonDetails);
            if (!this.isNodeDisplayed(node)) {
                signatureButton.style.display = 'none';
            } else {
                signatureButton.style.display = 'block';
            }
            return;
        } 

        this.printTrace('adding button...\n');
        
        var signatureButton = doc.createElementNS(this.HTMLNS, 'img');
        signatureButton.setAttribute('class', 'foxytunes-signature-button');
        signatureButton.setAttribute('src', 'chrome://foxytunes-public/content/signatures/signature-button.png');

        var tooltiptext = foxytunesGetLocaleString('signatunesButtonTooltip');
        signatureButton.setAttribute('title', tooltiptext);

        signatureButton.style.cursor   = 'pointer';
        signatureButton.style.display  = 'block';
        signatureButton.style.visibility  = 'visible';
        signatureButton.style.position = 'absolute';
        signatureButton.style.zIndex = 100;
        signatureButton.style.width = '30px';
        signatureButton.style.height = '20px';
        this.signatureButtonSetOpacityLevel(signatureButton, 'low');
        signatureButton.style.padding = '0px';
        if (loc == 'rightbottom') {
            signatureButton.style.marginTop = '5px';
        }
        if (loc == 'righttop') {
            signatureButton.style.marginBottom = '5px';
        }
        signatureButton.style.marginRight = '7px';
        signatureButton.style.border = '1px solid transparent';

        
        var me = this;
        signatureButton.addEventListener('click', function(event){ me.handleSignatureButtonClick(event, buttonDetails); }, false);
        signatureButton.addEventListener('mouseover', 
                                         function(event) { me.signatureButtonOnMouseHover(signatureButton); }, false );
        signatureButton.addEventListener('mouseout', 
                                          function(event) { me.signatureButtonOnMouseOut(signatureButton); }, false );
        signatureButton.addEventListener('mousedown', 
                                          function(event) { me.signatureButtonOnMouseDown(signatureButton); }, false );
        signatureButton.addEventListener('mouseup', 
                                          function(event) { me.signatureButtonOnMouseUp(signatureButton); }, false );
        if (!container) container = node;
        container.addEventListener('mouseover', function(event) { me.signatureButtonSetOpacityLevel(signatureButton, 'high') }, false );
        container.addEventListener('mouseout', function(event) { me.signatureButtonSetOpacityLevel(signatureButton, 'low') }, false );

        var date = new Date();
        var id = 'ft_' + (date.getTime() + Math.round(Math.random()*1000));
        signatureButton.setAttribute('id', id);
        node.setAttribute('foxytunes-signature-button-id',id);
        buttonDetails.buttonId = id;

        this.adjustButtonPosition(signatureButton, buttonDetails);

        if (!buttonDetails.bAttachToBody) {
            var parent = node.parentNode;
            if (node.nextSibling) {
                parent.insertBefore(signatureButton, node.nextSibling);
            } else {
                parent.appendChild(signatureButton);
            }
        } else {
            doc.documentElement.appendChild(signatureButton);
        }

        this.adjustButtonPosition(signatureButton, buttonDetails);
        if (this.isNodeDisplayed(node)) {
            if (this.firstRun) {
            foxytunesWriteSinglePreference('signature.first-run', 'false');
            this.firstRun = false;
                
                // Don't show balloon message if it's first time installation
                foxytunesWriteSinglePreference('signature.messeges.' + this.balloonMessage.messageID, 'false');
                this.showBalloonMessage = false;
                
                this.showFirstRunBallon(this.firstRunBallonInfo, doc, signatureButton, buttonDetails.loc);
            } 
            if (this.showBalloonMessage) {
                this.showBalloonMessage = false;
                foxytunesWriteSinglePreference('signature.messeges.' + this.balloonMessage.messageID, 'false');
                this.showFirstRunBallon(this.balloonMessage, doc, signatureButton, buttonDetails.loc);
            }
        }
    },

    closeFirstRunBallon: function(doc, signatureButton) {
        var firstRunBallon = doc.getElementById('foxytunes-signatures-first-run-ballon');
        if (!firstRunBallon) return;
        firstRunBallon.parentNode.removeChild(firstRunBallon);
        this.signatureButtonSetOpacityLevel(signatureButton, 'low');
    },

    showFirstRunBallon: function(balloonInfo, doc, signatureButton, signatureButtonLocation) {
        // don't show ballon twice
        if (doc.getElementById('foxytunes-signatures-first-run-ballon')) return;

        this.signatureButtonSetOpacityLevel(signatureButton, 'high');
        var firstRunBallon = doc.createElementNS(this.HTMLNS, 'div');
        firstRunBallon.setAttribute('id', 'foxytunes-signatures-first-run-ballon');
        firstRunBallon.setAttribute('foxytunes-signature-button-id', signatureButton.id);
        firstRunBallon.style.backgroundImage = "url(chrome://foxytunes-public/content/signatures/first-run-ballon.png)"
        firstRunBallon.style.display  = 'block';
        firstRunBallon.style.visibility  = 'visible';
        firstRunBallon.style.position = 'absolute';
        firstRunBallon.style.zIndex = 1000;
        firstRunBallon.style.width = '283px';
        firstRunBallon.style.height = '100px';    
        firstRunBallon.style.fontSize = '13px';    
        
        // if the signature button is located at the left and the balllon left = 0 than change the
        // ballon orientation
        if ((signatureButtonLocation == 'leftbottom') || (signatureButtonLocation == 'leftup')) {
            firstRunBallon.style.backgroundImage = "url(chrome://foxytunes-public/content/signatures/first-run-ballon-left.png)"
        }

        var top = signatureButton.style.top.substr(0, signatureButton.style.top.length - 2);
        var left = signatureButton.style.left.substr(0, signatureButton.style.left.length - 2); 
        this.adjustFirstRunBalloonPosition(firstRunBallon, top, left, signatureButtonLocation);

        var ballonText = balloonInfo.text;
        var learnMore = foxytunesGetLocaleString('learnMore');
        var innerHTML = '<table style="background-color: transparent; width: 283px; height: 80px;"><tr style="background-color: transparent;">' +
                        '<td valign="middle" style="background-color: transparent;font-size: 12px;font-family: arial; padding-left: 40px; padding-right: 25px;">' + ballonText;
        if (balloonInfo.showLernMore) {
            innerHTML += ' <a target="_blank" href="' + balloonInfo.learnMoreLink + '">' + learnMore + '</a>';
        }
        innerHTML += '</td></tr></table>';
        firstRunBallon.innerHTML = innerHTML; 

        signatureButton.parentNode.insertBefore(firstRunBallon, signatureButton);

        var me = this;
        firstRunBallon.addEventListener('click', function() { me.closeFirstRunBallon(doc, signatureButton); }, false);
    },

    getSiteHandler: function(doc) {
        // ignore iframe editors
        if (doc.designMode && (doc.designMode == 'on')) {
            return null;
        }

        if (doc.location && doc.location.href) {
        var href = doc.location.href;
        } else {
            return null;
        }

        // Trying to get an handler for this site by looking at the 
        // site domain and nodes
        for (var i=0; i < this.siteHandlers.length; i++) {
            var handler = this.siteHandlers[i];
            var bDomainMatched = false;
            // Hanlder can include several domains if all domains share the same nodes
            for (var iDomain=0; iDomain<handler.domainsRegExp.length; iDomain++) {
                var domainRegExp = handler.domainsRegExp[iDomain];
                if (href.match(domainRegExp)) {
                    bDomainMatched = true;
                    break;
                }
            }
            // Handler domains doesn't match the current href - continue to next handler
            if (!bDomainMatched) continue;

            for (var iNode=0; iNode<handler.nodes.length; iNode++) {
                // used for cases like Noteworthy mail where the xpath contains few xpathes joined by ';'
                // we're alwayes looking here for the first xpath
                var xpath = handler.nodes[iNode].xpath.split('####')[0];
                var node = doc.evaluate(xpath, doc, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null)
                            .snapshotItem(0);
                if (node) {
                    this.printTrace("Got handler: " + handler['name'] + "\n");
                    return (new FoxytunesSignaturesSiteHandler(handler, doc));
                }
            }
        }
        return null;
    },

    stopMonitoring: function(handler) {
        handler.stopMonitoring();
        delete handler; 
    },

    handleAutoMode: function(buttonDetails) {
        // Handle auto-mode, only insert one time.
        // I'm doing it here because in some cases (e.g. Google quick reply) we can't insert 
        // the signature right away so, we need to do it in from the monitoring function
        var node = buttonDetails.node;
        var signatureInsertNode = buttonDetails.signatureInsertNode;

        // Signature was already added - do nothing
        if (node.getAttribute('foxytunes-signature-was-inserted')) return;

        // Node is hidden - do nothing
        if (!this.isNodeDisplayed(node)) return;

        // Some sites take time to load their iframe editor so we're checking to see
        // if the iframe is visible and has designMode set to on
        if (signatureInsertNode.localName.toLowerCase() == 'iframe') {
            var doc = signatureInsertNode.contentWindow.document;
            if (!doc.designMode || (doc.designMode != 'on')) return;
        }

        // Mark the node
        node.setAttribute('foxytunes-signature-was-inserted', 'true');

        // Don't insert signature if no music. This means that even if the music starts playing
        // we wont insert signature because the 'foxytunes-signature-was-inserted' attribute was
        // already set
        if (this.isMusicPlaying()) {
            var me = this;
            setTimeout( function() { me.addNowPlayingAudio(buttonDetails, true); }, 1000);
        }
    },

    handleNode: function(handler, buttonDetails, doc) {
        if (this.bDisableSignatures) {
            this.removeSignatureButtons(doc);
            this.stopMonitoring(handler);
            return;
        }
    
        // Stop monitoring in case the document is no longer valid (for example after surfing to another site)
        // this is doen by looking at the defaultView
        if (!doc.defaultView) {
            this.stopMonitoring(handler);
            return;
        }

        var bAutoMode = (this.bAutoMode['all'] || this.bAutoMode[buttonDetails.type]);
        if (bAutoMode) {
            this.handleAutoMode(buttonDetails, doc);
        }

        this.addSignatureButton(buttonDetails, doc);
    },

    doSiteAdjustments:function(doc, handler) {
        for (var iMod=0; iMod<handler.adjustments.length; iMod++) {
            var adjustment = handler.adjustments[iMod];
            var node = doc.evaluate(adjustment.xpath, doc, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null)
                            .snapshotItem(0);
            if (!node) return;
            var styleArray = adjustment.style.split(';');
            for (var i=0; i<styleArray.length; i++) {
               var style = styleArray[i].split(':');
               try {
                node.style[style[0]] = style[1]; 
               } catch(e) {};
            }
        }
    },

    onDOMContentLoad: function(event) {
        if (this.bDisableSignatures) return;

        if (event.originalTarget.nodeName != "#document") { return; }
        var doc = event.originalTarget || document;
        if (!doc) return;

        var handler = this.getSiteHandler(doc);
        if (handler) {
            this.doSiteAdjustments(doc, handler.getHandlerData());
            handler.startMonitoring();
        }
    },

    printTrace : function(message) {
        // dump(message + '\n');
    }
}

function FoxytunesThunderbirdSignatures() {
    var me = this;
    this.onComposeWindowOpenCallback =  function(event) { me.onComposeWindowOpen(); }
    this.firstRun = false;
    this.showBalloonMessage = false;
    this.balloonMessageID = 1;
    this.balloonMessageText = foxytunesGetLocaleString('signatunesBalloonMessage1');
}

FoxytunesThunderbirdSignatures.prototype = {
    init: function() {
        var win = this.getComposeWindow();
        if (win) {
           // Used in order to show the first-run ballon
           if (foxytunesReadSinglePreference('signature.first-run', 'true') == 'true') {
             this.firstRun = true;
           }
           // Used in order to show the balloon message
           if (foxytunesReadSinglePreference('signature.messeges.' + this.balloonMessageID, 'true') == 'true') {
               this.showBalloonMessage = true;
           }
           this.onComposeWindowOpen();
           win.addEventListener("compose-window-reopen", this.onComposeWindowOpenCallback , false);
           this.observePreferences(true);
        }
    },

    unInit: function() {
        var win = this.getComposeWindow();
        if (win) {
           win.removeEventListener("compose-window-reopen", this.onComposeWindowOpenCallback , false);   
           this.observePreferences(false);
        }
    },

    observePreferences: function(bAdd) {
        var prefBranch2 = gFoxyTunesPref.QueryInterface(Components.interfaces.nsIPrefBranch2);
        if (bAdd) {
            prefBranch2.addObserver("", this, false);  
        } else {
            prefBranch2.removeObserver("", this);  
        }
    },

    observe: function(subject, topic, data) {
        if (topic == "nsPref:changed" && data == "signatures.disable") {
            this.onComposeWindowOpen();
        }
    },

    getComposeWindow: function() {
        return document.getElementById("msgcomposeWindow");
    },

    adjustFirstRunBalloonPosition: function(x, y) {
        var balloonWin = foxytunesGetMostRecentWindow('foxytunes:balloon-help');
        balloonWin.moveBy(-250, -97);
    },

    showFirstRunBallon: function(ballonText) {
        // Don't show ballon on Linux/Mac
        if (foxytunesGetPlatform() != 'windows') return;
        var signatunesButton = document.getElementById('signatunes-main-button');
        if (!signatunesButton) return;
        var x = signatunesButton.boxObject.screenX;
        var y = signatunesButton.boxObject.screenY;

        // Don't show ballon if signatunes button is off-screen
        if (x > (window.outerWidth + window.screenX)) return;

        foxytunesShowBalloonHelpWithTip(x, y, ballonText, -1);
        var me = this;
        setTimeout(function() { me.adjustFirstRunBalloonPosition(x,y); }, 0);
    },

    onComposeWindowOpen: function() {
        var bDisableSignatures = (foxytunesReadSinglePreference('signatures.disable', 'false') == 'true') ? true : false;
        var signatunesButton = document.getElementById('signatunes-main-button');
        if (bDisableSignatures) {
            signatunesButton.hidden = true;
            return;
        } else {
            signatunesButton.hidden = false;
        }

        if (this.firstRun) {
            foxytunesWriteSinglePreference('signature.first-run', 'false');
            this.firstRun = false;

            // Don't show balloon message if it's first time installation
            foxytunesWriteSinglePreference('signature.messeges.' + this.balloonMessageID, 'false');
            this.showBalloonMessage = false;

            var me = this;
            setTimeout (function() { me.showFirstRunBallon(foxytunesGetLocaleString('signatunesFirstRunBalloonText')); }, 0);
        }

        if (this.showBalloonMessage) {
            foxytunesWriteSinglePreference('signature.messeges.' + this.balloonMessageID, 'false');
            this.showBalloonMessage = false;

            var me = this;
            setTimeout (function() { me.showFirstRunBallon(me.balloonMessageText); }, 0);
        }

        var bAutoMode = (foxytunesReadSinglePreference('signatures.automode.mail', 'false') == 'true') ? true : false;
        if (bAutoMode) {
            if (gFoxytunesSignatures.isMusicPlaying()) {
                var me = this;
                setTimeout(function() { me.addNowPlayingAudio(null, true); }, 0);
            }
        }
    },

    // buttonDetails is passed only to keep the same interface as addNowPlayingAudio from FoxytunesSignatures
    addNowPlayingAudio:function(buttonDetails, bIsAutoMode, customTrackData) {
        var signature = gFoxytunesSignatures.getNowPlayingAudio(bIsAutoMode, 'mail', 'full-html', customTrackData);
        var editor = gMsgCompose.editor;
        var html_editor = editor.QueryInterface(Components.interfaces.nsIHTMLEditor);
        html_editor.insertHTML(signature); 
    },

    onClick: function(event) {
        var signatureButton = document.getElementById('signatunes-main-button');
        if (signatureButton.disabled) return;
        if ((event.clientX - signatureButton.boxObject.x) > 18) {
            this.showPopup();
        } else {
            if (gFoxytunesSignatures.isMusicPlaying()) {
                this.addNowPlayingAudio(null, false);
            } else {
                gFoxytunesSignatures.showNoMusicAlert();
            }
        }
    },

    showPopup: function() {
        var popup = document.getElementById('foxytunes-signature-menu');
        popup.showPopup(
           document.getElementById('signatunes-main-button'),   
           -1,
           -1,
           "popup",                  
           "bottomleft",                         
           "topleft");      
    }
}
