/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function FoxyTunesTooltipInfo() {
    this.text = null;
    this.learnMoreLink = null;
    this.position = 'top-right';
    this.showOnce = false;
    this.persistOnRefersh = false;
    this.topOffset = 0;
    this.leftOffset = 0;
    this.bUseAbsolutePosition = false;    
    this.id = 'foxytunes-html-tooltip';
}

function FoxyTunesHTMLTooltip() {
    this.tooltipWidth = 283;
    this.tooltipHeight = 105;
    this.fontSize = 13;
    this.tipOffset = 21;
}

FoxyTunesHTMLTooltip.prototype = {
    closeTooltip: function(doc, tooltipInfo) {
        var tooltipId = tooltipInfo.id;
        if (tooltipInfo.showOnce) {
            foxytunesWriteSinglePreference('html-tooltip.' + tooltipId + '.displayed', 'true');
        }

        var tooltip = doc.getElementById(tooltipId);
        if (!tooltip) return;

        tooltip.parentNode.removeChild(tooltip);
    },

    getNodePosition: function(node, containingElem) {
    	var curleft = curtop = 0;
    	if (node.offsetParent) {
    		curleft = node.offsetLeft;
	    	curtop = node.offsetTop;
    		while ((node = node.offsetParent) && (node != containingElem)) {
    			curleft += node.offsetLeft;
	    		curtop += node.offsetTop;
    		}
	    }
    	return [curleft,curtop];
    },

    showTooltip: function(doc, elem, tooltipInfo) {
        if (tooltipInfo.showOnce && 
            (foxytunesReadSinglePreference('html-tooltip.' + tooltipInfo.id + '.displayed', 'false') == 'true')) {
            return;
        }

        // don't show ballon twice
        if (doc.getElementById(tooltipInfo.id)) return;

        var elemPos = this.getNodePosition(elem, doc.documentElement);
        var elemWidth = elem.offsetWidth;
        var elementHeight = elem.offsetHeight;

        var tooltip = doc.createElement('div');
        tooltip.setAttribute('id', tooltipInfo.id);
        var top;
        var left;
        switch (tooltipInfo.position) {
            case 'top-right': 
                tooltip.style.backgroundImage = "url(chrome://foxytunes-public/content/html-tooltip/html-tooltip-top-right.png)"
                top = elemPos[1] - this.tooltipHeight;
                left = elemPos[0] + elemWidth - this.tooltipWidth + this.tipOffset;
                break;
            case 'top-left':
                tooltip.style.backgroundImage = "url(chrome://foxytunes-public/content/html-tooltip/html-tooltip-top-left.png)"
                top = elemPos[1] - this.tooltipHeight;
                left = elemPos[0] - this.tipOffset;
                break;
            case 'bottom-left':
                tooltip.style.backgroundImage = "url(chrome://foxytunes-public/content/html-tooltip/html-tooltip-bottom-left.png)"
                top = elemPos[1] + elementHeight;
                left = elemPos[0] - this.tipOffset;
                break;
            case 'bottom-right':
                tooltip.style.backgroundImage = "url(chrome://foxytunes-public/content/html-tooltip/html-tooltip-bottom-right.png)"
                top = elemPos[1] + elementHeight;
                left = elemPos[0] + elemWidth - this.tooltipWidth + this.tipOffset;
                break;
        }
        left = left + tooltipInfo.leftOffset;
        top = top + tooltipInfo.topOffset;

        tooltip.style.display  = 'block';
        tooltip.style.visibility  = 'visible';
        tooltip.style.position = tooltipInfo.bUseAbsolutePosition ? 'absolute' : 'fixed';
        tooltip.style.zIndex = 1000;
        tooltip.style.width = this.tooltipWidth + 'px';
        tooltip.style.height = this.tooltipHeight + 'px';    
        tooltip.style.fontSize = this.fontSize + 'px';    
        
        top = (top < 0) ? 0 : top;
        left = (left < 0) ? 0 : left;

        tooltip.style.top = top + 'px';
        tooltip.style.left = left + 'px'; 

        var tdStyle = 'color:black; background-color: transparent;font-size: 12px;font-family: arial;';
        if (tooltipInfo.position.indexOf('top') != -1) {
            tdStyle += 'padding-left: 40px; padding-right: 25px;';
        } else {
            tdStyle += 'padding-top: 30px; padding-left: 45px; padding-right: 25px;';
        }

        var learnMore = foxytunesGetLocaleStringExternalfunction('learnMore');
        var innerHTML = '<table style="color:black; background-color: transparent; width: 283px; height: 80px;"><tr style="background-color: transparent;">' +
                        '<td valign="middle" style="' + tdStyle + '">' + tooltipInfo.text;
        if (tooltipInfo.learnMoreLink) {
            innerHTML += ' <a style="color:blue" target="_blank" href="' + tooltipInfo.learnMoreLink + '">' + learnMore + '</a>';
        }
        innerHTML += '</td></tr></table>';
        tooltip.innerHTML = innerHTML; 

        if (tooltipInfo.bUseAbsolutePosition) {
            doc.documentElement.appendChild(tooltip);
        } else {
        elem.parentNode.insertBefore(tooltip, elem);
        }

        var me = this;
        tooltip.addEventListener('click', function() { me.closeTooltip(doc, tooltipInfo); }, false);

        if (tooltipInfo.showOnce && !tooltipInfo.persistOnRefersh) {
            foxytunesWriteSinglePreference('html-tooltip.' + tooltipInfo.id + '.displayed', 'true');
        }
    }
}
