/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function OAuthUtils(apiKey, serviceName) {
   
    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
        scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
        scriptLoader.loadSubScript("chrome://foxytunes/content/utils/oauth/sha1.js");
        scriptLoader.loadSubScript("chrome://foxytunes/content/utils/oauth/oauth.js");
    }

    this._authDetails = {
        "apiKey": apiKey,
        "serviceName": serviceName,
        "signatureMethod": "HMAC-SHA1",
        "token": null,
        "tokenSecret": null,
        "verifier": null,
        "accessToken": null,
        "accessTokenSecret": null,
        "sessionHandle": null
    };

    this._baseURL = null;
    this._urlEndPoints = {'request_token': '', 'access_token': '', 'auth_url': ''};
}

OAuthUtils.prototype = {
    setBaseURL: function(baseURL) {
        this._baseURL = baseURL;
    },

    setUrlEndPoints: function(endPoints) {
        for (var type in endPoints) {
           this._urlEndPoints[type] = endPoints[type];
        } 
    },

    setAuthDetails: function(detailsObj) {
        for (var i in this._authDetails) {
            if (typeof detailsObj[i] != 'undefined') {
                this._authDetails[i] = detailsObj[i];
            }
        }
    },

    getAuthDetails: function(params) {
        var authDetails = {};
        for (var i=0; i<params.length; i++) {
            if (typeof this._authDetails[params[i]] != 'undefined') {
                authDetails[params[i]] = this._authDetails[params[i]];
            }
        }
        return authDetails;
    },

    getRequestToken: function(callback) {
        var accessor = { service: this._authDetails["serviceName"] };
        var message  = { action: this._baseURL +  this._urlEndPoints['request_token'],
                         method: "GET",
                         parameters: []};
        message.parameters.push(["oauth_consumer_key", this._authDetails["apiKey"]]);
        message.parameters.push(["oauth_signature_method", this._authDetails["signatureMethod"]]);
        message.parameters.push(["oauth_callback", "oob"]);        

        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);

        var url = OAuth.addToURL(message.action, message.parameters);
        var me = this;
        var oauthCB = function(xmlHTTPrequest) {
            var resp = (xmlHTTPrequest.status != 200) ? null : me.tokenize(xmlHTTPrequest.responseText);
            if (callback) callback(resp);
        }
        foxytunesHTTPRequest({'url': url}, oauthCB);
    },

    getUserAuthURL: function() {
        var accessor = { service: this._authDetails["serviceName"] };
        var message = { action: this._baseURL + this._urlEndPoints['auth_url'],
                        method: "GET",
                        parameters: []};
        message.parameters.push(["oauth_token", this._authDetails["token"]]);
        return OAuth.addToURL(message.action, message.parameters);
    },

    getAccessToken: function(callback, bRefreshToken) {
        var accessor = { tokenSecret: (bRefreshToken) ? this._authDetails["accessTokenSecret"] : this._authDetails["tokenSecret"], 
                         service: this._authDetails["serviceName"] };
        var message = { action: this._baseURL + this._urlEndPoints['access_token'],
                        method: "GET",
                        parameters: []};
        message.parameters.push(["oauth_consumer_key", this._authDetails["apiKey"]]);
        message.parameters.push(["oauth_token", (bRefreshToken) ? this._authDetails["accessToken"] : this._authDetails["token"]]);
        message.parameters.push(["oauth_signature_method", this._authDetails["signatureMethod"]]);
        message.parameters.push(["oauth_verifier", this._authDetails["verifier"]]);
        if (bRefreshToken) {
            message.parameters.push(["oauth_session_handle", this._authDetails["sessionHandle"]]);
        }
                
        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);
        var url = OAuth.addToURL(message.action, message.parameters);
        var me = this;
        var oauthCB = function(xmlHTTPrequest) {
            var resp = (xmlHTTPrequest.status != 200) ? null : me.tokenize(xmlHTTPrequest.responseText);
            if (callback) callback(resp);
        }
        foxytunesHTTPRequest({'url': url}, oauthCB);
    },

    getXAuthAccessToken: function(params, callback) {
        var accessor = { service: this._authDetails["serviceName"] };
        var message = { action: this._baseURL + this._urlEndPoints['access_token'],
                        method: "GET",
                        parameters: []};
        message.parameters.push(["oauth_consumer_key", this._authDetails["apiKey"]]);
        message.parameters.push(["oauth_signature_method", this._authDetails["signatureMethod"]]);
        message.parameters.push(["x_auth_username", params["username"]]);
        message.parameters.push(["x_auth_password", params["password"]]);
        message.parameters.push(["x_auth_mode", "client_auth"]);
                
        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);
        var url = OAuth.addToURL(message.action, message.parameters);
        var me = this;
        var oauthCB = function(xmlHTTPrequest) {
            var resp = (xmlHTTPrequest.status != 200) ? null : me.tokenize(xmlHTTPrequest.responseText);
            if (callback) callback(resp);
        }
        foxytunesHTTPRequest({'url': url}, oauthCB);
    },

    getAPIURL: function(method, url, params) {
        var accessor =  { tokenSecret: this._authDetails["accessTokenSecret"],
                          service: this._authDetails["serviceName"] };
        var message = { action: url,
                        method: method,
                        parameters: []};
        message.parameters.push(["oauth_consumer_key", this._authDetails["apiKey"]]);
        message.parameters.push(["oauth_token", this._authDetails["accessToken"]]);
        message.parameters.push(["oauth_signature_method", this._authDetails["signatureMethod"]]);
        message.parameters.push(["oauth_version", "1.0"]);
        for (var item in params) {
            message.parameters.push([item, params[item]]);
        }
        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);
        return OAuth.addToURL(message.action, message.parameters);
    },

    isOAUTHReturnedError: function(resp) {
        if (!resp) return true;
        return (resp && resp.error_code);
    },

    // private
    tokenize: function(uriString) {
        var tokVals = uriString.split('&');
        var oauthInfo = {};
        for (var i=0; i<tokVals.length; i++) {
            var token=tokVals[i].split('=',2);
            oauthInfo[token[0]]=decodeURIComponent(token[1]);
        }
        return oauthInfo;
    }
}
