//   
//  Copyright (c) 2009 Yahoo! Inc. 
//  All Rights Reserved
//
// XUL Tooltip library
//
// parameters on calling window:
// ftaddclosebutton - add close button to popup
// ftaddtooltip - add tip to popup
// fttooltiplocation - the tip location
// fttooltipautolocation - always place the tip inside the screen
// ftshowonce - show the tooltip only once (uses the window id to determain if the tooltip has been showed before)
// ftadddontshowagain - add don't show this again checkbox (uses the window id to determain if the checkbox was turned on)
// 
// Passed parameters:
// elem or bounding box - elemen or bounding box of the element that the tooltip is connected to

function FoxyTunesXulTooltip()  {
    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
    }

    try {
        this._windowController = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesWindowController;1"]
                                     .getService(Components.interfaces.IFoxyTunesWindowController);
    } catch(e) {}

    this._win = null;
    this._mainBox = null;
    this._mainStack = null;
    this._tooltipImage = null;

    this._elemBoundingBox = null;
    this._mainBoxBorderWidth = 1;

    // Setting real width and height in addTooltip according to css
    this._tipWidth = 0;
    this._tipHeight = 0;

    this._tipDefaultHorizontalOffset = 24;
    this._tipDefaultVerticalOffset = 10;
    this._tooltipDistanceFromElement = 5;

    this.bAddCloseButton = false;
    this.bAddTooltip = false;
    this.tooltipLocation = 'bottom-right';
    this.bToolTipAuoLocation = true;
    this.bShowOnce = false;
    this.bAddDontShowAgain = false;

    this.constructorInit();
};

FoxyTunesXulTooltip.prototype = {
    // Happens before UI loaded
    constructorInit: function() {
        this._win = document.getElementsByTagName('window')[0];
        if (!this._win) return;

        if (!window.arguments[0]) return;

        if (window.arguments[0].boxObject) {
            this._elemBoundingBox = window.arguments[0].boxObject;
        } else {
            this._elemBoundingBox = window.arguments[0];
        }

        if (this._win.getAttribute('ftaddclosebutton') == 'true') {
            this.bAddCloseButton = true;
        }

        if (this._win.getAttribute('ftaddtooltip') == 'true') {
            this.bAddTooltip = true;
        }

        if (this._win.hasAttribute('fttooltiplocation')) {
            this.tooltipLocation = this._win.getAttribute('fttooltiplocation');
        }

        if (this._win.hasAttribute('fttooltipautolocation') && 
            (this._win.getAttribute('fttooltipautolocation') != 'true')) {
            this.bToolTipAuoLocation = false;
        }

        if (this._win.getAttribute('ftadddontshowagain') == 'true' && this._win.id) {
            this.bAddDontShowAgain = true;
        }

        if (this._win.getAttribute('ftshowonce') == 'true' && this._win.id) {
            this.bShowOnce = true;
        }

        if (this.bShowOnce) {
            if (foxytunesReadSinglePreference('xul-tooltip.' + this._win.id + '.displayed', 'false') == 'true') {
                close();
                return;
            }
        }

        if (this._win.id && 
            (foxytunesReadSinglePreference('xul-tooltip.' + this._win.id + '.dont-show-again', 'false') == 'true')) {
            close();
            return;
        }
    },

    init: function() {
        if (!this._win) return;

        this._win.style.opacity = '0';

        var baseWindow = foxytunesGetBaseWindow(window);
        if (this.isOnWindows()) {
           this._windowController.RemoveWindowFromTaskBar(baseWindow);
        }

        this.reparentWindow();

        if (this.bAddCloseButton) this.addCloseButton();
        if (this.bAddDontShowAgain) this.addDontShowAgain();
        if (this.bAddTooltip) {
            this.addTooltip();
            this.getMainBoxBorderWidth();
        }

        var me = this;
        setTimeout(function() { me.adjustWindow(); }, 10);
    },

    uninit: function() {
        if (this.isOnWindows()) {
            this._windowController.ResetRemovedFromTaskBarWindow(false); 
        }
        if (this.bAddDontShowAgain) {
            var cb = document.getElementById('xul-tooltip-dontshowagain-cb');
            if (cb && this._win.id) {
               foxytunesWriteSinglePreference('xul-tooltip.' + this._win.id + '.dont-show-again', cb.checked);  
            }
        }
    },

    adjustTipAndWindowLocation: function() {
        var tipOffsetFromRight = this._tipDefaultHorizontalOffset + this._tipWidth;
        var tipOffsetFromBottom = this._tipDefaultVerticalOffset + this._tipHeight;

        window.moveTo(this._elemBoundingBox.screenX, this._elemBoundingBox.screenY);
        
        switch (this.tooltipLocation) {
            case 'bottom-right': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', window.outerWidth-tipOffsetFromRight);
                            this._tooltipImage.setAttribute('top', this._mainBox.boxObject.height-this._mainBoxBorderWidth);
                            this._mainBox.setAttribute('top', '0');
                            sizeToContent();
                        }
                        window.moveBy(tipOffsetFromRight-window.outerWidth-this._tipWidth+(this._elemBoundingBox.width/2), 0-window.outerHeight-this._tooltipDistanceFromElement);
                        break;
            case 'top-right': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', window.outerWidth-tipOffsetFromRight);
                            this._tooltipImage.setAttribute('top', '0');
                            this._mainBox.setAttribute('top', this._tipHeight-this._mainBoxBorderWidth);
                            sizeToContent();
                        }
                        window.moveBy(tipOffsetFromRight-window.outerWidth-this._tipWidth+(this._elemBoundingBox.width/2), this._elemBoundingBox.height+this._tooltipDistanceFromElement);
                        break;
            case 'bottom-left': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', this._tipDefaultHorizontalOffset);
                            this._tooltipImage.setAttribute('top', this._mainBox.boxObject.height-this._mainBoxBorderWidth);
                            this._mainBox.setAttribute('top', '0');
                            sizeToContent();
                        }
                        window.moveBy(0-this._tipDefaultHorizontalOffset+(this._elemBoundingBox.width/2), 0-window.outerHeight-this._tooltipDistanceFromElement);
                        break;
            case 'top-left': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', this._tipDefaultHorizontalOffset);
                            this._tooltipImage.setAttribute('top', '0');
                            this._mainBox.setAttribute('top', this._tipHeight-this._mainBoxBorderWidth);
                            sizeToContent();
                            window.moveBy(0-this._tipDefaultHorizontalOffset+(this._elemBoundingBox.width/2), this._elemBoundingBox.height+this._tooltipDistanceFromElement);
                        }
                        break;
            case 'right-top': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', this._mainBox.boxObject.width-this._mainBoxBorderWidth);
                            this._tooltipImage.setAttribute('top', this._tipDefaultVerticalOffset);
                            this._mainBox.setAttribute('left', '0');
                            sizeToContent();
                        }
                        window.moveBy(0-window.outerWidth-this._tooltipDistanceFromElement, 0-this._tipDefaultVerticalOffset+(this._elemBoundingBox.height/2));
                        break;
            case 'right-bottom': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', this._mainBox.boxObject.width-this._mainBoxBorderWidth);
                            this._tooltipImage.setAttribute('top', this._mainBox.boxObject.height-tipOffsetFromBottom);
                            this._mainBox.setAttribute('left', '0');
                            sizeToContent();
                        }
                        window.moveBy(0-window.outerWidth-this._tooltipDistanceFromElement, this._tipDefaultVerticalOffset-window.outerHeight+(this._elemBoundingBox.height/2));
                        break;
            case 'left-top': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', '0');
                            this._tooltipImage.setAttribute('top', this._tipDefaultVerticalOffset);
                            this._mainBox.setAttribute('left', this._tipHeight-this._mainBoxBorderWidth);
                            sizeToContent();
                        }
                        window.moveBy(this._elemBoundingBox.width+this._tooltipDistanceFromElement, 0-this._tipDefaultVerticalOffset+(this._elemBoundingBox.height/2));
                        break;
            case 'left-bottom': 
                        if (this._tooltipImage) {
                            this._tooltipImage.setAttribute('left', '0');
                            this._tooltipImage.setAttribute('top', this._mainBox.boxObject.height-tipOffsetFromBottom);
                            this._mainBox.setAttribute('left', this._tipHeight-this._mainBoxBorderWidth);
                            sizeToContent();
                        }
                        window.moveBy(this._elemBoundingBox.width+this._tooltipDistanceFromElement, this._tipDefaultVerticalOffset-window.outerHeight+(this._elemBoundingBox.height/2));
                        break;
        }
    },

    autoFixTooltipLocation: function() {
        var position = new Object();
        position.top = window.screenY;
        position.bottom = window.screenY + window.outerHeight;
        position.left = window.screenX;
        position.right = window.screenX + window.outerWidth;

        // Tooltip is outside the left-top side of the screen
        if ((position.left < 0) && (position.top < 0)) {
            switch (this.tooltipLocation) {
                case 'bottom-right': 
                        this.tooltipLocation = 'top-left';
                        break;
                case 'right-top': 
                        this.tooltipLocation = 'left-top';
                        break;
            }
        }

        // Tooltip is outside the left-bottom side of screen
        if ((window.left < 0) && (position.bottom > window.screen.availHeight)) {
            switch (this.tooltipLocation) {
                case 'top-right': 
                        this.tooltipLocation = 'bottom-left';
                        break;
                case 'right-top':  
                        this.tooltipLocation = 'left-bottom';
                        break;
            }
        }

        // Tooltip is outside the left side of the screen
        if ((position.left < 0) && (position.top > 0) && (position.bottom < window.screen.availHeight)) {
            switch (this.tooltipLocation) {
                case 'top-right': 
                        this.tooltipLocation = 'top-left';
                        break;
                case 'bottom-right': 
                        this.tooltipLocation = 'bottom-left';
                        break;
                case 'right-top': 
                        this.tooltipLocation = 'left-top';
                        break;
                case 'right-bottom': 
                        this.tooltipLocation = 'left-bottom';
                        break;
            }        
        }

        // Tooltip is outside the bottom side of the screen
        if ((position.bottom > window.screen.availHeight) && (position.left > 0) && (position.right < window.screen.availWidth)) {
            switch (this.tooltipLocation) {
                case 'left-top': 
                        this.tooltipLocation = 'left-bottom';
                        break;
                case 'top-left': 
                        this.tooltipLocation = 'bottom-left';
                        break;
                case 'top-right': 
                        this.tooltipLocation = 'bottom-right';
                        break;
                case 'right-top': 
                        this.tooltipLocation = 'right-bottom';
                        break;
            }
        }

        // Tooltip is outside the right-bottom side of the screen
        if ((position.right > window.screen.availWidth) && (position.bottom > window.screen.availHeight)) {
            switch (this.tooltipLocation) {
                case 'top-left': 
                        this.tooltipLocation = 'bottom-right';
                        break;
                case 'left-top': 
                        this.tooltipLocation = 'right-bottom';
                        break;
            }
        }

        // Tooltip is outside the right side of the screen
        if ((position.right > window.screen.availWidth) && (position.top > 0) && (position.bottom < window.screen.availHeight)) {
            switch (this.tooltipLocation) {
                case 'top-left': 
                        this.tooltipLocation = 'top-right';
                        break;
                case 'left-top': 
                        this.tooltipLocation = 'right-top';
                        break;
                case 'left-bottom': 
                        this.tooltipLocation = 'right-bottom';
                        break;
                case 'bottom-left': 
                        this.tooltipLocation = 'bottom-right';
                        break;
            }
        }

        // Tooltip is outside the right-top side of the screen
        if ((position.right > window.screen.availWidth) && (position.top < 0)) {
           switch (this.tooltipLocation) { 
               case 'bottom-left': 
                    this.tooltipLocation = 'top-right';
                    break;
               case 'left-bottom': 
                    this.tooltipLocation = 'right-top'
                    break;
           }
        }

        // Tooltip is outside the top side of the screem
        if ((position.top < 0) && (position.left > 0) && (position.right < window.screen.availWidth)) {
           switch (this.tooltipLocation) { 
               case 'left-bottom': 
                    this.tooltipLocation = 'left-top';
                    break;
               case 'bottom-left': 
                    this.tooltipLocation = 'top-left';
                    break;
               case 'bottom-right': 
                    this.tooltipLocation = 'top-right';
                    break;
               case 'right-bottom': 
                    this.tooltipLocation = 'right-top';
                    break;
           }
        }

        if (this.bAddTooltip) {
            this.setTooltipImageID();
            this.getMainBoxBorderWidth();
        }
        this.adjustTipAndWindowLocation();
    },

    adjustWindow: function() {
      sizeToContent();

      this.adjustTipAndWindowLocation();
      if (this.bToolTipAuoLocation) this.autoFixTooltipLocation();

      this._win.style.opacity = '1';

      var me = this;
      // Close tooltip if opener window got focus (usually click, resize, moveof Firefox window)
      if (window.opener) window.opener.addEventListener("focus", function() { me.closeTooltip(); }, true);
    },

    reparentWindow: function() {
        if (this.bAddTooltip) {
            this._mainStack = document.createElement('stack');
            this.addElemParent('xul-tooltip-content', this._mainStack);
        }

        this._mainBox = document.createElement('hbox');
        this._mainBox.setAttribute('id', 'xul-tooltip-main-box');
        this.addElemParent('xul-tooltip-content', this._mainBox);

        if (this.bAddDontShowAgain) {
            var vbox = document.createElement('vbox');
            this.addElemParent('xul-tooltip-content', vbox);
        }
    },

    addElemParent: function(id, newParent) {
        var elem = document.getElementById(id);
        if (!elem) return;

        var origParent = elem.parentNode;
        origParent.removeChild(elem);
        newParent.appendChild(elem);
        origParent.appendChild(newParent);
    },

    addCloseButton: function() {
        var mainContent = document.getElementById('xul-tooltip-content');
        if (mainContent) {
            mainContent.style.paddingTop = '3px';
        }

        var closeBox = document.createElement('vbox');
        var closeImage = document.createElement('image');

        var me = this;
        closeImage.setAttribute('id', 'xul-tooltip-close');
        closeImage.addEventListener('click' ,function() { me.closeTooltip(); }, false);
        
        closeBox.appendChild(closeImage);

        this._mainBox.appendChild(closeBox);
    },

    addDontShowAgain: function() {
         var mainContent = document.getElementById('xul-tooltip-content');
         if (!mainContent) return;

         var hbox = document.createElement('hbox');
         var cb =  document.createElement('checkbox');
         hbox.setAttribute('flex', '0');
         cb.setAttribute('id', "xul-tooltip-dontshowagain-cb");
         cb.setAttribute('label', foxytunesGetLocaleStringExternalfunction('xulTooltipsDontShowThisAgain'));
         var me = this;
         hbox.appendChild(cb);

         mainContent.parentNode.appendChild(hbox);
    },

    setTooltipImageID: function() {
        var tooltipImageID = 'tip-' + this.tooltipLocation;
        this._tooltipImage.setAttribute('id', tooltipImageID);
    },

    addTooltip: function() {
        // Sanity check
        if (!this._mainStack) return;
        
        this._tooltipImage = document.createElement('image');
        this._tooltipImage.setAttribute('class', 'tip');
        this.setTooltipImageID();
        var style = this._tooltipImage.ownerDocument.defaultView.getComputedStyle(this._tooltipImage, null);
        var width = style.getPropertyValue("width");
        var height = style.getPropertyValue("height");
        this._tipWidth = parseInt(width.substr(0, width.length - 2));
        this._tipHeight = parseInt(height.substr(0, height.length - 2));

        // Top and left are adjusted on timeout in adjustWindow()
        this._tooltipImage.setAttribute('top', '0');
        this._tooltipImage.setAttribute('left', '0');
        this._mainBox.setAttribute('top', '0');

        this._mainStack.appendChild(this._tooltipImage);
    },

    getMainBoxBorderWidth: function() {
        var style = this._mainBox.ownerDocument.defaultView.getComputedStyle(this._mainBox, null);
        var cssBorderWidth = 'border-bottom-width';
        switch (this.tooltipLocation) { 
            case 'bottom-left':
            case 'bottom-right':
                        cssBorderWidth = 'border-bottom-width';
                        break;
            case 'top-left':
            case 'top-right':
                        cssBorderWidth = 'border-top-width';
                        break;
            case 'left-bottom':
            case 'left-top':
                        cssBorderWidth = 'border-left-width';
                        break;
            case 'right-top':
            case 'right-bottom':
                        cssBorderWidth = 'border-right-width';
                        break;

        } 
        var borderWidth = style.getPropertyValue(cssBorderWidth);
        this._mainBoxBorderWidth = borderWidth.substr(0, borderWidth.length - 2);
    },

    closeTooltip: function() {
        if (this.bShowOnce) {
            foxytunesWriteSinglePreference('xul-tooltip.' + this._win.id + '.displayed', 'true');
        }
        close();
    },

    isOnWindows : function() {
        return (foxytunesGetPlatform() == 'windows');
    }
};

var gFoxyTunesXulTooltip = new FoxyTunesXulTooltip();
